unit Mp3TrackData;

interface

uses classes;

type
  TTagObject = class;
  PTagObject = ^TTagObject;

  TStrObj = class
    str : string;
  end;

  TStrEvent = procedure (str: string) of object;
  TTagEvent = procedure(tag: TTagObject) of object;

  TColumnName =
    (cnName, cnPath, cnDuration, cnCount, cnSize, cnPlayed, cnPlaycnt,
    cnTagged, cnYear, cnArtist, cnAlbum, cnTitle, cnComment, cnGenre,
    cnTrack, cnBitrate, cnBpm, cnMode, cnStyle);

  TSumObject = class(TObject)
    name: string;
    path: string;
    duration: integer;
    count: integer;
    childs: TStringList;
    size: int64;
    played: integer;
    playcnt: integer;
    parent: TSumObject;
    constructor Create(aname: string; dirlist: TStringList);
    destructor Destroy; override;
    procedure addTag(obj: TTagObject);
    procedure addSum(obj: TSumObject);
    procedure addParent(obj: TSumObject);
    procedure save(var fs: textfile);

    procedure Sort;

    procedure saveLocal;
    procedure loadLocal(filelist: TStringList);

    procedure saveUserLocal(username: string);
    procedure loadUserLocal(username: string);
    procedure loadUserInfoTxt;

    procedure load(var fs: textfile; dirlist, filelist: TStringList);

    function getField(col: TColumnName): string;
    function getFieldVal(col: TColumnName): string;
    procedure updatePlaycnt;
    procedure sum;
    function FindSum(aname:string):TSumObject;
  end;
  PSumObject = ^TSumObject;

  TTagObject = class(TObject)
    parents: TList;
    filename: string;
    path: string;
    isFound: boolean;

// userdata
    playcnt:  integer;
    merk:     integer;
    favorite: boolean;

// special data
    style:    string;
    FileLength: LongInt;          { File length }
    Duration: word;               { Song duration }

// id3v1 tagdata
    istagged: boolean;
    year: string;
    artist: string;
    album: string;
    title: string;
    Comment: string;              { Comment }
    Genre: Byte;                  { Genre code }

// mp3 info from frame
    Track: byte;                  { Track number on Album }
    Version: byte;                { MPEG audio version index
                                     (1 - Version 1,
                                      2 - Version 2,  3 - Version 2.5,
                                      0 - unknown }
    Layer: byte;                  { Layer (1, 2, 3, 0 - unknown) }
    SampleRate: LongInt;          { Sampling rate in Hz}
    BitRate: LongInt;             { Bit Rate }
    BPM: word;                    { bits per minute - for future use }
    Mode: byte;                   { Number of channels (0 - Stereo,
                                     1 - Joint-Stereo, 2 - Dual-channel,
                                     3 - Single-Channel) }

    constructor Create(aname: string; apath: string; scantags: boolean; filelist:TstringList);
    destructor Destroy; override;
    procedure ReadId3Tags;
    procedure ReadMpgTags;
    function fullName: string;

    procedure save(var fs: textfile; local:boolean);
    procedure saveParent;
    procedure load(line: string; apath: string; filelist:TStringList);

    procedure saveuserdata(var fs: textfile);
    procedure loaduserdata(line: string);

    function getField(col: TColumnName): string;
    function getFieldVal(col: TColumnName): string;
    procedure addParent(parent: TSumObject);
    procedure incrementPlaycnt(username:string);
  end;

  TMp3ScanThread = class(TThread)
    files: TStringList;
    dirs: TStringList;
    basesum: TSumObject;
  private
    FOnProgress: TStrEvent;
    FOnRefreshTree: TNotifyEvent;
    progress: string; // used by progress reporting
    scanDirs: TStringList;

    procedure ScanProgress;
  protected
    procedure ReportProgress(str: string);
    procedure RefreshTree;

    procedure Execute; override;
    procedure scanDir(parentobj: TSumObject; path : string);
    procedure scanTags;
  public
    DoScanDir: Boolean;
    DoScanTags: Boolean;
    UseMpg: boolean;
    FUserName: string;
    property OnProgress: TStrEvent read FOnProgress write FOnProgress;
    property OnRefreshTree: TNotifyEvent read FOnRefreshTree write FOnRefreshTree;
  end;

  TMp3Data = class
    files:   TStringList;  // list of all tags, string is name
    dirs:    TStringList;  // list of all sum, string is name
    styles:  TStringList;
    basesum: TSumObject;   // basesumobject
    bases:   TList;
  private
    scandirs: TStringList;
    FOnProgress: TStrEvent;
    scanner: TMp3ScanThread;
    scannerActive: Boolean;
    FOnRefreshTree: TNotifyEvent;
    FUserName: string;
    procedure ClearData;
    procedure OnTerminate(obj: TObject);
    procedure DelBase(index: integer);
  protected
    procedure StartScanner(dir,tags,mpg : boolean);
  public
    constructor Create;
    destructor  Destroy; override;

    procedure Save(filename: string);
    procedure Load(filename: string);
    procedure update_playcount(afilename: string);
    function  GetTagObject(afilename: string): TTagObject;

    procedure ScanDir(dirs,tags,mpg: boolean);
    procedure ScanTags;
    procedure setDirs(theDirs: TStringList);
    procedure RefreshTree(sender: TObject);
    procedure Artists(list: TStringList);
    procedure ArtistsNoStyle(list: TStringList);
    procedure SumGenre;
    procedure SumArtist;
    procedure SumStyle;
  public
    property OnProgress: TStrEvent read FOnProgress write FOnProgress;
    property OnRefreshTree: TNotifyEvent read FOnRefreshTree write FOnRefreshTree;
    property UserName: string read FUserName write FUserName;
  end;

// This struct and the function readid3tag is copied from
// BUid3tags

  TId3v1Tag = packed record
     Identifier : array [0..2] of char;
     SongName   : array [1..30] of char;
     Artist     : array [1..30] of char;
     Album      : array [1..30] of char;
     Year       : array [0..3] of char;
     Comment    : array [1..30] of char;
     Genre      : byte;
  end;

  TArtist2Artist = class
  public
    constructor Create;
    destructor Destroy; override;
    function artist(str: string): string;
    function style(str: string):string;
  private
    a2a : TStringList;
    a2s : TStringList;
  end;

implementation

uses mp3names, uDbg, sysutils, jbstr, mpgtools;

function ctrlchar(str: string):string;
var i : integer;
begin
  if (pos('|',str)= 0) and
     (pos('!',str)= 0)
  then
    result := str
  else
  begin
    for i := 1 to length(str) do
    begin
      if str[i]='|' then
        result := result + '!1'
      else if str[i] = '!' then
        result := result + '!2'
      else
        result := result + str[i];
    end;
  end;
end;

function charctrl(str: string):string;
var i : integer; found: boolean;
begin
  if pos('!',str)= 0
  then
    result := str
  else
  begin
    found := false;
    for i := 1 to length(str) do
    begin
      if found then
      begin
        if str[i] = '1' then
          result := result + '|'
        else if str[i] = '2' then
          result := result + '!';
        found := false;
      end
      else
      begin
        if str[i]='!' then
          found := true
        else
          result := result + str[i];
      end;
    end;
  end;
end;

  function next_token(var p : integer; var l : integer; line: string) : string;
  var q: integer;
  begin
    q := p + 1;
    while (q <= l) and (not (line[q] = '|')) do
      inc(q);
    if q > (l + 1) then q := l;
    result := copy(line, p + 1, q - p - 1);
    p := q;
  end;

  function strtobool(str: string): boolean;
  begin
    result := (str = 'TRUE');
  end;

procedure parseStyle(str: string; list: TStringList);
var i,j,l: integer;
 style: string;
begin
i := 1;
j := 1;
l := length(str);
if l > 0 then
begin
while j < l do
begin
  if str[j] = ',' then
  begin
    style := lowercase(trim(copy(str,i,j-i )));
    list.add(style);
    i := j+1;
  end;
  inc(j);
end;
 style := lowercase(trim(copy(str,i,j-i+1)));
    list.add(style);
    end;
end;

  constructor TArtist2Artist.create;
  var
    i : integer;
    p : integer;
    line,alias,name,artist,style : string;
    fs: TextFile;
    obj: TStrObj;
  begin
    a2a := TStringList.Create;
    a2s := TStringList.Create;
    if fileexists('artist.txt') then
    begin
    assignfile(fs,'artist.txt');
    reset(fs);
    while not eof(fs) do
    begin
      readln(fs,line);
      p := pos('|',line);
      if p>0 then
      begin
        name := lowercase(copy(line,p+1,length(line)));
        alias := copy(line,1,p-1);
        if a2a.IndexOf(alias)< 0 then
        begin
          obj := TStrObj.create;
          obj.str := name;
          a2a.AddObject(alias,obj);
        end;
      end;
    end;
    end;
    if fileexists('styles.txt') then
    begin
    assignfile(fs,'styles.txt');
    reset(fs);
    while not eof(fs) do
    begin
      readln(fs,line);
      p := pos('|',line);
      if p>0 then
      begin
        style := lowercase(copy(line,p+1,length(line)));
        artist := copy(line,1,p-1);
        if a2s.IndexOf(artist)< 0 then
        begin
          obj := TStrObj.create;
          obj.str := style;
          a2s.AddObject(artist,obj);
        end;
      end;
    end;
    end;
  end;
  destructor TArtist2Artist.Destroy;
  var i : integer;
  begin
    for i := 0 to a2a.count-1 do
      TStrObj(a2a.objects[i]).free;
    for i := 0 to a2s.count-1 do
      TStrObj(a2s.objects[i]).free;
    a2a.free;
    a2s.free;
  end;
  function TArtist2Artist.artist(str: string): string;
  var
    lartist : string;
    i : integer;
  begin
    lartist := lowercase(str);
    i := a2a.IndexOf(lartist);
    if i >= 0 then
      lartist := TStrObj(a2a.objects[i]).str;
    result := lartist;
  end;

  function TArtist2Artist.style(str: string):string;
  var
   lartist : string;
   i : integer;
  begin
    result := '';
    lartist := artist(str);
    i := a2s.IndexOf(lartist);
    if i >= 0 then
      result := TStrObj(a2s.objects[i]).str;
  end;

constructor TSumObject.Create(aname: string; dirlist: TStringList);
begin
  inherited Create;
  name := aname;
  childs := TStringList.Create;
  parent := nil;
  dirlist.addobject(Name, self);
end;

destructor TSumObject.Destroy;
begin
//  debugger.logmsg('destroy: ' + name);
  childs.free;
  inherited Destroy;
end;

procedure TSumObject.addTag(obj: TTagObject);
begin
  childs.addobject(obj.filename, obj);
  obj.addParent(self);
end;

procedure TSumObject.addSum(obj: TSumObject);
begin
  childs.addObject(obj.name, obj);
  obj.addParent(self);
end;

function TSumObject.FindSum(aname:string):TSumObject;
var i : integer;
begin
  result := nil;
  for i := 0 to childs.count-1 do
   if childs.objects[i] is TSumObject then
     if TSumObject(childs.objects[i]).name = aname then
       result := TSumObject(childs.objects[i]);
end;

procedure TSumObject.addParent(obj: TSumObject);
begin
  if assigned(parent) then
    debugger.logmsg('add parent to sum with parent')
  else
    parent := obj;
end;

procedure TSumObject.save(var fs: textfile);
var i : integer;
begin
  writeln(fs, name, '|', path, '|', duration, '|', count, '|', size,'|',childs.count);
  for i := 0 to childs.count-1 do
  begin
    if childs.objects[i] is TSumObject then
    begin
      writeln(fs,'S');
      TSumObject(childs.objects[i]).save(fs);
    end
    else if childs.objects[i] is TTagObject then
    begin
      writeln(fs,'T');
      TTagObject(childs.objects[i]).save(fs,false);
    end;
  end;
end;

procedure TSumObject.Sort;
begin
  childs.Sort;
end;

procedure TSumObject.saveLocal;
var
  i : integer;
  mp3s_present: boolean;
  fs : textfile;
begin
  // save the mp3 content of a directory in the directory itself (with the mp3's)
  // the data is the mp3 info, it is stored for fast access
   mp3s_present := false;
   for i := 0 to childs.count -1 do
   begin
     if childs.objects[i] is TTagObject then
     begin
       if not mp3s_present then
       begin
         assignfile(fs, path + '\' + name + '\mp3info.dta');
         rewrite(fs);
         mp3s_present := true;
       end;
       TTagObject(childs.objects[i]).save(fs,true);
     end;
   end;
   if mp3s_present then
     closefile(fs);
end;

procedure TSumObject.loadLocal(filelist: TStringList);
var
 line : string;
 tag: TTagObject;
 fs : textfile;
begin
  if fileexists(path + '\' + name + '\mp3info.dta') then
  begin
    assignfile(fs,path + '\' + name + '\mp3info.dta');
    reset(fs);
    try
    while not eof(fs) do
    begin
      readln(fs,line);
      if length(line)>0 then
      begin
        tag := TTagObject.Create('','',false, filelist);
        try
          tag.load(line,path + '\' + name, filelist);
          addtag(tag);
        except
          debugger.logmsg('Except in load: ' + line);
          tag.free;
        end;
      end;
    end;
    finally
      closefile(fs);
    end;
  end;
end;

procedure TSumObject.saveUserLocal(username: string);
var
  i : integer;
  mp3s_present: boolean;
  fs : textfile;
begin
  // save the mp3 user content of a directory in the directory itself (with the mp3's)
  // the data is the mp3 user info, it is stored for fast access
   assignfile(fs, path + '\' + name + '\' + username + '.dta');
   rewrite(fs);
   for i := 0 to childs.count -1 do
   begin
     if childs.objects[i] is TTagObject then
     begin
       TTagObject(childs.objects[i]).saveuserdata(fs);
     end;
   end;
   closefile(fs);
end;

procedure TSumObject.loadUserLocal(username: string);
var
 p,l:integer;
 line, token : string;
 tag: TTagObject;
 fs : textfile;
 i : integer;
begin
  if fileexists(path + '\' + name + '\' + username + '.dta') then
  begin
    assignfile(fs,path + '\' + name + '\' + username + '.dta');
    reset(fs);
    try
    while not eof(fs) do
    begin
      readln(fs,line);
      if length(line)>0 then
      begin
        p:=0;
        l := length(line);
        token := charctrl(next_token(p,l,line));
        i := childs.IndexOf(token);
        if i >= 0 then
        begin
          tag := TTagObject(childs.objects[i]);
          tag.loadUserData(line);
        end
        else
        begin
          debugger.LogMsg('loaduserdata: ' + line);
          debugger.logmsg('loaduserdata: not found: ' + path + '\' + name + ' ' + token);
        end;
      end;
    end;
    finally
      closefile(fs);
    end;
  end;
end;

procedure TSumObject.loadUserInfoTxt;
var
  fs : textfile;
  i,p,c : integer;
  line : string;
begin
  if fileexists(path + '\' + name + '\info.txt') then
  begin
    assignfile(fs,path + '\' + name + '\info.txt');
    reset(fs);
    i := 0;
    try
      while not eof(fs) and (i < childs.count) do
      begin
        readln(fs,line);
        p := pos(' ',line);
        if p > 0 then
        begin
          c := strtoint(copy(line,1,p-1));
          ttagobject(childs.objects[i]).playcnt := c;
        end;
        inc(i);
      end;
    finally
     closefile(fs);
    end;
  end;
end;


procedure TSumObject.load(var fs: textfile; dirlist, filelist: TStringList);
var
  k: integer;
  p: integer;
  l: integer;
  line: string;
  childCnt : integer;
  i : integer;
  sum: TSumObject;
  tag: TTagObject;
begin
  readln(fs,line);
  l := length(line);
  p := 0;
  k := 0;
  try
  name := next_token(p,l,line);
  inc(k);
  path := next_token(p,l,line);
  inc(k);
  duration := strtoint(next_token(p,l,line));
  inc(k);
  count := strtoint(next_token(p,l,line));
  inc(k);
  size := strtoint64(next_token(p,l,line));
  inc(k);
  childCnt := strtoint(next_token(p,l,line));
  inc(k);
  for i := 1 to childCnt do
  begin
    readln(fs,line);
    if line = 'S' then
    begin
      sum := TSumObject.Create('', dirlist);
      sum.load(fs,dirlist, filelist);
      addSum(sum);
    end
    else
    begin
      tag := TTagObject.Create('','',false, filelist);
      readln(fs,line);
      tag.load(line,'', filelist);
      addTag(tag);
    end;
  end;
  except
    on e: exception do
      debugger.LogError('sumobject.load.' + e.message + ':' + inttostr(k) + '>' + line);
  end;
end;

function TSumObject.getField(col: TColumnName): string;
begin
  case col of
    cnName: result := name;
    cnPath: result := path;
    cnDuration: result := secondsToString(duration);
    cnCount: result := inttostr(count);
    cnSize: result := sizetostr(size);
    cnPlayed: result := booleantochar(playcnt > 0);
    cnPlaycnt: result := inttostr(playcnt);
    cnTagged: result := '';
    cnYear: result := '';
    cnArtist: result := '';
    cnAlbum: result := '';
    cnTitle: result := '';
    cnComment: result := '';
    cnGenre: result := '';
    cnTrack: result := '';
    cnBitrate: result := '';
    cnBpm: result := '';
    cnMode: result := '';
  end;
end;

function TSumObject.getFieldVal(col: TColumnName): string;
begin
  case col of
    cnName: result := lowercase(name);
    cnPath: result := lowercase(path);
    cnDuration: result := leftpad(inttostr(duration), 10);
    cnCount: result := leftpad(inttostr(count), 10);
    cnSize: result := leftpad(inttostr(size), 20);
    cnPlayed: result := booleantochar(playcnt > 0);
    cnPlaycnt: result := leftpad(inttostr(playcnt), 10);
    cnTagged: result := '';
    cnYear: result := '';
    cnArtist: result := '';
    cnAlbum: result := '';
    cnTitle: result := '';
    cnComment: result := '';
    cnGenre: result := '';
    cnTrack: result := '';
    cnBitrate: result := '';
    cnBpm: result := '';
    cnMode: result := '';
  end;
end;

procedure TSumObject.updatePlaycnt;
var j: integer;
begin
  played := 0;
  playcnt := 0;
  for j := 0 to childs.count - 1 do
  begin
    if childs.objects[j] is TSumObject then
    begin
      played := played + TSumObject(childs.objects[j]).played;
      playcnt := playcnt + TSumObject(childs.objects[j]).playcnt;
    end
    else if childs.objects[j] is TTagObject then
    begin
      if TTagObject(childs.objects[j]).playcnt > 0 then
      begin
        inc(played);
        playcnt := playcnt + TTagObject(childs.objects[j]).playcnt;
      end;
    end;
  end;
  if assigned(parent) then
    parent.updatePlaycnt;
end;

procedure TSumObject.sum;
var j : integer;
begin
  duration := 0;
  size := 0;
  count := 0;
  played := 0;
  playcnt := 0;
  for j := 0 to childs.count - 1 do
  begin
    if childs.objects[j] is TSumObject then
    begin
      TSumObject(childs.objects[j]).sum;
      duration := duration + TSumObject(childs.objects[j]).duration;
      size := size + TSumObject(childs.objects[j]).size;
      count := count + TSumObject(childs.objects[j]).count;
      played := played + TSumObject(childs.objects[j]).played;
      playcnt := playcnt + TSumObject(childs.objects[j]).playcnt;
    end
    else if childs.objects[j] is TTagObject then
    begin
      duration := duration + TTagObject(childs.objects[j]).duration;
      size := size + TTagObject(childs.objects[j]).filelength;
      if TTagObject(childs.objects[j]).playcnt > 0 then
      begin
        inc(played);
        playcnt := playcnt + TTagObject(childs.objects[j]).playcnt;
      end;
      inc(count);
    end;
  end;
end;

// this is for sort, the returned string is representative for sort

function TTagObject.fullName: string;
begin
  result := path + '\' + filename;
end;

function TTagObject.getField(col: TColumnName): string;
begin
  case col of
    cnName: result := filename;
    cnPath: result := path;
    cnDuration: result := secondsToString(duration);
    cnCount: result := inttostr(1);
    cnSize: result := sizetostr(filelength);
    cnPlayed: result := booleantochar(playcnt > 0);
    cnPlaycnt: result := inttostr(playcnt);
    cnTagged: result := booleantochar(isTagged);
    cnYear: result := year;
    cnArtist: result := artist;
    cnAlbum: result := album;
    cnTitle: result := title;
    cnComment: result := comment;
    cnGenre: result := genre2string(genre);
    cnTrack: result := inttostr(track);
    cnBitrate: result := inttostr(bitrate);
    cnBpm: result := inttostr(bpm);
    cnMode: result := inttostr(mode);
    cnStyle: result := style;
  end;
end;

function TTagObject.getFieldVal(col: TColumnName): string;
begin
  case col of
    cnName: result := lowercase(filename);
    cnPath: result := lowercase(path);
    cnDuration: result := leftpad(inttostr(duration), 10);
    cnCount: result := '         1';
    cnSize: result := leftpad(inttostr(filelength), 20);
    cnPlayed: result := booleantochar(playcnt > 0);
    cnPlaycnt: result := leftpad(inttostr(playcnt), 10);
    cnTagged: result := booleantochar(isTagged);
    cnYear: result := year;
    cnArtist: result := lowercase(artist);
    cnAlbum: result := lowercase(album);
    cnTitle: result := lowercase(title);
    cnComment: result := comment;
    cnGenre: result := genre2string(genre);
    cnTrack: result := leftpad(inttostr(track), 3);
    cnBitrate: result := leftpad(inttostr(bitrate), 4);
    cnBpm: result := leftpad(inttostr(bpm), 4);
    cnMode: result := inttostr(mode);
    cnStyle: result := style;
  end;
end;

procedure TTagObject.ReadId3Tags;
var
  fich: TFileStream;
  tag: TId3v1Tag;
  thename: string;
  buf: array[0..10000] of byte;
  cnt: integer;
begin
  try
    thename := fullname;
    fich := TFileStream.Create(theName, fmOpenRead or fmShareDenyWrite);
    filelength := fich.Size;

    fich.Position := fich.Size - 128;
    fich.Read(tag, sizeof(tag));
    if tag.Identifier = 'TAG' then
    begin
      title := trim(tag.SongName);
      artist := trim(tag.Artist);
      album := trim(tag.Album);
      year := trim(tag.Year);
      comment := trim(tag.Comment);
      genre  := tag.Genre;
    end
    else
    begin
      title := removeTrackNumber(get_trackname(thename));;
      artist := get_artist(theName);
      album := get_album(theName);
      year := '';
      comment := '';
      genre := 126;
    end;
    fich.Seek(0,soFromBeginning);   // go to beginning of file
    cnt := fich.Read(buf,10000);
    fich.Free;
  except;
  end;
end;

procedure TTagObject.ReadMpgTags;
var
  mpg : TMPEGAUDIO;
begin
  mpg := TMPegAudio.Create;
  mpg.FileName := FullName;
  if mpg.isTagged then
  begin
      title := trim(mpg.Title);
      artist := trim(mpg.Artist);
      album := trim(mpg.Album);
      year := trim(mpg.Year);
      comment := trim(mpg.Comment);
      genre  := mpg.Genre;
  end
  else
  begin
    title := removeTrackNumber(get_trackname(mpg.filename));;
    artist := get_artist(mpg.filename);
    album := get_album(mpg.filename);
    year := '';
    comment := '';
    genre := 126;
  end;
  bitrate    := mpg.BitRate;
  duration   := mpg.Duration;
  filelength := mpg.FileLength;
  layer      := mpg.Layer;
  mode       := mpg.Mode;
  samplerate := mpg.SampleRate;
  mpg.free;
end;

procedure TTagObject.saveuserdata(var fs: textfile);
begin
  try
    writeln(fs,
      ctrlchar(filename),'|',
      playcnt, '|',
      merk);
  except
    writeln(fs, '0|0');
  end;
end;

procedure TTagObject.loaduserdata(line: string);
var
  p: integer;
  l: integer;
  lname: string;
begin
  try
    l := length(line);
    p := 0;
    lname := charctrl(next_token(p,l,line));
    if filename = lname then
    begin
      playcnt := strtoint(next_token(p,l,line));
      merk := strtoint(next_token(p,l,line));
    end;
  except
    on e: exception do
      debugger.LogError('Loaduserdata.' + e.Message + ' ' + line);
  end;
end;

procedure TTagObject.save(var fs: textfile; local: boolean);
var  lpath: string;
   userdat: string;
begin
  if not local then
  begin
    lpath := path;
    userdat := '|' + inttostr(playcnt) + '|' + inttostr(merk);
  end;
  writeln(fs,
    '1|', // current_version number
    ctrlchar(filename), '|',
    ctrlchar(lpath),'|',
    istagged, '|',
    ctrlchar(year), '|',
    ctrlchar(artist), '|',
    ctrlchar(album), '|',
    ctrlchar(title), '|',
    ctrlchar(comment), '|',
    ctrlchar(style), '|',
    genre, '|',
    track, '|',
    duration, '|',
    filelength, '|',
    version, '|',
    layer, '|',
    samplerate, '|',
    bitrate, '|',
    bpm, '|',
    mode, userdat);
end;

procedure TTagObject.load(line: string; aPath : string; filelist:TStringList);
var
  k:integer;
  p: integer;
  l: integer;
  version: integer;
begin
  try
    l := length(line);
    p := 0;
    k := 0;
    version := strtoint(next_token(p,l,line));
    if version = 1 then
    begin
      inc(k);
      filename := charctrl(next_token(p,l,line));
      inc(k);
      path := charctrl(next_token(p,l,line));
      inc(k);
      if length(apath) > 0 then
        path := apath;  // override the path, local load
      inc(k);
      istagged := strtobool(next_token(p,l,line));
      inc(k);
      year := charctrl(next_token(p,l,line));
      inc(k);
      artist := charctrl(next_token(p,l,line));
      inc(k);
      album := charctrl(next_token(p,l,line));
      inc(k);
      title := charctrl(next_token(p,l,line));
      inc(k);
      comment := charctrl(next_token(p,l,line));
      inc(k);
      style := charctrl(next_token(p,l,line));
      inc(k);
      genre := strtoint(next_token(p,l,line));
      inc(k);
      track := strtoint(next_token(p,l,line));
      inc(k);
      duration := strtoint(next_token(p,l,line));
      inc(k);
      filelength := strtoint(next_token(p,l,line));
      inc(k);
      version := strtoint(next_token(p,l,line));
      inc(k);
      layer := strtoint(next_token(p,l,line));
      inc(k);
      samplerate := strtoint(next_token(p,l,line));
      inc(k);
      bitrate := strtoint(next_token(p,l,line));
      inc(k);
      bpm := strtoint(next_token(p,l,line));
      inc(k);
      mode := strtoint(next_token(p,l,line));
      inc(k);
      if length(apath) = 0 then
      begin
        try
          playcnt := strtoint(next_token(p,l,line));
          merk := strtoint(next_token(p,l,line));
        except
        end;
      end;
    end;
    filelist.AddObject(fullname,self);
  except
    on e: exception do
      debugger.LogError('tagobject.load.' + e.message + ':' + inttostr(k) + '>' + line);
  end;
end;


constructor TTagObject.Create(aname: string; apath: string; scantags: boolean; filelist:TstringList);
begin
  inherited Create;
  parents := TList.Create;
  filename := aname;
  path := apath;
  // an exception is made for loading from dta file
  // then the name is not known jet, after the load the object will be put in the list
  if length(aname) > 0 then
    filelist.addobject(fullname, Self);
  if scantags then
  begin
    ReadId3Tags;
  end;
end;

destructor TTagObject.Destroy;
begin
  parents.free;
  inherited;
end;

procedure TTagObject.addParent(parent: TSumObject);
begin
  if parents.IndexOf(pointer(parent)) < 0 then
    parents.Add(parent);
end;

procedure TTagObject.incrementPlaycnt(username:string);
var i: integer;
begin
  inc(playcnt);
  for i := 0 to parents.count - 1 do
  begin
    TSumObject(parents[i]).updatePlaycnt;
  end;
  TSumObject(parents[0]).saveuserlocal(username);
end;

procedure TTagObject.saveParent;
begin
  TSumObject(parents[0]).savelocal;
end;

procedure TMp3ScanThread.Execute;
var
  duration: integer;
  size: int64;
  played, playcnt: integer;
  cnt: integer;
  i, j, p, n: integer;
  sum, artist: TSumObject;
  tag: TTagObject;
begin
  priority := tpLower;
  try
  if DoScanDir then
  if assigned(scandirs) then
  if scandirs.count > 0 then
  begin
    for i := 0 to scandirs.count -1 do
    begin
      if not Terminated then
      begin
        scanDir(basesum,scandirs[i]);
        basesum.sum;
        refreshtree;
      end;
    end;
  end;
  if DoScanTags and (not Terminated) then
  begin
    scanTags;
    refreshtree;
  end;
  finally
    basesum.sum;
  end;
//  Terminate;
end;

procedure TMp3ScanThread.ScanProgress;
begin
  if assigned(FOnProgress) then
    FOnProgress(progress);
end;

procedure TMp3ScanThread.RefreshTree;
begin
  if assigned(FOnRefreshTree) then
    FOnRefreshTree(self);
end;

procedure TMp3ScanThread.scanDir(parentobj: TSumObject; path : string);
var
  search: Tsearchrec;
  result, index: integer;
  tag: TTagObject;
  sum: TSumObject;
begin
  // first try to recurse the directory
  ReportProgress('scan: ' + path);
  result := findfirst(path + '\*.*', faDirectory,search);
  while (result = 0) and (not Terminated) do
  begin
    if ((search.attr and fadirectory) = fadirectory) and
       (search.Name <> '.') and
       (search.name <> '..') then
    begin
      sum := nil;
      if assigned(parentobj) then
      begin
        index := parentobj.childs.IndexOf(search.name);
        if Index >= 0 then
          sum := TSumObject(parentobj.childs.objects[index]);
      end;
      if not assigned(sum) then
      begin
        sum := TSumObject.Create(search.name, dirs);
        sum.path := path;
        parentobj.addSum(sum);
      end;
      scanDir(sum,path + '\' + search.name);
    end;
    result := findnext(search);
  end;
  findclose(search);

//  ReportProgress('scan done : ' + path);

  // then try to find the mp3info file
  if fileexists(path + '\mp3info.dta') then
  begin
//    ReportProgress('mp3info: ' + path);
    parentobj.loadLocal(files);
    if fileexists(path + '\' + Fusername + '.dta') then
      parentobj.loadUserLocal(FUserName)
  end
  else // try to find the mp3's
  begin
    result := findfirst(path + '\*.mp3', faanyfile, search);
    while result = 0 do
    begin
      try
        tag := TTagObject.Create(search.name,path,true, files);
//       tag.path := copy(path, 1, length(path) - 3);
        parentobj.addTag(tag);
      except
        on e: exception do
          debugger.LogError('Scandir: ' + search.name + ' exc: ' + e.Message);
      end;
      result := findnext(search);
    end;
    if fileexists(path + '\' + Fusername + '.dta') then
      parentobj.loadUserLocal(FUserName)
    else
      if fileexists(path + '\info.txt') then
      begin
        parentobj.loadUserInfoTxt;
        parentobj.saveuserlocal(FUserName);
      end;
    parentobj.saveLocal;
  end;
  findclose(search);
end;

procedure TMp3ScanThread.scanTags;
var i,p,n : integer;
  str : string;
begin
  p := 0;
  i := files.count-1;
  while (i >= 0) and (not terminated) do
  begin
    if useMpg then
      TTagObject(files.objects[i]).ReadMpgTags
    else
      TTagObject(files.objects[i]).ReadId3Tags;
    if TTagObject(files.objects[i]).path <> str then
    begin
//      ReportProgress(TTagObject(files.objects[i]).path);
      str := TTagObject(files.objects[i]).path;
    end;
    dec(i);

    // report progress in %
    n := ((files.count - i) * 100) div files.count;
    if n <> p then
    begin
      ReportProgress('tags ' + inttostr(n) + ' %');
      p := n;
    end;
  end;
end;

procedure TMp3ScanThread.ReportProgress(str: string);
begin
  progress := str;
  synchronize(ScanProgress);
end;

procedure TMp3Data.ClearData;
var ix : integer;
begin
//  finish := true;
  for ix := 0 to dirs.count-1 do
    if dirs.objects[ix] <> basesum then
      dirs.objects[ix].free;
  basesum.childs.clear;
  dirs.clear;
  dirs.addobject(basesum.name,basesum);
  for ix := 0 to files.count-1 do
    files.objects[ix].free;
  files.clear;
end;

procedure TMp3Data.OnTerminate(obj: TObject);
begin
  scannerActive := false;
end;

constructor TMp3Data.Create;
begin
  dirs := TStringList.Create;
  files := TStringList.Create;
  basesum := TSumObject.Create('base', dirs);
  bases := TList.Create;
  scandirs := TStringList.Create;
  styles := TStringList.Create;
  styles.duplicates := dupIgnore;
  styles.sorted := true;
end;

destructor TMp3Data.Destroy;
begin
  if scannerActive then
  begin
    scanner.terminate;
    scanner.WaitFor;
  end;
  if assigned(scanner) then
    scanner.free;
  ClearData;
  basesum.free;
  bases.free;
  dirs.free;
  files.free;
  scandirs.free;
end;

procedure TMp3Data.Save(filename: string);
var fs: textfile;
begin
  assignfile(fs,filename);
  FileMode := 2;
  rewrite(fs);
  try
    basesum.save(fs);
  finally
    closefile(fs);
  end;
end;

procedure TMp3Data.Load(filename: string);
var fs: textfile;
i : integer;
begin
  if fileexists(filename) then
  begin
  assignfile(fs,filename);
  FileMode := 0;
  reset(fs);
  try
    basesum.load(fs,dirs,files);
  finally
    closefile(fs);
  end;
  basesum.sum;
  for i := 0 to files.count-1 do
  begin
    parseStyle(TTagObject(files.objects[i]).style,styles);
  end;
  end;
end;

procedure TMp3Data.update_playcount(afilename: string);
var i : integer;
  found : boolean;
  name : string;
  l1,l2,k: integer;
begin
  // find the tag in filelist, update the tag, parent sum etc
  // write localuserdata if username is set
  found := false;
  i := 0;
  while not found and (i<files.count) do
  begin
    with TTagObject(files.Objects[i]) do
    begin
      name := fullname;
      if name = afilename then
      begin
        incrementplaycnt(fusername);
        found := true;
      end;
    end;
    inc(i);
  end;
end;

function TMp3Data.GetTagObject(afilename: string): TTagObject;
var i : integer;
  found : boolean;
  name : string;
begin
  // find the tag in filelist
  result := nil;
  found := false;
  i := 0;
  while not found and (i<files.count) do
  begin
    with TTagObject(files.Objects[i]) do
    begin
      name := fullname;
      if name = afilename then
      begin
        result := TTagObject(files.objects[i]);
        found := true;
      end;
    end;
    inc(i);
  end;
end;

procedure TMp3Data.Artists(list: TStringList);
var i : integer;
begin
  list.duplicates := dupIgnore;
  list.sorted := true;
  for i := 0 to files.count-1 do
  begin
    list.add(TTagObject(files.objects[i]).artist);
  end;
end;

procedure TMp3Data.ArtistsNoStyle(list: TStringList);
var i : integer;
 a2a : TArtist2Artist;
 artist: string;
 style: string;
begin
  list.duplicates := dupIgnore;
  list.sorted := true;
  a2a := TArtist2Artist.create;
  for i := 0 to files.count-1 do
  begin
    if ttagobject(files.objects[i]).style = '' then
    begin
      artist := a2a.artist(TTagObject(files.objects[i]).artist);
      style := a2a.style(artist);
      if style = '' then
      begin
        list.add(a2a.artist(TTagObject(files.objects[i]).artist));
      end
      else
      begin
        ttagobject(files.objects[i]).style := style;
        ttagobject(files.objects[i]).saveparent;
      end;
    end;
  end;
end;

procedure TMp3Data.ScanDir(dirs,tags,mpg: boolean);
begin
  if dirs then
    ClearData;
  StartScanner(dirs,tags,mpg);
end;

procedure TMp3Data.ScanTags;
begin
end;

procedure TMp3Data.RefreshTree(sender: TObject);
begin
  if assigned(FOnRefreshTree) then
   FOnRefreshTree(sender);
end;

procedure TMp3Data.setDirs(theDirs: TStringList);
begin
  scandirs.Clear;
  scandirs.AddStrings(theDirs);
end;

procedure TMp3Data.StartScanner(dir,tags,mpg : boolean);
begin
  if scannerActive then
  begin
    scanner.Terminate;
    scanner.waitfor;
  end;
  if assigned(scanner) then
  begin
    scanner.free;
    scanner := nil;
    scannerActive := false;
  end;

  scanner := TMp3ScanThread.Create(true);
  scanner.files   := files;
  scanner.dirs    := dirs;
  scanner.basesum := basesum;
  scanner.FUserName := FUserName;

  scanner.scanDirs    := scanDirs;
  scanner.DoScanDir   := dir;
  scanner.DoScanTags  := tags;
  scanner.UseMpg      := mpg;
  scanner.OnTerminate := OnTerminate;
  scanner.OnProgress  := OnProgress;
  scanner.OnRefreshTree := RefreshTree;

  scanner.Resume;
  scannerActive := true;
//  scanner.Execute;
//  scanner.free;
//  scanner := nil;
end;

procedure TMp3Data.DelBase(index: integer);
begin
// not jet implemented
end;

procedure TMp3Data.SumGenre;
var i : integer;
  sum: TSumObject;
  sgenre: TSumObject;
  sArtist: TSumObject;
  tag: TTagObject;
begin
  for i := 0 to bases.count-1 do
  if TSumObject(bases[i]).name = 'Genre' then
    exit;
  // Genre does not exist jet
  sum := TSumObject.Create('Genre',dirs);
  bases.Add(sum);
  for i := 0 to files.count-1 do
  begin
    tag := TTagObject(files.objects[i]);
    sGenre := sum.FindSum(genre2string(tag.genre));
    if not assigned(sGenre) then
    begin
      sGenre := TSumObject.Create( genre2string(tag.genre),dirs);
      sum.addSum(sGenre);
    end;
    sArtist := sGenre.FindSum(tag.artist);
    if not assigned(sArtist) then
    begin
      sArtist := TSumObject.Create(tag.artist,dirs);
      sGenre.addSum(sArtist);
    end;
    sArtist.addTag(tag);
  end;
  sum.sum;
end;

procedure TMp3Data.SumArtist;
var i,p : integer;
  sum: TSumObject;
  sArtist: TSumObject;
  tag: TTagObject;
  a2a: TArtist2artist;
  artist: string;
begin
  for i := 0 to bases.count-1 do
  if TSumObject(bases[i]).name = 'Artist' then
    exit;
  // Genre does not exist jet
  a2a := TArtist2artist.Create;
  sum := TSumObject.Create('Artist',dirs);
  bases.Add(sum);
  for i := 0 to files.count-1 do
  begin
    tag := TTagObject(files.objects[i]);
    Artist := a2a.artist(tag.artist);
    sArtist := sum.FindSum(artist);
    if not assigned(sArtist) then
    begin
      sArtist := TSumObject.Create(artist,dirs);
      sum.addSum(sArtist);
    end;
    sArtist.addTag(tag);
  end;
  sum.sum;
  a2a.free;
end;

procedure TMp3Data.SumStyle;
var i,j : integer;
  sum: TSumObject;
  sStyle: TSumObject;
  sArtist: TSumObject;
  sAlbum: TSumObject;
  a2a: TArtist2artist;
  tag: TTagObject;
  artist: string;
  styles: TStringList;
  allStyles: TStringList;
begin
  styles := TStringList.Create;
  styles.Duplicates := dupIgnore;
  styles.sorted := true;

  allstyles := TStringList.Create;
  a2a := TArtist2artist.Create;
  for i := 0 to bases.count-1 do
  if TSumObject(bases[i]).name = 'Style' then
    exit;
  // Genre does not exist jet
  sum := TSumObject.Create('Style',dirs);
  bases.Add(sum);
  for i := 0 to files.count-1 do
  begin
    tag := TTagObject(files.objects[i]);
    styles.clear;
    parseStyle(tag.style,styles);
    for j := 0 to styles.count-1 do
    begin
    sStyle := sum.FindSum((styles[j]));
    if not assigned(sStyle) then
    begin
      sStyle := TSumObject.Create( styles[j],dirs);
      sum.addSum(sStyle);
    end;
    sArtist := sStyle.FindSum(tag.artist);
    if not assigned(sArtist) then
    begin
      sArtist := TSumObject.Create(tag.artist,dirs);
      sStyle.addSum(sArtist);
    end;
    sAlbum := sArtist.FindSum(tag.album);
    if not assigned(sAlbum) then
    begin
      sAlbum := TSumObject.Create(tag.album,dirs);
      sArtist.addSum(sAlbum);
    end;
    sAlbum.addTag(tag);
    end;
  end;
  sum.sum;

  a2a.free;
  styles.Free;
  allstyles.free;
end;

{scan a directory

 this procedure scans a directory and tries to find the mp3's,
 it first looks for a file called mp3info.dta this file contains the
 mp3 info of the current directory (mp3 files with taginfo etc)

 if not found then it tries to find mp3's int the dir, if found the
 tag info is read and the mp3info file is saved
}

end.
