//---------------------------------------------------------------------------

#include <vcl.h>
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <DB.hpp>

#include "VirtualTrees.hpp"

#pragma hdrstop

#include "BaseVirtualDBTree.h"
#pragma link "VirtualTrees"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
// ComponentState

static inline void ValidCtrCheck(TBaseVirtualDBTree *)
{
  new TBaseVirtualDBTree(NULL);
}
//==============================================================================
//        
//------------------------------------------------------------------------------
__fastcall TVTDataLink::TVTDataLink(TBaseVirtualDBTree* ATree)
: TDataLink()
{
  FTree = ATree;
}

//------------------------------------------------------------------------------
__fastcall TBaseVirtualDBTree::TBaseVirtualDBTree(TComponent* Owner)
  : TCustomVirtualStringTree(Owner)
{
  //   
  FDataLink     = new TVTDataLink(this);

  //  
  FKeyField     = 0;
  FParentField  = 0;
  FPathField    = 0;
  FLevelField   = 0;

  //   .
  //    ,
  //       
  FDBDataSize   = sizeof(TDBVTData);
  NodeDataSize  = FDBDataSize;

  //  
  FDBOptions << dboCheckDBStructure << dboParentStructure << dboWriteLevel << dboWriteSecondary
             << dboTrackActive << dboTrackChanges << dboTrackCursor << dboAlwaysStructured
             << dboViewAll;

  // 
  FOnReadNodeFromDB  = 0;
  FOnOpeningDataSet  = 0;
  FOnWritingDataSet  = 0;
  FOnReadPathFromDB  = 0;
  FOnWritePathToDB   = 0;
  FOnNodeDataChanged = 0;

  OnDragOver         = OnDragOverHandler;

  FMaxLevel = 0;
  FCurID    = 0;
}

//------------------------------------------------------------------------------
__fastcall TBaseVirtualDBTree::~TBaseVirtualDBTree()
{
  delete FDataLink;
}

//==============================================================================
//      
//------------------------------------------------------------------------------
//   
//
void __fastcall TBaseVirtualDBTree::SetDataSource(TDataSource* value)
{
  FDataLink->DataSource = value;
  if(value)
    value->FreeNotification(this);
}

//------------------------------------------------------------------------------
TDataSource* __fastcall TBaseVirtualDBTree::GetDataSource()
{
  return FDataLink->DataSource;
}

//------------------------------------------------------------------------------
void __fastcall TBaseVirtualDBTree::Notification(TComponent* AComponent, TOperation Operation)
{
  TCustomVirtualStringTree::Notification(AComponent, Operation);
  if(Operation == opRemove && FDataLink && AComponent == DataSource)
    DataSource = 0;
}

//------------------------------------------------------------------------------
//  
//

//    
//------------------------------------------------------------------------------
//   - ,    -   
//
void __fastcall TBaseVirtualDBTree::SetKeyFieldName(AnsiString value)
{
  if(FKeyFieldName != value)
  {
    FKeyFieldName = value;
    if(FDBOptions.Contains(dboTrackActive))
      DataLinkActiveChanged();
    else
    {
      FDBStatus << dbtsChanged;
      FKeyField = 0;
      if(FDataLink->Active && FKeyFieldName != "")
        FKeyField = FDataLink->DataSet->FieldByName(FPathFieldName);
    }
  }
}

//    
//------------------------------------------------------------------------------
// Parent - ,       
//               ( dboWriteSecondary 
//           dboParentStructure).     ListView
//          ( dboListView),    
//
void __fastcall TBaseVirtualDBTree::SetParentFieldName(AnsiString value)
{
  if(FParentFieldName != value)
  {
    FParentFieldName  = value;
    FParentField      = 0;

    if(FDBOptions.Contains(dboParentStructure) && FDBOptions.Contains(dboTrackActive))
    {
      if(!FDBOptions.Contains(dboListView))
        DataLinkActiveChanged();
      else if(FDBOptions.Contains(dboAlwaysStructured))
        DataLinkActiveChanged();
      else
      {
        FDBStatus >> dbtsStructured;
        if(FDataLink->Active && FParentFieldName != "")
          FParentField = FDataLink->DataSet->FieldByName(FParentFieldName);
      }
    }
    else
    {
      if(!FDBOptions.Contains(dboTrackActive))
        FDBStatus << dbtsChanged;
      if(FDataLink->Active && FParentFieldName != "")
        FParentField = FDataLink->DataSet->FieldByName(FParentFieldName);
    }
  }
}

//------------------------------------------------------------------------------
// Path   - ,       
//               ( dboWriteSecondary 
//           dboPathStructure).     ListView
//          ( dboListView),    
//
void __fastcall TBaseVirtualDBTree::SetPathFieldName(AnsiString value)
{
  if(FPathFieldName != value)
  {
    FPathFieldName  = value;
    FPathField      = 0;

    if(FDBOptions.Contains(dboPathStructure) && FDBOptions.Contains(dboTrackActive))
    {
      if(!FDBOptions.Contains(dboListView))
        DataLinkActiveChanged();
      else if(FDBOptions.Contains(dboAlwaysStructured))
        DataLinkActiveChanged();
      else
      {
        FDBStatus >> dbtsStructured;
        if(FDataLink->Active && FPathFieldName != "")
          FPathField = FDataLink->DataSet->FieldByName(FPathFieldName);
      }
    }
    else
    {
      if(!FDBOptions.Contains(dboTrackActive))
        FDBStatus << dbtsChanged;
      if(FDataLink->Active && FPathFieldName != "")
        FPathField = FDataLink->DataSet->FieldByName(FPathFieldName);
    }
  }
}

//------------------------------------------------------------------------------
// Level - .      dboWriteLevel.
//                 
//
void __fastcall TBaseVirtualDBTree::SetLevelFieldName(AnsiString value)
{
  if(FLevelFieldName != value)
  {
    FLevelFieldName = value;
    FLevelField     = 0;
    if(FDataLink->Active && FLevelFieldName != "")
      FLevelField = FDataLink->DataSet->FieldByName(FLevelFieldName);
  }
}

//==============================================================================
//    
//
//---------------------------------------------------------------------------
//  /   
//,      ,  ,
//     (    )
//
void __fastcall TVTDataLink::ActiveChanged()
{
  FTree->DataLinkActiveChanged();
}

void __fastcall TBaseVirtualDBTree::DataLinkActiveChanged()
{
  if(!ComponentState.Contains(csDesigning))
  {
    //    
    ResetFields();

    if(FDBOptions.Contains(dboTrackActive))
    {
      //   
      if(FDataLink->Active)
      {
        InitFields();
      }

      //    
      Update();
    }
    else
      FDBStatus << dbtsChanged;
  }
}

//------------------------------------------------------------------------------
//   .
//      Next, Prior, MoveBy
//
void __fastcall TVTDataLink::DataSetScrolled(int Distance)
{
  FTree->DataLinkScrolled();
}

void __fastcall TBaseVirtualDBTree::DataLinkScrolled()
{
  if(!ComponentState.Contains(csDesigning))
  {
    // ,         
    //(   )        
    if(FKeyField && !FDBStatus.Contains(dbtsDataChanging) && FDBOptions.Contains(dboTrackCursor))
    {
      //   (       )
      FDBStatus << dbtsDataChanging;

      //   ID
      int ID = FKeyField->AsInteger;
      //   ,   (     )
      //       - 
      if(ID && ID != FCurID)
      {
        //   
        SetFocusToNode(FindNode(0, ID));
      }
      //  
      FDBStatus >> dbtsDataChanging;

      //      ,    
      //       
      if(!FocusedNode)
        SetFocusToNode(GetFirst());
    }
  }
}

//------------------------------------------------------------------------------
// /     ,
//    Locale,     
//
void __fastcall TVTDataLink::DataSetChanged()
{
  FTree->DataLinkChanged();
}

void __fastcall TBaseVirtualDBTree::DataLinkChanged()
{
  if(!ComponentState.Contains(csDesigning))
  {
    //  ,      / ,  
    //    .     
    //   ,    
    if(!FDataLink->Editing && !FDBStatus.Contains(dbtsDataChanging))
    {
      if(FDBOptions.Contains(dboTrackChanges))
        RefreshNodes();
      else
        FDBStatus << dbtsChanged;
    }
  }
}

//------------------------------------------------------------------------------
//     
//
void __fastcall TVTDataLink::RecordChanged(TField* Field)
{
  FTree->DataLinkRecordChanged(Field);
}

void __fastcall TBaseVirtualDBTree::DataLinkRecordChanged(TField* Field)
{
  if(!ComponentState.Contains(csDesigning))
  {
    //        (     ),
    //   .   ,    
    // 
    if(Field && FDBOptions.Contains(dboTrackChanges))
    {
      bool UpdateField = false;
      PVirtualNode Node = FDBOptions.Contains(dboTrackCursor) ?
                          FocusedNode : FindNode(0, FKeyField->AsInteger);
      if(Node)
      {
        //     
        DoNodeDataChanged(Node, Field, UpdateField);
        if(UpdateField)
          InvalidateNode(Node);
      }
    }
  }
}

//------------------------------------------------------------------------------
//     
//
void __fastcall TVTDataLink::EditingChanged()
{
  FTree->DataLinkEditingChanged();
}

void __fastcall TBaseVirtualDBTree::DataLinkEditingChanged()
{
  if(!ComponentState.Contains(csDesigning))
  {
    //     ,   
    // .      ,
    //         .
    //        
    //        
    if(FDataLink->Editing && !FDBStatus.Contains(dbtsEditing) && !FDBStatus.Contains(dbtsInsert))
    {
      if(FDBOptions.Contains(dboTrackChanges))
      {
        //       ,     
        TDBVTData* Data;
        if(FDBOptions.Contains(dboTrackCursor) && FDataLink->DataSet->State == dsEdit)
        {
          //  ,    
          if(FocusedNode && FDBOptions.Contains(dboTrackCursor))
          {
            TDBVTData* Data = (TDBVTData* )GetNodeData(FocusedNode);
            Data->Status    = dbnsEdit;
          }
        }
        else if(FDataLink->DataSet->State == dsInsert)
        {
          //   ,      
          PVirtualNode Node = AddChild(0);
          ValidateNode(Node, false);

          //  
          TDBVTData* Data = (TDBVTData* )GetNodeData(Node);
          Data->ID      = 0;
          Data->Level   = 0;
          Data->Parent  = 0;
          Data->Status  = dbnsInited;

          //   ,       
          ReadNodeFromDB(Node);

          Data->Status  = dbnsNew;

          //          
          if(FDBOptions.Contains(dboTrackCursor))
            SetFocusToNode(Node);
        }
      }
      else
        FDBStatus << dbtsChanged;
    }

    //  ,    
    if(FocusedNode && FDBOptions.Contains(dboTrackChanges) && FDBOptions.Contains(dboTrackCursor))
      InvalidateNode(FocusedNode);
  }
}

//==============================================================================
//    
//------------------------------------------------------------------------------
//   (  )
//
void __fastcall TBaseVirtualDBTree::DoFocusChange(PVirtualNode Node, int Column)
{
  //     , 
  //  -  
  //  -       
  //  -       
  if(Node)
  {
    TDBVTData* Data = (TDBVTData* )GetNodeData(Node);
    if(Data->ID != FCurID)
    {
      FCurID = Data->ID;
      if(FCurID && !FDBStatus.Contains(dbtsDataChanging) && FDBOptions.Contains(dboTrackCursor))
      {
        FDBStatus << dbtsDataChanging;
        FDataLink->DataSet->Locate(FKeyFieldName, Data->ID, TLocateOptions());
        FDBStatus >> dbtsDataChanging;
      }
    }
  }
  TCustomVirtualStringTree::DoFocusChange(Node, Column);
}

//------------------------------------------------------------------------------
//     
//
bool __fastcall TBaseVirtualDBTree::CanEdit(PVirtualNode Node, int Column)
{
  if(!FDBStatus.Contains(dbtsEditing) && !FDBStatus.Contains(dbtsInsert))
  {
    return CanWriteToDataSet(Node, Column, dbcmEdit);
  }

  return true;
}

//------------------------------------------------------------------------------
//    ( )
//
void __fastcall TBaseVirtualDBTree::DoEdit()
{
  TCustomVirtualStringTree::DoEdit();
  if(IsEditing())
  {
    TDBVTData* Data = (TDBVTData* )GetNodeData(FocusedNode);
    if(Data->Status == dbnsEdit)
      FDBStatus << dbtsEditing;
    else if(Data->Status == dbnsNew)
      FDBStatus << dbtsInsert;
    else
    {
      if(!FDBStatus.Contains(dbtsInsert))
      {
        FDBStatus << dbtsEditing;
        FDataLink->DataSet->Edit();
      }
    }
  }
}

//------------------------------------------------------------------------------
//  
//
void __fastcall TBaseVirtualDBTree::DoEndEdit()
{
  TCustomVirtualStringTree::DoEndEdit();

  TDBVTData* Data = (TDBVTData* )GetNodeData(FocusedNode);
  Data->Status    = dbnsRefreshed;

  if(FDBStatus.Contains(dbtsEditing))
  {
    FDBStatus >> dbtsEditing;

    FDBStatus << dbtsDataChanging;
    if(FDataLink->Editing)
      FDataLink->DataSet->Post();
    FDBStatus >> dbtsDataChanging;
  }
  else if(FDBStatus.Contains(dbtsInsert))
  {
    FDBStatus >> dbtsInsert;

    FDBStatus << dbtsDataChanging;
    if(FDataLink->Editing)
      FDataLink->DataSet->Post();
    FDBStatus >> dbtsDataChanging;

    Data->ID        = FKeyField->AsInteger;
    FCurID          = Data->ID;
  }
}

//------------------------------------------------------------------------------
//  :    
//
void __fastcall TBaseVirtualDBTree::DoCancelEdit()
{
  if(FDBStatus.Contains(dbtsInsert))
  {
    FDBStatus >> dbtsInsert;

    FDBStatus << dbtsDataChanging;
    if(FDataLink->Editing)
      FDataLink->DataSet->Cancel();
    FDBStatus >> dbtsDataChanging;

    TCustomVirtualStringTree::DoCancelEdit();
    DeleteNode(FocusedNode);

    DataLinkScrolled();
    return;
  }
  if(FDBStatus.Contains(dbtsEditing))
  {
    FDBStatus >> dbtsEditing;

    FDBStatus << dbtsDataChanging;
    if(FDataLink->Editing)
      FDataLink->DataSet->Cancel();
    FDBStatus >> dbtsDataChanging;

    TDBVTData* Data = (TDBVTData* )GetNodeData(FocusedNode);
    Data->Status    = dbnsRefreshed;
  }

  TCustomVirtualStringTree::DoCancelEdit();
}

//------------------------------------------------------------------------------
//         
//
void __fastcall TBaseVirtualDBTree::DoCollapsed(PVirtualNode Node)
{
  if(Node)
  {
    PVirtualNode Focus;

    if(FocusedNode && HasAsParent(FocusedNode, Node))
    {
      Focus = Node;
      if(!Selected[Focus])
      {
        Focus = GetNextSibling(Node);
        while(Focus && !Selected[Focus])
          Focus = GetNextVisible(Focus);
        if(!Focus)
        {
          Focus = GetPreviousVisible(Node);
          while(Focus && !Selected[Focus])
            Focus = GetPreviousVisible(Focus);
          if(!Focus)
            Focus = Node;
        }
      }
      FocusedNode     = Focus;
      Selected[Focus] = true;
    }

    Focus = GetNextSibling(Node);
    if(!Focus)
    {
      Focus = GetLastChild(Node);
      if(!Focus)
        Focus = Node;
      Focus = GetNext(Focus);
    }

    Node  = GetNext(Node);
    while(Node != Focus)
    {
      Selected[Node] = false;
      Node = GetNext(Node);
    }
  }

  TCustomVirtualStringTree::DoCollapsed(Node);
}

//------------------------------------------------------------------------------
//   
//TODO: 
HRESULT __fastcall TBaseVirtualDBTree::DoDragDrop(const _di_IDataObject DataObject, int KeyState, const TPoint& Pt, int &Effect)
{
  Effect = DROPEFFECT_MOVE;

  if(CanWriteToDataSet(DropTargetNode, 0, dbcmStructure))
  {

    bool CanProcess = true;
    if(CanProcess)
    {
      PVirtualNode Focus = FocusedNode;

      BeginUpdate();
      FDataLink->DataSet->DisableControls();
      FDBStatus << dbtsDataChanging << dbtsDragDrop;

      ProcessDrop(DataObject, DropTargetNode, Effect, amAddChildLast);
      Effect = DROPEFFECT_LINK;

      FocusedNode = 0;

      EndUpdate();
      FDataLink->DataSet->EnableControls();
      FDBStatus >> dbtsDataChanging >> dbtsDragDrop;

      FCurID = 0;
      FocusedNode = Focus;
    }
    else
      Effect = DROPEFFECT_NONE;
  }
  else
    Effect = DROPEFFECT_NONE;

  return TCustomVirtualStringTree::DoDragDrop(DataObject, KeyState, Pt, Effect);
}

//------------------------------------------------------------------------------
//      ?
//
void __fastcall TBaseVirtualDBTree::OnDragOverHandler(TBaseVirtualTree *Sender, TObject *Source,
  TShiftState Shift, TDragState State, const TPoint &Pt, TDropMode Mode, int &Effect, bool &Accept)
{
  Accept = CanWriteToDataSet(DropTargetNode, 0, dbcmStructure);
}

//------------------------------------------------------------------------------
//  .      Drag & Drop
//
void __fastcall TBaseVirtualDBTree::DoNodeMoved(PVirtualNode Node)
{
  if(FDBStatus.Contains(dbtsDragDrop))
  {
    TDBVTData*    Data;
    AnsiString    Path;
    PVirtualNode  Parent;

    int ParentID  = 0;
    int Level     = 0;

    Parent        = Node->Parent;

    if(Parent != RootNode)
    {
      Data      = (TDBVTData* )GetNodeData(Parent);
      Level     = Data->Level + 1;
      ParentID  = Data->ID;
      if(FDBOptions.Contains(dboPathStructure) || FDBOptions.Contains(dboWriteSecondary))
      {
        Path    = ParentID;
        Parent  = Parent->Parent;
        while(Parent != RootNode)
        {
          Data    = (TDBVTData* )GetNodeData(Parent);
          Path    = AnsiString(Data->ID) + AnsiString('.') + Path;
          Parent  = Parent->Parent;
        }
      }
    }

    Data        = (TDBVTData* )GetNodeData(Node);
    Data->Level = Level;

    FDataLink->DataSet->Locate(FKeyFieldName, Data->ID, TLocateOptions());
    FDataLink->DataSet->Edit();

    if(FDBOptions.Contains(dboPathStructure) || FDBOptions.Contains(dboWriteSecondary))
    {
      DoWritePathToDB(Path);
      FPathField->AsString = Path;
    }
    if(FDBOptions.Contains(dboParentStructure) || FDBOptions.Contains(dboWriteSecondary))
    {
      FParentField->AsInteger = ParentID;
    }
    if(FDBOptions.Contains(dboWriteLevel))
    {
      FLevelField->AsInteger = Level;
    }
    FDataLink->DataSet->Post();

    TCustomVirtualStringTree::DoNodeMoved(Node);

    Node = GetFirstChild(Node);
    while(Node)
    {
      DoNodeMoved(Node);
      Node = GetNextSibling(Node);
    }
  }
}

//------------------------------------------------------------------------------
//     ( ,    DeleteSelection)
//
void __fastcall TBaseVirtualDBTree::DoFreeNode(PVirtualNode Node)
{
  TDBVTData* Data = (TDBVTData* )GetNodeData(Node);
  if(Data->Status == dbnsDelete)
  {
    if(FDataLink->DataSet->Locate(FKeyFieldName, Data->ID, TLocateOptions()))
      FDataLink->DataSet->Delete();
  }

  TCustomVirtualStringTree::DoFreeNode(Node);
}

//------------------------------------------------------------------------------

//==============================================================================
//    -
//------------------------------------------------------------------------------
//
//      
//
void __fastcall TBaseVirtualDBTree::SetFocusToNode(PVirtualNode Node)
{
  //    - ,      
  if(FocusedNode)
  {
    Selected[FocusedNode] = false;
  }
  FocusedNode = Node;
  //       
  if(Node)
  {
    Selected[Node]     = true;
    FullyVisible[Node] = true;
  }
}

//------------------------------------------------------------------------------
//       
//
PVirtualNode __fastcall TBaseVirtualDBTree::FindChild(PVirtualNode Node, int ID)
{
  PVirtualNode Result = GetFirstChild(Node);
  TDBVTData*   Data;
  while(Result)
  {
    Data = (TDBVTData* )GetNodeData(Result);
    if(Data->ID == ID)
      break;
    Result = GetNextSibling(Result);
  }
  return Result;
}

//------------------------------------------------------------------------------
//      ,   
//
PVirtualNode __fastcall TBaseVirtualDBTree::FindNode(PVirtualNode Start, int ID)
{
  TDBVTData* Data;
  PVirtualNode Result = (Start) ? Start : GetFirst();

  while(Result)
  {
    Data = (TDBVTData* )GetNodeData(Result);
    if(Data->ID == ID)
      break;
    Result = GetNext(Result);
  }
  return Result;
}

//------------------------------------------------------------------------------
//      
//
void __fastcall TBaseVirtualDBTree::GoTo(WideString AString, TDBVTGoToMode Mode)
{
  EndEditNode();

  WideString   Text;
  PVirtualNode Node;

  int Column = Header->MainColumn;
  switch(Mode)
  {
    case gtmFromFirst:
      Node = GetFirst();
      Mode = gtmNext;
      break;

    case gtmNext:
      Node = GetNext(FocusedNode);
      break;

    case gtmPrev:
      Node = GetPrevious(FocusedNode);
      break;

    default:
      Node = 0;
  }

  while(Node)
  {
    DoGetText(Node, Column, ttNormal, Text);
    if(Text.Pos(AString) == 1)
      break;
    Node = (Mode == gtmNext) ? GetNext(Node) : GetPrevious(Node);
  }
  if(Node)
    SetFocusToNode(Node);
}

//------------------------------------------------------------------------------
//      ID
//
bool __fastcall TBaseVirtualDBTree::GoTo(int ID)
{
  PVirtualNode Node = FindNode(0, ID);
  if(Node)
    SetFocusToNode(Node);
  return (Node != 0);
}

//------------------------------------------------------------------------------
//     
//
void __fastcall TBaseVirtualDBTree::AddNode(PVirtualNode Parent)
{
  EndEditNode();

  if(CanWriteToDataSet(Parent, 0, dbcmInsert))
  {
    int          Level;
    int          ParentID;
    AnsiString   Path;
    PVirtualNode Node;
    TDBVTData*   Data;

    FDBStatus << dbtsDataChanging;

    Node = AddChild(Parent);
    if(Parent == 0 || Parent == RootNode)
    {
      Level     = 0;
      ParentID  = 0;
      Path      = "";
    }
    else
    {
      Data      = (TDBVTData* )GetNodeData(Parent);
      Level     = Data->Level + 1;
      ParentID  = Data->ID;
      if(FDBOptions.Contains(dboPathStructure) || FDBOptions.Contains(dboWriteSecondary))
      {
        Path    = ParentID;
        Parent  = Parent->Parent;
        while(Parent != RootNode)
        {
          Data    = (TDBVTData* )GetNodeData(Parent);
          Path    = AnsiString(Data->ID) + AnsiString('.') + Path;
          Parent  = Parent->Parent;
        }
      }
    }
    Data        = (TDBVTData* )GetNodeData(Node);
    Data->ID    = 0;
    Data->Level = Level;
    Data->Parent= 0;

    FDBStatus << dbtsInsert;
    FDataLink->DataSet->Insert();

    if(!FDBOptions.Contains(dboListView))
    {
      if(FDBOptions.Contains(dboPathStructure) || FDBOptions.Contains(dboWriteSecondary))
      {
        DoWritePathToDB(Path);
        FPathField->AsString = Path;
      }
      if(FDBOptions.Contains(dboParentStructure) || FDBOptions.Contains(dboWriteSecondary))
      {
        FParentField->AsInteger = ParentID;
      }
      if(FDBOptions.Contains(dboWriteLevel))
      {
        FLevelField->AsInteger = Level;
      }
    }
    DoReadNodeFromDB(Node);

    FCurID = 0;

    SetFocusToNode(Node);

    FDBStatus >> dbtsDataChanging;

    EditNode(Node, Header->MainColumn);
  }
}

//------------------------------------------------------------------------------
//   
//TODO:  
//
void __fastcall TBaseVirtualDBTree::DeleteSelection()
{
  if(!FDBStatus.Contains(dbtsDataChanging) && FocusedNode && SelectedCount > 0 &&
     !FDBOptions.Contains(dboReadOnly) && FDataLink->Active && FKeyField &&
     !FDataLink->ReadOnly)
  {
    TDBVTData*   Data;
    PVirtualNode Temp;
    PVirtualNode Last;
    PVirtualNode Focus;

    PVirtualNode Node = GetFirst();

    Focus = FocusedNode;
    while(Selected[Focus->Parent])
      Focus = Focus->Parent;

    Temp = Focus;
    do
    {
      Focus = GetNextSibling(Focus);
    }while(Focus && Selected[Focus]);

    if(!Focus)
    {
      Focus = Temp;
      do
      {
        Focus = GetPreviousSibling(Focus);
      }while(Focus && Selected[Focus]);
      if(!Focus)
        Focus = Temp->Parent;
      if(Focus == RootNode)
        Focus = 0;
    }

    FDBStatus << dbtsDataChanging;
    BeginUpdate();
    FDataLink->DataSet->DisableControls();

    while(Node)
    {
      if(Selected[Node])
      {
        Temp = Node;
        Last = GetNextSibling(Node);
        do
        {
          Data = (TDBVTData* )GetNodeData(Temp);
          Data->Status = dbnsDelete;

          Temp = GetNext(Temp);
        }while(Temp != Last);

        if(!Temp && Node->Parent != RootNode)
          Temp = GetNextSibling(Node->Parent);
        DeleteNode(Node);
        Node = Temp;
      }
      else
        Node = GetNextVisible(Node);
    }

    FDataLink->DataSet->EnableControls();
    EndUpdate();
    FDBStatus >> dbtsDataChanging;

    if(Focus && Focus != RootNode)
    {
      SetFocusToNode(Focus);
    }
  }
}

//------------------------------------------------------------------------------
//  
//
void* __fastcall TBaseVirtualDBTree::GetDBNodeData(PVirtualNode Node)
{
  if(!Node || DBNodeDataSize == 0)
    return 0;

  return (void* )(((char* )GetNodeData(Node)) + FDBDataSize);
}

//------------------------------------------------------------------------------
//  
//
void __fastcall TBaseVirtualDBTree::RefreshNodes()
{
  if(!FDBStatus.Contains(dbtsDataChanging) && CanOpenDataSet())
  {
  //     
    FDBStatus << dbtsDataChanging;
    BeginUpdate();

    TDBVTData*   Data;
    PVirtualNode Node;
    PVirtualNode Temp;

    FMaxLevel = 0;
    FCurID    = 0;

    if(FDBOptions.Contains(dboAlwaysStructured))
    {
      if(!FDBStatus.Contains(dbtsStructured))
        Clear();
      else if(FDBOptions.Contains(dboListView))
      {
        FDBOptions >> dboListView;
        ToggleListView();
        FDBOptions << dboListView;
      }
      FDBStatus << dbtsStructured;
    }
    else
    {
      if(FDBOptions.Contains(dboListView))
        FDBStatus >> dbtsStructured;
      else
      {
        if(!FDBStatus.Contains(dbtsStructured))
          Clear();
        FDBStatus << dbtsStructured;
      }
    }

    Temp = GetFirst();
    if(!Temp)
      FDBStatus << dbtsEmpty;


    while(Temp)
    {
      Data = (TDBVTData* )GetNodeData(Temp);
      if(Data->Status == dbnsRefreshed)
        Data->Status = dbnsNone;
      Temp = GetNext(Temp);
    }

  //   
    FDataLink->DataSet->DisableControls();
  //    
    if(!FDataLink->DataSet->Eof || !FDataLink->DataSet->Bof)
      FCurID = FKeyField->AsInteger;

    int i = 0;
    while(!FDataLink->DataSet->Eof)
    {
    // TODO: RefreshNode   
      RefreshNode();
      FDataLink->DataSet->Next();
      i++;
    }
    if(i > 0)
      FDataLink->DataSet->MoveBy(-i);

    i = 0;
    while(!FDataLink->DataSet->Bof)
    {
    // TODO: RefreshNode   
      RefreshNode();
      FDataLink->DataSet->Prior();
      i++;
    }
    if(i > 0)
      FDataLink->DataSet->MoveBy(i);

  //   
    //  


    if(FDBOptions.Contains(dboTrackCursor) && FocusedNode)
    {
      Selected[FocusedNode] = false;
      FocusedNode = 0;
    }

    Temp = GetFirst();
    while(Temp)
    {
      Node = GetNext(Temp);
      Data = (TDBVTData* )GetNodeData(Temp);
      if(Data->Status != dbnsRefreshed)
      {
      //  ,       
        Node = GetNextSibling(Temp);
        DeleteNode(Temp);
      }
      else
      {
        // DONE:       Parent
        if(FDBStatus.Contains(dbtsStructured) && FDBOptions.Contains(dboParentStructure))
        {
          Data->Level = GetNodeLevel(Temp);
          FMaxLevel   = max(FMaxLevel, Data->Level);
        }
        if(FDBOptions.Contains(dboTrackCursor) && !FocusedNode && Data->ID == FCurID)
        {
          //  
          Selected[Temp]      = true;
          FocusedNode         = Temp;
          FullyVisible[Temp]  = true;
        }
        Node = GetNext(Temp);
      }
      Temp = Node;
    }

    FDataLink->DataSet->EnableControls();
    FDBStatus >> dbtsDataChanging >> dbtsChanged >> dbtsEmpty;

    if(FDBOptions.Contains(dboAlwaysStructured) && FDBOptions.Contains(dboListView))
      ToggleListView();
    if(!FDBOptions.Contains(dboListView) && !FDBOptions.Contains(dboViewAll) && !FDBStatus.Contains(dbtsToggleAll))
      ToggleViewMode();
    EndUpdate();
  }
}

//------------------------------------------------------------------------------
//   
//
void __fastcall TBaseVirtualDBTree::RefreshNode()
{
  if(!FDBStatus.Contains(dbtsStructured))
  {
    RefreshListNode();
  }
  else
  {
    if(FDBOptions.Contains(dboPathStructure))
      RefreshNodeByPath();
    else
      RefreshNodeByParent();
  }
}

//------------------------------------------------------------------------------
//     
//
void __fastcall TBaseVirtualDBTree::RefreshListNode()
{
  TDBVTData*    Data;
  PVirtualNode  Node;

  int ID  = FKeyField->AsInteger;
  Node    = FDBStatus.Contains(dbtsEmpty) ? 0 : FindChild(0, ID);
  if(!Node)
  {
    Node = AddChild(0);
    ValidateNode(Node, false);

    Data      = (TDBVTData* )GetNodeData(Node);
    Data->ID  = ID;
    Data->Parent = 0;

    Data->Status = dbnsInited;
  }
  ReadNodeFromDB(Node);
}

//------------------------------------------------------------------------------
//     ,     Path
//
void __fastcall TBaseVirtualDBTree::RefreshNodeByPath()
{
//      ,   ( ID)
//  : .. ...
//                4 , ,    256 
//                - 25.

  // ,  
  int           Pos;
  int           ID;

  PVirtualNode  Node;
  TDBVTData*    Data;

  AnsiString    Temp;
  AnsiString    Path = FPathField->AsString;

  PVirtualNode  Last = RootNode;


//     
  DoReadPathFromDB(Path);

//  
  Temp = FKeyField->AsInteger;
  if(Path.IsEmpty())
    Path = Temp;
  else
    Path += AnsiString('.') + Temp;
//  ,      
//  ID    
  do
  {
    Node  = Last;
    Pos   = Path.Pos('.');

    if(Pos == 0)
    {
      Temp = Path;
      Pos  = Path.Length();
    }
    else
      Temp  = Path.SubString(1, Pos-1);

  // ,     
    try
    {
      ID  = Temp.ToInt();
    }
    catch(EConvertError& )
    {
      return;
      //TODO: Add my own exception: wrong tree structure
    }

  //      ID
    Last  = FindChild(Node, ID);
    if(Last)
      Path.Delete(1, Pos);
  }while(Last && !Path.IsEmpty());

//  Last == 0 (=> Path.IsEmpty() != 0),     
//   
//   Path.IsEmpty() == 0 (=> Last != 0),     . 
// ,      

  if(Path.IsEmpty())
  {
    Node = Last;
    Data = (TDBVTData* )GetNodeData(Node);
  }
  else
  {
  //    
    int Level;

    if(Node == RootNode)
      Level = -1;
    else
    {
      Data  = (TDBVTData* )GetNodeData(Node);
      Level = Data->Level;
    }
    do
    {
      Pos = Path.Pos('.');

      if(Pos == 0)
      {
        Temp = Path;
        Pos = Path.Length();
      }
      else
        Temp  = Path.SubString(1, Pos-1);

    // ,     
      try
      {
        ID  = Temp.ToInt();
      }
      catch(EConvertError& )
      {
        return;
        //TODO: Add my own exception: wrong tree structure
      }

      Node = AddChild(Node);
      ValidateNode(Node, false);

      Data = (TDBVTData* )GetNodeData(Node);

      Data->ID      = ID;
      Data->Level   = ++Level;
      Data->Status  = dbnsInited;
      Data->Parent  = 0;

      Path.Delete(1, Pos);
    }while(!Path.IsEmpty());
  }

  //    -   
  // Node -         ,  
  FMaxLevel = max(FMaxLevel, Data->Level);
  ReadNodeFromDB(Node);
}

//------------------------------------------------------------------------------
//     Parent
//
void __fastcall TBaseVirtualDBTree::RefreshNodeByParent()
{
  TDBVTData*   Data;
  PVirtualNode This;
  PVirtualNode Parent;
  PVirtualNode Temp;

  int ID       = FKeyField->AsInteger;
  int ParentID = FParentField->AsInteger;

  if(ID == 0)
  {
    return;
    //TODO: Throw exception. ID mustn't be zero
  }

  This    = 0;
  Parent  = 0;
  Temp    = GetFirst();

  //DONE:  ParentID == ID,    ,  ParentID == 0
  //     ,     
  if(ParentID == 0 || ParentID == ID)
  {
    Parent    = RootNode;
    ParentID  = 0;
  }

  //  
  while(Temp && (!This || !Parent))
  {
    Data = (TDBVTData* )GetNodeData(Temp);
    if(Data->ID == ID)
    {
      if(Data->Status == dbnsRefreshed)
      {
        return;
        //TODO: Throw error. Key field dublicated
      }
      This    = Temp;
    }
    if(Data->ID == ParentID && ParentID != 0)
      Parent  = Temp;

    Temp = GetNext(Temp);
  }

  //      ,  
  if(!Parent)
  {
    Parent = AddChild(0);
    ValidateNode(Parent, 0);

    Data          = (TDBVTData* )GetNodeData(Parent);
    Data->ID      = ParentID;
    Data->Status  = dbnsInited;
    Data->Parent  = 0;
  }

  //    
  bool Created = true;
  if(This)
  {
    //      ,  
    if(This->Parent != Parent)
    {
      //    -    
      if(HasAsParent(Parent, This))
      {
        Data = (TDBVTData* )GetNodeData(Parent);
        if(Data->Status == dbnsRefreshed)
        {
          return;
          // Trow exception. Loop in DB
        }
        Temp    = This;
        This    = Parent;
        Parent  = Temp;

        Data      = (TDBVTData* )GetNodeData(Parent);
        Data->ID  = ParentID;
      }
      else
      {
        //      
        MoveTo(This, Parent, amAddChildLast, false);
      }
    }
  }
  else
  {
    //      ,  
    Created = false;
    This = AddChild(Parent);
    ValidateNode(This, false);
  }

  //   
  Data = (TDBVTData* )GetNodeData(This);

  Data->ID      = ID;

  if(!Created)
    Data->Status  = dbnsInited;

  ReadNodeFromDB(This);
}

//------------------------------------------------------------------------------
//  
//
void __fastcall TBaseVirtualDBTree::Update()
{
  Clear();
  RefreshNodes();
}

//------------------------------------------------------------------------------
//       
//
void __fastcall TBaseVirtualDBTree::ToggleListView()
{
  if(FDBStatus.Contains(dbtsDragDrop))
    return;

  TDBVTData*   Data;
  PVirtualNode Node;
  PVirtualNode Temp;

  BeginUpdate();
  if(FDBOptions.Contains(dboListView))
  {
    //  - 
    Node = GetFirst();
    while(Node)
    {
      Data          = (TDBVTData* )GetNodeData(Node);

      if(Node->Parent != RootNode)
      {
        Data->Parent  = Node->Parent;

        Temp = GetNextSibling(Node);
        if(!Temp)
        {
          Temp = GetLastChild(Node);
          if(Temp)
          {
            Temp = GetNext(Temp);
            if(!Temp)
              Temp = GetNext(Node);
          }
          else
            Temp = GetNext(Node);
        }
        MoveTo(Node, RootNode, amAddChildLast, false);
        Node = Temp;
      }
      else
      {
        Node = GetNext(Node);
      }
    }
    if(!FDBOptions.Contains(dboViewAll))
      ToggleViewMode();
  }
  else
  {
    //  - 
    Node = GetFirst();
    while(Node)
    {
      Data = (TDBVTData* )GetNodeData(Node);
      if(Data->Parent != 0)
      {
        Temp = GetNextSibling(Node);
        MoveTo(Node, Data->Parent, amAddChildLast, false);
        Data->Parent = 0;
        Node = Temp;
      }
      else
        Node = GetNextSibling(Node);
    }

    if(!FDBOptions.Contains(dboViewAll))
    {
      FDBStatus << dbtsToggleAll;
      ToggleViewMode();
      FDBStatus >> dbtsToggleAll;
    }
  }
  if(FocusedNode)
    FullyVisible[FocusedNode] = true;

  EndUpdate();
}

//------------------------------------------------------------------------------
void __fastcall TBaseVirtualDBTree::ResetFields()
{
  FKeyField     = 0;
  FParentField  = 0;
  FPathField    = 0;
  FLevelField   = 0;
}

void __fastcall TBaseVirtualDBTree::InitFields()
{
  if(FKeyFieldName != "")
    FKeyField = FDataLink->DataSet->FieldByName(FKeyFieldName);
  if(FParentFieldName != "")
    FParentField = FDataLink->DataSet->FieldByName(FParentFieldName);
  if(FPathFieldName != "")
    FPathField = FDataLink->DataSet->FieldByName(FPathFieldName);
  if(FLevelFieldName != "")
    FLevelField = FDataLink->DataSet->FieldByName(FLevelFieldName);
}

//==============================================================================
// 
//
//------------------------------------------------------------------------------
//    
//
void __fastcall TBaseVirtualDBTree::SetDBNodeDataSize(int value)
{
  if(value != DBNodeDataSize && value >= 0)
  {
    NodeDataSize = FDBDataSize + value;
    Update();
  }
}
int __fastcall TBaseVirtualDBTree::GetDBNodeDataSize()
{
  return NodeDataSize - FDBDataSize;
}

//------------------------------------------------------------------------------
//  
//
TDBVTStatuses __fastcall TBaseVirtualDBTree::GetDBStatus()
{
  return FDBStatus;
}

TDBVTOptions __fastcall TBaseVirtualDBTree::GetDBOptions()
{
  return FDBOptions;
}

void __fastcall TBaseVirtualDBTree::SetDBOptions(TDBVTOptions value)
{
  EndEditNode();

  TDBVTOptions ToBeSet, ToBeCleared;

  ToBeSet     = value - FDBOptions;
  ToBeCleared = FDBOptions - value;

  FDBOptions = value;

  if(ToBeSet.Contains(dboTrackCursor))
  {
    if(!FDBOptions.Contains(dboTrackActive) || !FDBOptions.Contains(dboTrackChanges))
    {
      FDBOptions << dboTrackActive << dboTrackChanges;
      FDBOptions >> dboAllowChecking;
      if(FDBStatus.Contains(dbtsChanged))
        DataLinkActiveChanged();
    }
    else
      DataLinkScrolled();
  }
  else if(ToBeSet.Contains(dboTrackChanges))
  {
    if(!FDBOptions.Contains(dboTrackActive))
    {
      FDBOptions << dboTrackActive;
    }
    if(FDBStatus.Contains(dbtsChanged))
      DataLinkActiveChanged();
  }
  else if(ToBeSet.Contains(dboTrackActive))
  {
    if(FDBStatus.Contains(dbtsChanged))
      DataLinkActiveChanged();
  }
  else if(ToBeCleared.Contains(dboTrackActive))
  {
    FDBOptions >> dboTrackCursor >> dboTrackChanges;
    FDBOptions << dboReadOnly;
  }
  else if(ToBeCleared.Contains(dboTrackChanges))
  {
    FDBOptions >> dboTrackCursor;
    FDBOptions << dboReadOnly;
  }
  else if(ToBeCleared.Contains(dboTrackCursor))
  {
    FDBOptions << dboReadOnly;
  }

  if(ToBeSet.Contains(dboShowChecks))
  {
    if(FDBOptions.Contains(dboTrackCursor))
    {
      FDBOptions >> dboShowChecks;
      FDBOptions << dboViewAll;
    }
    else
    {
      BeginUpdate();
      Options = Options << voCheckSupport;
      if(!FDBOptions.Contains(dboViewAll))
      {
        ToggleViewMode();
      }
      EndUpdate();
    }
  }
  else if(ToBeCleared.Contains(dboShowChecks))
  {
    BeginUpdate();
    Options = Options >> voCheckSupport;
    if(!FDBOptions.Contains(dboViewAll))
    {
        FDBOptions << dboViewAll;
        RefreshNodes();
    }
    EndUpdate();
  }
  else if(ToBeSet.Contains(dboViewAll))
  {
    if(FDBOptions.Contains(dboShowChecks))
    {
      ToggleViewMode();
    }
  }
  else if(ToBeCleared.Contains(dboViewAll))
  {
    if(FDBOptions.Contains(dboShowChecks))
    {
      ToggleViewMode();
    }
    else
      FDBOptions << dboViewAll;
  }

  if(ToBeSet.Contains(dboPathStructure))
  {
    FDBOptions >> dboParentStructure;
    if(FDBOptions.Contains(dboTrackActive))
      Update();
  }
  else if(ToBeSet.Contains(dboParentStructure))
  {
    FDBOptions >> dboPathStructure;
    if(FDBOptions.Contains(dboTrackActive))
      Update();
  }
  else if(ToBeCleared.Contains(dboPathStructure))
  {
    FDBOptions << dboParentStructure;
    if(FDBOptions.Contains(dboTrackActive))
      Update();
  }
  else if(ToBeCleared.Contains(dboParentStructure))
  {
    FDBOptions << dboPathStructure;
    if(FDBOptions.Contains(dboTrackActive))
      Update();
  }

  if(ToBeSet.Contains(dboAlwaysStructured))
  {
    if(!FDBStatus.Contains(dbtsStructured))
      RefreshNodes();
  }
  else if(ToBeCleared.Contains(dboAlwaysStructured))
  {
    if(FDBOptions.Contains(dboShowChecks))
      FDBOptions << dboAlwaysStructured;
  }

  if(ToBeSet.Contains(dboListView))
  {
    ToggleListView();
  }
  else if(ToBeCleared.Contains(dboListView))
  {
    if(FDBStatus.Contains(dbtsStructured))
      ToggleListView();
    else
      RefreshNodes();
  }

  if(ToBeCleared.Contains(dboReadOnly) &&
    (!FDBOptions.Contains(dboTrackCursor) || !FDBOptions.Contains(dboTrackChanges) || !FDBOptions.Contains(dboTrackActive)))
      FDBOptions << dboReadOnly;
}

//==============================================================================
// 
//

//------------------------------------------------------------------------------
// ,        
//
bool __fastcall TBaseVirtualDBTree::CanOpenDataSet()
{
  //   , 
  //  -   
  //  -  ,   
  //  -   
  //
  bool Allow = (FKeyField != 0);
  if(Allow && (!FDBOptions.Contains(dboListView) || FDBOptions.Contains(dboAlwaysStructured)))
    Allow    = ((FDBOptions.Contains(dboPathStructure) && FPathField) ||
               (FDBOptions.Contains(dboParentStructure) && FParentField));
  if(Allow)
    DoOpeningDataSet(Allow);

  return Allow;
}

void __fastcall TBaseVirtualDBTree::DoOpeningDataSet(bool& Allow)
{
  if(FOnOpeningDataSet)
    FOnOpeningDataSet(this, Allow);
}

//------------------------------------------------------------------------------
void __fastcall TBaseVirtualDBTree::DoNodeDataChanged(PVirtualNode Node, TField* Field, bool& UpdateNode)
{
  if(FOnNodeDataChanged)
    FOnNodeDataChanged(this, Node, Field, UpdateNode);
}

//------------------------------------------------------------------------------
//     
//
void __fastcall TBaseVirtualDBTree::DoReadPathFromDB(AnsiString& Path)
{
  if(FOnReadPathFromDB)
    FOnReadPathFromDB(this, Path);
}

//------------------------------------------------------------------------------
//    
//
void __fastcall TBaseVirtualDBTree::DoWritePathToDB(AnsiString& Path)
{
  if(FOnWritePathToDB)
    FOnWritePathToDB(this, Path);
}

//------------------------------------------------------------------------------
//   
// :    
//  - ID
//  - Parent
//  - Data
// NB Level  !
//
void __fastcall TBaseVirtualDBTree::ReadNodeFromDB(PVirtualNode Node)
{
  TDBVTData* Data = (TDBVTData* )GetNodeData(Node);
  //     
  if(Data->Status != dbnsNone && Data->Status != dbnsRefreshed)
    DoReadNodeFromDB(Node);

  Data->Status    = dbnsRefreshed;
}

void __fastcall TBaseVirtualDBTree::DoReadNodeFromDB(PVirtualNode Node)
{
  if(FOnReadNodeFromDB)
    FOnReadNodeFromDB(this, Node);
}

//------------------------------------------------------------------------------
//     
//
bool __fastcall TBaseVirtualDBTree::CanWriteToDataSet(PVirtualNode Node, int Column, TDBVTChangeMode ChangeMode)
{
  //     
  //  -     (dboReadOnly)
  //  -    ,   , 
  //  -     (CanModify)
  //  - ,   ,  
  //  -  ,   ,    
  //
  bool Allow = !FDBOptions.Contains(dboReadOnly) &&
               FKeyField &&
               FDataLink->DataSet->CanModify;
  if(Allow)
  {
    if(DBOptions.Contains(dboListView))
    {
      if(ChangeMode == dbcmStructure || ChangeMode == dbcmInsert)
        Allow = (Node == 0 || Node == RootNode);
    }
    else if(ChangeMode == dbcmStructure || ChangeMode == dbcmInsert)
    {
      Allow = (((FDBOptions.Contains(dboPathStructure) || FDBOptions.Contains(dboWriteSecondary)) && FPathField && FPathField->CanModify) ||
              ((FDBOptions.Contains(dboParentStructure) || FDBOptions.Contains(dboWriteSecondary)) && FParentField && FParentField->CanModify));
      if(Allow && FDBOptions.Contains(dboWriteLevel))
        Allow = (FLevelField && FLevelField->CanModify);
    }
    if(Allow)
    {
      DoWritingDataSet(Node, Column, ChangeMode, Allow);
    }
  }
  return Allow;
}

void __fastcall TBaseVirtualDBTree::DoWritingDataSet(PVirtualNode Node, int Column, TDBVTChangeMode ChangeMode, bool& Allow)
{
  if(FOnWritingDataSet)
    FOnWritingDataSet(this, Node, Column, ChangeMode, Allow);
}

//------------------------------------------------------------------------------
//    
//
bool __fastcall TBaseVirtualDBTree::DoChecking(PVirtualNode Node, TCheckState NewCheckState)
{
  if(FDBStatus.Contains(dbtsDataChanging))
    return true;

  if(FDBOptions.Contains(dboShowChecks) && FDBOptions.Contains(dboAllowChecking))
    return TCustomVirtualStringTree::DoChecking(Node, NewCheckState);

  return false;
}

//------------------------------------------------------------------------------
//  
//
void __fastcall TBaseVirtualDBTree::DoChecked(PVirtualNode Node)
{
  if(!FDBStatus.Contains(dbtsDataChanging))
  {
    BeginUpdate();
    if(CheckState[Node] == csChecked)
    {
      if(FDBOptions.Contains(dboCheckChildren))
        CheckAllChildren(Node);
    }
    else if(!FDBOptions.Contains(dboViewAll))
    {
      ToggleViewMode();
    }
    if(!FDBStatus.Contains(dbtsChecking) && FDBOptions.Contains(dboPathStructure))
      RefreshNodes();

    EndUpdate();

    TCustomVirtualStringTree::DoChecked(Node);
  }
}

//------------------------------------------------------------------------------
//   ,   
//
void __fastcall TBaseVirtualDBTree::CheckAllChildren(PVirtualNode Node)
{
  if(FDBOptions.Contains(dboShowChecks) && FDBOptions.Contains(dboAllowChecking))
  {
    FDBStatus << dbtsChecking;
    Node = GetFirstChild(Node);
    while(Node)
    {
      CheckState[Node] = csChecked;
      Node = GetNextSibling(Node);
    }
    FDBStatus >> dbtsChecking;
  }
}

//------------------------------------------------------------------------------
//     ,  
//
void __fastcall TBaseVirtualDBTree::UnCheckAll(PVirtualNode Node, bool OnlyChildren)
{
  if(FDBOptions.Contains(dboShowChecks) && FDBOptions.Contains(dboAllowChecking))
  {
    bool Changed      = false;

    PVirtualNode Last = GetNextSibling(Node);
    if(!Last)
    {
      Last = GetLastChild(Node);
      if(!Last)
        Last = Node;
      Last = GetNext(Last);
    }

    if(OnlyChildren)
      Node = GetNext(Node);

    while(Node != Last)
    {
      if(CheckState[Node] != csUnchecked)
      {
        CheckState[Node] = csUnchecked;
        if(CheckState[Node] == csUnchecked)
          Changed = true;
      }
    }
    if(Changed)
      ToggleViewMode();
  }
}

//------------------------------------------------------------------------------
//    
//

void __fastcall TBaseVirtualDBTree::DoInitNode(PVirtualNode Parent, PVirtualNode Node, TVirtualNodeInitStates& InitStates)
{
  TCustomVirtualStringTree::DoInitNode(Parent, Node, InitStates);

  Node->CheckType   = ctCheckBox;
  Node->CheckState  = csUnchecked;
}

//------------------------------------------------------------------------------
//   
//
void __fastcall TBaseVirtualDBTree::ToggleViewMode()
{
  if(!FDBStatus.Contains(dbtsDataChanging) && FDBOptions.Contains(dboShowChecks))
  {
    PVirtualNode Node;

    BeginUpdate();
    if(FDBOptions.Contains(dboViewAll))
    {
      RefreshNodes();
    }
    else
    {
      if(FDBStatus.Contains(dbtsToggleAll))
        RefreshNodes();
      Node = GetLastChild(RootNode);
      while(Node && Node != RootNode)
      {
        if(CheckState[Node] != csChecked && !GetFirstChild(Node))
        {
          DeleteNode(Node);
          if(FDBOptions.Contains(dboListView))
            FDBStatus >> dbtsStructured;
        }
        Node = GetPrevious(Node);
      }
    }
    EndUpdate();
  }
}

bool __fastcall TBaseVirtualDBTree::HasVisibleChildren(PVirtualNode Node)
{
  if(!Node)
    return false;

  PVirtualNode Last = GetNextSibling(Node);
  if(!Last)
  {
    Last = GetLastChild(Node);
    if(!Last)
      Last = Node;
    Last = GetNext(Last);
  }
  Node = GetNext(Node);
  while(Node != Last)
  {
    if(IsVisible[Node])
      return true;
    Node = GetNext(Node);
  }
  return false;
}

void __fastcall TBaseVirtualDBTree::ExpandAll()
{
  PVirtualNode Node = GetFirst();

  BeginUpdate();
  while(Node)
  {
    Expanded[Node] = true;
    Node = GetNext(Node);
  }
  EndUpdate();
}

void __fastcall TBaseVirtualDBTree::CollapseAll()
{
  PVirtualNode Node = GetFirst();

  BeginUpdate();
  while(Node)
  {
    Expanded[Node] = false;
    Node = GetNext(Node);
  }
  EndUpdate();
}
