//---------------------------------------------------------------------------

#ifndef BaseVirtualDBTreeH
#define BaseVirtualDBTreeH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <DB.hpp>

#include "VirtualTrees.hpp"

//==============================================================================
//  TBaseVirtualDBTree
// ------------------------
//        
//  -      Parent    Path,
//  -    ,
//  -  ,   , ,
//  -  ,
//  -  ,
//  -    ID   ,
//  -    (Level, Path, Parent),
//  TODO:
//  -     ,
//  -       ,
//  -        
//  -     
//  -     
//
//------------------------------------------------------------------------------
//  :
//
#pragma option push -b-
enum TDBVTOption
{
  //  
  dboCheckDBStructure,      //        
  dboParentStructure,       //    Parent
  dboPathStructure,         //    Path

  //  
  dboWriteLevel,            //        
  dboWriteSecondary,        //      (Parent & Path)    ?

  //   
  dboAllowStructureChange,  //  , 

  //    
  dboReadOnly,              // *-  / 
  dboTrackActive,           //  -  / 
  dboTrackChanges,          //  -  ,
  dboTrackCursor,           //  -         
  //    
  //1 -      ,   
  //dboTrackActive, dboTrackChanges, dboTrackCursor

  //2 -   ,    
  //dboReadOnly, dboTrackActive, dboTrackChanges

  //3 -    /  
  //dboReadOnly, dboTrackActive

  //4 -     
  //dboReadOnly -   

  // 
  dboAlwaysStructured,      //       (  )
  //     
  dboShowChecks,            //    
  dboAllowChecking,         //  
  dboViewAll,               //  ,    
  dboCheckChildren,         //        

  dboListView               //  
};
#pragma option pop

typedef Set<TDBVTOption, dboCheckDBStructure, dboListView> TDBVTOptions;

//------------------------------------------------------------------------------
//  
//  -   
#pragma option push -b-
enum TDBVTStatus
{
  dbtsDataChanging,       //      
  dbtsDataOpening,        //      
  dbtsDragDrop,           //   DragDrop
  dbtsEditing,            //       
  dbtsInsert,             //    
  dbtsStructured,         //       
  dbtsChanged,            //  ,    
  dbtsChecking,
  dbtsToggleAll,          //  VisibleState   
  dbtsEmpty               //      
};
#pragma option pop

typedef Set<TDBVTStatus, dbtsDataChanging, dbtsEmpty> TDBVTStatuses;

//------------------------------------------------------------------------------
//      
// ( OnWritingDataSet)
//
#pragma option push -b-
enum TDBVTChangeMode
{
  dbcmEdit,       //  
  dbcmInsert,     //  
  dbcmStructure   //  / 
};
#pragma option pop

//------------------------------------------------------------------------------
//    
//  (    GoTo)
//
#pragma option push -b-
enum TDBVTGoToMode
{
  gtmFromFirst,  //    
  gtmNext,       //   
  gtmPrev        //   
};
#pragma option pop

//------------------------------------------------------------------------------
//       
//
#pragma option push -b-
enum TDBVTNodeStatus
{
  dbnsNone,            //   ,       
  dbnsInited,          //          
  dbnsRefreshed,       //    
  dbnsEdit,            //   
  dbnsNew,             //      ( )
  dbnsDelete           //  ,   ,   
};
#pragma option pop

//------------------------------------------------------------------------------
//  
//   (  )    ,
//   ( DBNodeDataSize)
//
#pragma packed(push, 1)
struct TDBVTData                 //  32 
{
  int             ID;            //    ; 0,      
  int             Level;         //   (   )
  TDBVTNodeStatus Status;        //     
  PVirtualNode    Parent;        //    
  int             Reserved2;
  int             Reserved3;
  int             Reserved4;
  int             Reserved5;
};
#pragma packed(pop)

//------------------------------------------------------------------------------
//  
//
class PACKAGE TBaseVirtualDBTree;
//
//    ,    ,  
//      
//.    ID, Parent. Level 
//        Path.  ID  0,
//     (  )
//
typedef void __fastcall (__closure *TVTNodeFromDBEvent)
  (TBaseVirtualDBTree* Sender, PVirtualNode Node);

//
//      
//     
//
typedef void __fastcall (__closure *TVTDBOpenQueryEvent)
  (TBaseVirtualDBTree* Sender, bool& Allow);
//
//     ,    .
//       
//
typedef void __fastcall (__closure *TVTDBWriteQueryEvent)
  (TBaseVirtualDBTree* Sender, PVirtualNode Node, int Column, TDBVTChangeMode ChangeMode, bool& Allow);

//
//       Path.
//  ,      
//
//
typedef void __fastcall (__closure *TVTPathToDBEvent)
  (TBaseVirtualDBTree* Sender, AnsiString& Path);

//
// ,       
//   ,     
//UpdateNode  true.
//
typedef void __fastcall (__closure *TVTNodeDataChangedEvent)
  (TBaseVirtualDBTree* Sender, PVirtualNode Node, TField* Field, bool& UpdateNode);

//------------------------------------------------------------------------------
//  
//
class PACKAGE TVTDataLink;

class PACKAGE TBaseVirtualDBTree : public TCustomVirtualStringTree
{
  friend
    class TVTDataLink;

  private:
    //    
    TVTDataLink*  FDataLink;

    //  
    AnsiString    FKeyFieldName;
    AnsiString    FParentFieldName;
    AnsiString    FLevelFieldName;
    AnsiString    FPathFieldName;
    // 
    TField*       FKeyField;
    TField*       FParentField;
    TField*       FLevelField;
    TField*       FPathField;

    TDBVTStatuses FDBStatus;     //    
    TDBVTOptions  FDBOptions;    //  

    int           FDBDataSize;   //   

  //  
    TVTNodeFromDBEvent      FOnReadNodeFromDB;
    TVTDBOpenQueryEvent     FOnOpeningDataSet;
    TVTDBWriteQueryEvent    FOnWritingDataSet;
    TVTPathToDBEvent        FOnReadPathFromDB;
    TVTPathToDBEvent        FOnWritePathToDB;
    TVTNodeDataChangedEvent FOnNodeDataChanged;

  //  
    int FMaxLevel;
    int FCurID;

  private:
  //   
    void          __fastcall SetDataSource(TDataSource* value);
    TDataSource*  __fastcall GetDataSource();

  //   
    void __fastcall SetKeyFieldName(AnsiString value);
    void __fastcall SetParentFieldName(AnsiString value);
    void __fastcall SetLevelFieldName(AnsiString value);
    void __fastcall SetPathFieldName(AnsiString value);

  //  
    void __fastcall SetDBNodeDataSize(int value);
    int  __fastcall GetDBNodeDataSize();

  //   
    void __fastcall SetDBOptions(TDBVTOptions value);
    TDBVTOptions  __fastcall GetDBOptions();
    TDBVTStatuses __fastcall GetDBStatus();

  //   
    void __fastcall RefreshNodeByParent();
    void __fastcall RefreshNodeByPath();
    void __fastcall RefreshListNode();

  protected:
  //   
    void __fastcall RefreshNodes();

  //     0,     
    virtual void    __fastcall Notification(TComponent* AComponent, TOperation Operation);

  //    
    virtual void __fastcall DataLinkActiveChanged();
    virtual void __fastcall DataLinkScrolled();
    virtual void __fastcall DataLinkChanged();
    virtual void __fastcall DataLinkRecordChanged(TField* Field);
    virtual void __fastcall DataLinkEditingChanged();
  //   
    void __fastcall ToggleListView();
    void __fastcall ToggleViewMode();

    virtual void __fastcall InitFields();
    virtual void __fastcall ResetFields();
  //    
    virtual void    __fastcall DoFocusChange(PVirtualNode Node, int Column);

    virtual bool    __fastcall CanEdit(PVirtualNode Node, int Column);
    DYNAMIC void    __fastcall DoEdit();
    DYNAMIC void    __fastcall DoEndEdit();
    DYNAMIC void    __fastcall DoCancelEdit();

    DYNAMIC void    __fastcall DoCollapsed(PVirtualNode Node);

    virtual void    __fastcall DoFreeNode(PVirtualNode Node);
    DYNAMIC void    __fastcall DoNodeMoved(PVirtualNode Node);

    virtual HRESULT __fastcall DoDragDrop(const _di_IDataObject DataObject, int KeyState, const TPoint& Pt, int &Effect);

  //  
    virtual void __fastcall ReadNodeFromDB(PVirtualNode Node);
    virtual void __fastcall DoReadNodeFromDB(PVirtualNode Node);

    virtual bool __fastcall CanOpenDataSet();
    virtual void __fastcall DoOpeningDataSet(bool& Allow);

    virtual bool __fastcall CanWriteToDataSet(PVirtualNode Node, int Column, TDBVTChangeMode ChangeMode);
    virtual void __fastcall DoWritingDataSet(PVirtualNode Node, int Column, TDBVTChangeMode ChangeMode, bool& Allow);

    virtual void __fastcall DoReadPathFromDB(AnsiString& Path);
    virtual void __fastcall DoWritePathToDB(AnsiString& Path);

    virtual void __fastcall DoNodeDataChanged(PVirtualNode Node, TField* Field, bool& UpdateNode);

  //  
    //  
    void            __fastcall SetFocusToNode(PVirtualNode Node);
    //      
    PVirtualNode    __fastcall FindChild(PVirtualNode Node, int ID);
    //    ,   
    PVirtualNode    __fastcall FindNode(PVirtualNode Node, int ID);
    //    
    virtual void __fastcall RefreshNode();

    DYNAMIC bool __fastcall DoChecking(PVirtualNode Node, TCheckState NewCheckState);
    DYNAMIC void __fastcall DoChecked(PVirtualNode Node);

    virtual void __fastcall DoInitNode(PVirtualNode Parent, PVirtualNode Node, TVirtualNodeInitStates& InitStates);
    bool __fastcall HasVisibleChildren(PVirtualNode Node);



  public:

    __property TDBVTStatuses DBStatus = { read = GetDBStatus };


    //   
    __fastcall TBaseVirtualDBTree(TComponent* Owner);
    virtual __fastcall ~TBaseVirtualDBTree();


    void __fastcall OnDragOverHandler(TBaseVirtualTree *Sender, TObject *Source,
        TShiftState Shift, TDragState State, const TPoint &Pt, TDropMode Mode, int &Effect, bool &Accept);

  public:
    void __fastcall ExpandAll();
    void __fastcall CollapseAll();

    void __fastcall GoTo(WideString AString, TDBVTGoToMode Mode);
    bool __fastcall GoTo(int ID);

    void __fastcall DeleteSelection();
    void __fastcall AddNode(PVirtualNode Parent);
    void __fastcall Update();

    void* __fastcall GetDBNodeData(PVirtualNode Node);

    void __fastcall CheckAllChildren(PVirtualNode Node);
    void __fastcall UnCheckAll(PVirtualNode Node, bool OnlyChildren);


    //  
    __property TDataSource* DataSource  = { read=GetDataSource, write=SetDataSource };

    //   
    __property AnsiString KeyFieldName      = { read=FKeyFieldName,     write=SetKeyFieldName };
    __property AnsiString ParentFieldName   = { read=FParentFieldName,  write=SetParentFieldName };
    __property AnsiString LevelFieldName    = { read=FLevelFieldName,   write=SetLevelFieldName };
    __property AnsiString PathFieldName     = { read=FPathFieldName,    write=SetPathFieldName };

    //   ( )
    __property TField* KeyField     = { read=FKeyField };
    __property TField* ParentField  = { read=FParentField };
    __property TField* LevelField   = { read=FLevelField };
    __property TField* PathField    = { read=FPathField };

    //  
    __property TVTNodeFromDBEvent       OnReadNodeFromDB  = { read=FOnReadNodeFromDB,   write=FOnReadNodeFromDB };
    __property TVTDBOpenQueryEvent      OnOpeningDataSet  = { read=FOnOpeningDataSet,   write=FOnOpeningDataSet };
    __property TVTDBWriteQueryEvent     OnWritingDataSet  = { read=FOnWritingDataSet,   write=FOnWritingDataSet };
    __property TVTPathToDBEvent         OnReadPathFromDB  = { read=FOnReadPathFromDB,   write=FOnReadPathFromDB };
    __property TVTPathToDBEvent         OnWritePathToDB   = { read=FOnWritePathToDB,    write=FOnWritePathToDB };
    __property TVTNodeDataChangedEvent  OnNodeDataChanged = { read=FOnNodeDataChanged,  write=FOnNodeDataChanged };

    //    
    __property int          DBNodeDataSize  = { read=GetDBNodeDataSize, write=SetDBNodeDataSize };
    //  
    __property TDBVTOptions DBOptions       = { read=GetDBOptions,      write=SetDBOptions };

};
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//     
//

class PACKAGE TVTDataLink : public TDataLink
{
  private:
    TBaseVirtualDBTree* FTree;     // 

  public:
    __fastcall TVTDataLink(TBaseVirtualDBTree* ATree);

  protected:
  //  
    virtual void __fastcall ActiveChanged();
    virtual void __fastcall DataSetScrolled(int Distance);
    virtual void __fastcall DataSetChanged();
    virtual void __fastcall RecordChanged(TField* Field);
    virtual void __fastcall EditingChanged();
};

#endif
