{                                                                              }
{ TIB_VTree                                                                    }
{                                                                              }
{Written by Paul Gallagher paul@tansit.com                                     }
{Thanks to John Tomaseli jrt@gtz.com and Konstantin mrcat@mrcat.org            }
{for their help simplifying the installation                                   }

{This component is free to all who want it. But, you must obtain               }
{proper licenses for Interbase Objects www.ibobjects.com and                   }
{for Virtual Tree www.lischke-online.de/VirtualTreeview.html                   }

{This control adds IBO database awareness to the Virtual Tree Control.         }

{The data table must be structured.					       }

{KeyField is an integer field, that gives   				       }
{each node a distinct value. The table MUST be ordered ascending on KeyField.  }

{ParentField is the parent of the record.                                      }
{The first record in the dataset should have a parent of -1                    }
{Until a better way is determined, maximum columns in the tree is 11           }
{Also, use the Tag property of the column to designate the column index        }
{of the corresponding Table column.                                            }

{FolderField should be setup as Boolean. This lets IB_VTree know if the node is}
{is a folder or not. When folder constraints are in place, it is not possible  }
{to drag nodes into non folders.                                               }
{FolderConstraints tells IB_VTree that nodes can only be dragdropped into      }
{folders. Also, child nodes can only be created under a Folder.                }
{FolderImageIndex is the ImageIndex for the closed folder. The next sequential }
{Index would be for the Open Folder.                                           }

{ImageIndexField is the column that will be used for assigning images. If it   }
{nil, then Images will not be automatically assigned                           }

{ODER BY: For most efficient loading, Order by FOLDERFIELD DESCENDING,         }
{PARENTFIELD, KEYFIELD. This will reduce the number of table traverses         }

unit IB_VTree;

interface

uses
 Classes, Controls, Messages, DsgnIntf, VirtualTrees,
 IB_Header, IB_Constants, IB_Components, IB_Editors,
 IB_FieldDataLink, SysUtils, ActiveX, ExtCtrls, ComCtrls, CommCtrl,
 StdCtrls, Buttons, Windows, Forms, Graphics, Menus, ImgList;

{$DEFINE IB_FieldControl}
{$DEFINE IB_UserClass}
{$DEFINE IBC_VTree}

procedure Register;

type

TIB_CustomVTree = class(TVirtualStringTree)

{$I IB_Control.INT}

private
  FIB_ParentFieldDataLink: TIB_FieldDataLink;
  FParentField : string;
  FKeyField : string;
  FDisplayField : string;
  FFolderField : string;
  FImageIndexField : string;
  FFolderConstraints : boolean;
  FFolderImageIndex : integer;
  procedure RecordInserted; //called to update tree when a new record is added
  procedure RecordDeleted;
  procedure RecordEdited;
  procedure DatasetClosed;
  procedure DatasetOpened;
  function AddRootNode: boolean;
  function AddChildNode: boolean;
  function FindNode(AID: integer): PVirtualNode;


protected
  procedure DoGetText(Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
      var Text: WideString); override;
  procedure DoInitNode(Parent, Node: PVirtualNode;
    var InitStates: TVirtualNodeInitStates); override;
  procedure DoChange(Node: PVirtualNode); override;
  function DoDragDrop(const DataObject: IDataObject; KeyState: Integer; Pt: TPoint;
    var Effect: Integer): HResult; override;
  function DoGetImageIndex(Node: PVirtualNode; Kind: TVTImageKind;
    Column: Integer): Integer; override;
  procedure DoNewText(Node: PVirtualNode; Column: Integer; Text: WideString); override;

public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure LoadData;
  procedure AddDataNode(Node: PVirtualNode; AIsFolder: boolean);

published
  property ParentField: string read FParentField write FParentField;
  property KeyField: string read FKeyField write FKeyField;
  property DisplayField: string read FDisplayField write FDisplayField;
  property FolderField: string read FFolderField write FFolderField;
  property ImageIndexField: string read FImageIndexField write FImageIndexField;
  property FolderConstraints: boolean read FFolderConstraints write FFolderConstraints;
  property FolderImageIndex: integer read FFolderImageIndex write FFolderImageIndex default -1;


end;

  TIB_VTree = class(TIB_CustomVTree)


published
  property DataSource;


end;

implementation

{$I IB_Control.IMP}

var
  CurrentRootID, EditedRecord, DeletedRecord : integer;
  LastNodeAdded: PVirtualNode;
  PreviousRecord, InsertedRecord: string;
  PriorState: TIB_DatasetState;
  PreviousNode: PVirtualNode;
  TreeInEdit, DatasetInEdit: boolean;


type
  PVTreeRec = ^TVTreeRec;
  TVTreeRec = record
    Caption: WideString;
    ID: integer;
    Parent: integer;
    Image: integer;
    BookMark: string;
    Folder: boolean;
    ColumnText: array of string;
end;

type
  PRecList = ^ARecord;
  ARecord = record
    Loaded: boolean;
end;

constructor TIB_CustomVTree.Create(AOwner: TComponent);
begin
  FIB_FieldDataLink := TIB_FieldDataLink.Create( Self );
  FIB_ParentFieldDataLink := TIB_FieldDataLink.Create( Self );
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  inherited AutoSize := false;
  DataLink.OnDataChange := DataChange;
  DataLink.OnStateChanged := StateChanged;

end;

destructor TIB_CustomVTree.Destroy;
begin
  DataSource := nil;
  DataLink.OnDataChange := nil;
  DataLink.OnStateChanged := nil;
  inherited Destroy;
end;

function TIB_CustomVTree.DoGetImageIndex(Node: PVirtualNode; Kind: TVTImageKind;
    Column: Integer): Integer;
var
  NodeData: PVTreeRec;
begin
  Result := -1;
  if column < 1 then
  begin
    NodeData := GetNodeData(Node);
    if assigned(NodeData) then
    begin
      DataLink.Dataset.BufferBookmark := NodeData.BookMark;//sync buffer to node
      if (not NodeData.Folder) then Result := NodeData.Image else
        begin
          if Expanded[Node] and Selected[Node] then Result := FolderImageIndex + 1 else
          if (not HasChildren[Node]) and Selected[Node] then Result := FolderImageIndex + 1 else
          Result := FolderImageIndex;
        end;
    end;
  end;
end;

function TIB_CustomVTree.DoDragDrop(const DataObject: IDataObject; KeyState: Integer; Pt: TPoint;
      var Effect: Integer): HResult;
var
  SourceData, DestData: PVTreeRec;
  Attachmode: TVTNodeAttachMode;
begin
  SourceData := GetNodeData(FocusedNode);
  DestData := GetNodeData(DropTargetNode);
  if assigned(DestData) then
  begin
    if (not DestData.Folder) and FolderConstraints then
      begin
        Result := inherited DoDragDrop(DataObject, KeyState, Pt, Effect);
        exit;
      end;
  end;
  DataLink.Dataset.Bookmark := SourceData.BookMark;
  if assigned(DestData) then DataLink.Dataset.FieldByName(ParentField).AsInteger := DestData.ID
    else DataLink.Dataset.FieldByName(ParentField).AsInteger := -1;
  DataLink.Dataset.Post;
  Effect := DROPEFFECT_MOVE;
  AttachMode := amAddChildLast;
  ProcessDrop(DataObject, DropTargetNode, Effect, AttachMode);
  Effect := DROPEFFECT_LINK;
  Result := inherited DoDragDrop(DataObject, KeyState, Pt, Effect);
end;

function TIB_CustomVTree.FindNode(AID: integer): PVirtualNode;
var
NodeData: PVTreeRec;
Temp: PVirtualNode;
i: cardinal;
begin
  Result := nil;
  Temp := TopNode;
  i := 1;
  while i <= TotalCount do
  begin
    NodeData := GetNodeData(Temp);
    if NodeData.ID = AID then
    begin
      Result := Temp;
      exit;
    end;
    Temp := GetNext(Temp);
    inc(i);
  end;
end;


function TIB_CustomVTree.AddRootNode: boolean;
begin
  Result := false;
  LastNodeAdded := AddChild(RootNode);
  if assigned(LastNodeAdded) then Result := true;
  CurrentRootID := DataLink.DataSet.BufferFieldByName(KeyField).asinteger;
  ValidateNode(LastNodeAdded, true);
end;

function TIB_CustomVTree.AddChildNode: boolean;
var
CurrentNode: PVirtualNode;
i: integer;
begin
  Result := false;
  i := DataLink.DataSet.BufferFieldByName(ParentField).asinteger;
  CurrentNode := FindNode(i);
  if assigned(CurrentNode) then
  begin
    Result := true;
    AddChild(CurrentNode);
    ValidateNode(CurrentNode, true);
  end;
end;

procedure TIB_CustomVTree.AddDataNode(Node: PVirtualNode; AIsFolder: boolean);
var
NodeData:PVTreeRec;
begin
  DataLink.Dataset.Insert;
  If assigned(Node) then
  begin
    if assigned(FocusedNode) then Selected[FocusedNode] := false;
    NodeData := GetNodeData(Node);
    DataLink.Dataset.FieldByName(ParentField).asinteger := NodeData.ID;
  end else Datalink.Dataset.FieldByName(ParentField).asinteger := -1;
  DataLink.Dataset.FieldByName(FolderField).asboolean := AIsFolder;
  if AIsFolder then Datalink.Dataset.FieldByName(DisplayField).asstring := 'New Folder'
    else DataLink.Dataset.FieldByName(DisplayField).asstring := 'New Item';
  Datalink.Dataset.Post;
  EditNode(LastNodeAdded, -1);
end;

procedure TIB_CustomVTree.LoadData;
var
  RecCount: integer;
  NodeFound: boolean;
  FirstPass: boolean;
  CheckList: TStringList;
begin
  if (DataLink.Dataset.State = dssPrepared) or (DataLink.Dataset.State = dssInactive)
    then exit; //Don't build if dataset is not active;
  CheckList := TStringList.Create;
  FirstPass := true;
  RecCount := DataLink.DataSet.RecordCount;
  TreeInEdit := false;
  DatasetInEdit := false;
  NodeDataSize := SizeOf(TVTreeRec);
  RootNodeCount := 0;
  while RecCount > 0 do
  begin
  DataLink.DataSet.BufferFirst;
  NodeFound := false;
  while not DataLink.DataSet.BufferEOF do
    begin
      if DataLink.Dataset.BufferFieldByName(KeyField).asinteger = 12 then
      beep(1,1);
      if FirstPass then CheckList.Add('F');
      if CheckList.Strings[DataLink.Dataset.BufferRowNum - 1] = 'F' then
      begin
        if DataLink.Dataset.BufferFieldByName(ParentField).asinteger = -1 then
        begin // is a root node
          if AddRootNode then
          begin //RootNode Succesfully added
            dec(RecCount);
            CheckList.Strings[DataLink.Dataset.BufferRowNum - 1] := 'T';
            NodeFound := true;
          end;
        end else //is not a root node
        begin
          if AddChildNode then
          begin  //Parent found, and child node added
            dec(RecCount);
            CheckList.Strings[DataLink.Dataset.BufferRowNum - 1] := 'T';
            NodeFound := true;
          end;
        end;
      end;
      DataLink.DataSet.BufferNext;
    end;
    Firstpass := false;
    if not NodeFound then RecCount := 0; // this gets out of loop is a node cannot be found.
  end;
end;

procedure TIB_CustomVTree.DoInitNode(Parent, Node: PVirtualNode; var InitStates: TVirtualNodeInitStates);
var
  NodeData: PVTreeRec;
  i,ii,j: integer;
  s: string;
begin
  inherited;
  NodeData := GetNodeData(Node);
  NodeData.Caption := DataLink.Dataset.BufferFieldByName(DisplayField).asstring;
  NodeData.ID      := DataLink.Dataset.BufferFieldByName(KeyField).asinteger;
  NodeData.Parent  := DataLink.Dataset.BufferFieldByName(ParentField).asinteger;
  NodeData.Image   := DataLink.Dataset.BufferFieldByName(ImageIndexField).asinteger;
  if FolderConstraints then
    NodeData.Folder  := DataLink.Dataset.BufferFieldByName(FolderField).asboolean;
  NodeData.BookMark:= DataLink.Dataset.BufferBookmark;

  i := Header.Columns.Count;
  SetLength(NodeData.ColumnText, Header.Columns.Count);
  For ii := 0 to i - 1 do
  begin
    j := Header.Columns[ii].Tag;
    if J <= 10 then s := DataLink.Dataset.BufferFields[j].asstring else s :='';
    NodeData.ColumnText[ii] := s;
  end;
end;

procedure TIB_CustomVTree.DoChange(Node: PVirtualNode);
var
  NodeData: PVTreeRec;
  CB: integer;
begin
  if focused then //only do this if tree is focused. Prevents recursion
  begin
    NodeData := GetNodeData(Node);
    if Assigned(NodeData) then
    begin
      CB := Datalink.Dataset.CallbackInc;
      //Changing callback fixes a glitch that causes IBO to give up on finding
      //the bookmark too early. This is probably not a good fix, but it works.
      DataLink.Dataset.CallbackInc := -1;
      DataLink.DataSet.Bookmark := NodeData.BookMark;
      DataLink.Dataset.CallbackInc := CB;
    end;
  end;
end;

procedure TIB_CustomVTree.DoGetText(Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
      var Text: WideString);
var
  NodeData : PVTreeRec;
begin
  inherited;
  NodeData := GetNodeData(Node);
  if Assigned(NodeData) then
  begin
  if Column >= 0 then text := NodeData.ColumnText[Column] else
    text := NodeData.Caption;
  end;
end;

procedure TIB_CustomVTree.DoNewText(Node: PVirtualNode; Column: Integer; Text: WideString);
var
NodeData: PVTreeRec;
begin
  TreeInEdit := true;
  NodeData := GetNodeData(Node);
  NodeData.Caption := Text;
  DataLink.Dataset.FieldByName(DisplayField).asstring := Text;
  DataLink.Dataset.post;
  TreeInEdit := false;
end;

procedure TIB_CustomVTree.SysDataChange;
var
  CurrentNode: PVirtualNode;
begin
  if (not focused) and (not TreeInEdit) then //do this only if Tree is not the sender
  begin
    if not ((Datalink.Dataset.Bookmark = 'FFFF0000') or (DataLink.Dataset.Bookmark = '')) then
    PreviousRecord := Datalink.Dataset.Bookmark;
    CurrentNode := FindNode(DataLink.DataSet.FieldByName(KeyField).asinteger);
    if assigned(CurrentNode) then
    begin
      Selected[CurrentNode] := true;
      PreviousNode := CurrentNode;
    end;
  end;
end;

procedure TIB_CustomVTree.DatasetClosed;
begin
  RootNodeCount := 0;
  PriorState := DataLink.Dataset.State;
end;

procedure TIB_CustomVTree.DatasetOpened;
begin
  LoadData;
  PriorState := DataLink.Dataset.State;
end;

procedure TIB_CustomVTree.RecordEdited;
var
NodeData: PVTreeRec;
begin
  if assigned(PreviousNode) then
  begin
    NodeData := GetNodeData(PreviousNode);
    DataLink.Dataset.BufferBookmark := PreviousRecord;
    NodeData.Caption := DataLink.Dataset.BufferFieldByName(DisplayField).asstring;
    NodeData.Folder := DataLink.Dataset.BufferFieldByName(FolderField).asboolean;
    NodeData.BookMark := DataLink.Dataset.BufferBookmark;
    NodeData.ID := DataLink.Dataset.BufferFieldByName(KeyField).asinteger;
    NodeData.Image := DataLink.Dataset.BufferFieldByName(ImageIndexField).asinteger;
    NodeData.Parent := DataLink.Dataset.BufferFieldByName(ParentField).asinteger;
  end;
end;

procedure TIB_CustomVTree.RecordInserted;
var
  ParentNode: PVirtualNode;
  ParentID: integer;
begin
  if DataLink.Dataset.Bookmark <> PreviousRecord then exit;//Insert was cancelled
  DataLink.Dataset.BufferBookmark := DataLink.Dataset.Bookmark;
  ParentID := DataLink.Dataset.BufferFieldByName(ParentField).asinteger;
  ParentNode := FindNode(ParentID);
  LastNodeAdded := AddChild(ParentNode);
end;

procedure TIB_CustomVTree.RecordDeleted;
begin
  DeleteNode(FindNode(DeletedRecord));
end;

procedure TIB_CustomVTree.SysStateChanged;
begin
  if DataLink.Dataset.State = dssDelete then
    DeletedRecord := DataLink.Dataset.FieldByName(KeyField).asinteger;
  if DataLink.Dataset.State = dssInsert then InsertedRecord := DataLink.Dataset.Bookmark;
  if DataLink.Dataset.State = dssEdit then
    EditedRecord := DataLink.Dataset.FieldByName(KeyField).asinteger;
  if (DataLink.Dataset.State = dssInactive) or (DataLink.Dataset.State = dssPrepared) then DatasetClosed
  else if (PriorState = dssPrepared) or (PriorState = dssInactive) then DatasetOpened
  else if (PriorState = dssInsert) and (DataLink.Dataset.IsPosting) then RecordInserted
  else if PriorState = dssDelete then RecordDeleted
  else if (not focused) and (PriorState = dssEdit) then RecordEdited;
  PriorState := DataLink.Dataset.State;
end;

procedure Register;
begin
 RegisterPropertyEditor(TypeInfo(string),TIB_VTree,'ParentField',TDataFieldProperty);
 RegisterPropertyEditor(TypeInfo(string),TIB_VTree,'KeyField',TDataFieldProperty);
 RegisterPropertyEditor(TypeInfo(string),TIB_VTree,'DisplayField',TDataFieldProperty);
 RegisterPropertyEditor(TypeInfo(string),TIB_VTree,'FolderField',TDataFieldProperty);
 RegisterPropertyEditor(TypeInfo(string),TIB_VTree,'ImageIndexField',TDataFieldProperty);
 RegisterDataFieldPropertyEditor(TIB_VTree);
 RegisterComponents('iboComponents', [TIB_VTree]);
end;

end.