unit VTHeaderPopup;

//----------------------------------------------------------------------------------------------------------------------
// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License at
// http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
// License for the specific language governing rights and limitations
// under the License.
//
// The Original Code is VTHeaderPopup.pas.
//
// The Initial Developer of the Original Code is Ralf Junker <delphi@zeitungsjunge.de>. All Rights Reserved.
//----------------------------------------------------------------------------------------------------------------------

{$I Compilers.inc}

interface

uses
  Menus, VirtualTrees;

type
  TVTHeaderPopupMenu = class(TPopupMenu)
  protected
    procedure OnMenuItemClick(Sender: TObject);
  public
    procedure Popup(x, y: Integer); override;
  end;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  Classes;

type
  TVirtualTreeCast = class(TBaseVirtualTree); // Necessary to make the header accessible.

//----------------- TVTHeaderPopupMenu ---------------------------------------------------------------------------------

procedure TVTHeaderPopupMenu.OnMenuItemClick(Sender: TObject);

var
  I: Integer;

begin
  if PopupComponent = nil then
    Exit;
  I := Items.IndexOf(TMenuItem(Sender));
  if I < 0 then
    Exit;
  with TVirtualTreeCast(PopupComponent).Header.Columns.Items[I] do
    if TMenuItem(Sender).Checked then
      Options := Options - [coVisible]
    else
      Options := Options + [coVisible];
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TVTHeaderPopupMenu.Popup(x, y: Integer);

var
  I: Integer;
  NewMenuItem: TMenuItem;

begin
  if PopupComponent = nil then
    Exit;

  // Delete existing menu items.
  I := Items.Count;
  while I > 0 do
    begin
      Dec(I);
      Items.Delete(I);
    end;

  // Add column menu items.
  with TVirtualTreeCast(PopupComponent).Header do
  begin
    if hoShowImages in Options then
      Self.Images := Images;
    for I := 0 to Columns.Count - 1 do
      with Columns[I] do
      begin
        NewMenuItem := TMenuItem.Create(Self);
        NewMenuItem.Caption := Text;
        NewMenuItem.Hint := Hint;
        NewMenuItem.ImageIndex := ImageIndex;
        NewMenuItem.Checked := coVisible in Options;
        NewMenuItem.OnClick := OnMenuItemClick;
        Items.Add(NewMenuItem);
      end;
  end;

  inherited;
end;

//----------------------------------------------------------------------------------------------------------------------

end.

