//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "PropertyEditorMain.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "VirtualTrees"
#pragma resource "*.dfm"
TPropertyEditorForm *PropertyEditorForm;

__fastcall TPropertyEditorForm::TPropertyEditorForm(TComponent* Owner) : TForm(Owner) {
  Tree->NodeDataSize = sizeof(TPropertyData);
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::FormDestroy(TObject *Sender) {
   //
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::FormShow(TObject *Sender) {
	AllowEdit->Checked = true;
   Tree->Clear();
   Tree->BeginUpdate();
   AddString(NULL, "Additional switch", "Light switch|inactiv", ptCombo);
   AddString(NULL, "Lighting",  "Function", ptNone);
   AddString(NULL, "Switch 1", "On|Off", ptCombo);
   AddString(NULL, "Switch 2", "On|Off", ptCombo);
   AddString(NULL, "Led 1", "red|green", ptCombo);
   AddString(NULL, "Led 2", "red|green", ptCombo);
   AddString(NULL, "Relais 1+2", "Time switch|Heating", ptCombo);
   Tree->EndUpdate();
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift) {
   if (Key == VK_RETURN)  {
   	Tree->EndEditNode();
       //  call here the function to handle the default CR, 
       //  for example:
       //      QuittClick(NULL);
   }
  	if (Key == VK_ESCAPE)  {
   	Tree->CancelEditNode();
       //  call here the function to handle the default ESC, 
       //  for example:
       //      Close();
   }
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::QuittClick(TObject *Sender) {
	Close();
}
//---------------------------------------------------------------------------

TVirtualNode* __fastcall TPropertyEditorForm::AddString(TVirtualNode* BasisNode, AnsiString Str1, AnsiString Str2, TPropertyTyp Property) {
   TVirtualNode* Node = Tree->AddChild(BasisNode, NULL);
   TPropertyData* Data = (TPropertyData*)Tree->GetNodeData(Node);
   Data->PropertyTyp = Property;
   Data->Node = Node;
   Data->Comment = Str1;
   Data->Value = Str2;
   Data->Changed = false;
   return(Node);
}

//===========================================================================
//----------------- Tree-Notify ---------------------------------------------

void __fastcall TPropertyEditorForm::TreeCreateEditor(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, IVTEditLink **EditLink) {
   if (!Node) return;                                                //  only for safety the program !
   TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);
   TPropertyEdit* PropertyLink = new TPropertyEdit((TVirtualStringTree*)Sender, Node, Column, Data);
   PropertyLink->QueryInterface(__uuidof(IVTEditLink), (void**)EditLink);
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreeChange(TBaseVirtualTree *Sender, PVirtualNode Node) {
   if (!Node) return;                                                //  only for safety the program !
   TPropertyData* Data = (TPropertyData*)Tree->GetNodeData(Node);
   if (Data && (Data->PropertyTyp != ptNone)) Sender->EditNode(Node, 2);
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreeEditing(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, bool &Allowed) {
   Allowed = false;
   if (!Node) return;                                                //  only for safety the program !
   if (!AllowEdit->Checked) return;
   TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);
   if (Data) Allowed = ((Column == 2) && (Data->PropertyTyp != ptNone));
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreeFreeNode(TBaseVirtualTree *Sender, PVirtualNode Node) {
   if (!Node) return;                                                //  only for safety the program !
   TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);
   if (Data) {
   	Data->Comment= "";
       Data->Value = "";
   }
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreeGetHint(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, TVSTTextType TextType, WideString &Text) {
   if (!Node) return;                                                //  only for safety the program !
   TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);
   if (Data && (Column == 1)) Text = Data->Comment;  else  Text = "";
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreeGetImageIndex(TBaseVirtualTree *Sender, PVirtualNode Node, TVTImageKind Kind, int Column, bool &Ghosted, int &Index) {
   Index = -1;
   if (!Node) return;                                                //  only for safety the program !
   TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);
   if (Data && (Column == 0))
   	if (Node->States.Contains(vsSelected))
           if (Data->PropertyTyp == ptNone)
           	Index = 1;
           else
       		Index = 0;
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreeGetText(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, TVSTTextType TextType, WideString &Text) {
   if (!Node) return;                                                //  only for safety the program !
   TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);  int Pos;
   if (Data) {
   	switch (Column) {
       	case 0:  Text = "";  break;
   		case 1:  Text = Data->Comment;  break;
       	case 2:  switch (Data->PropertyTyp) {
                    	 case ptNone:      Text = Data->Value;  break;
                        case ptEdit:      Text = Data->Value;  break;
                        case ptCombo:     Pos = Data->Value.Pos("|");  Text = Data->Value.SubString(1, Pos-1);  break;
                    }
           	break;
   	}
   }
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreePaintText(TBaseVirtualTree *Sender, const TCanvas *Canvas, PVirtualNode Node, int Column, TVSTTextType TextType) {
   if (!Node) return;                                                //  only for safety the program !
   TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);
   if (Data) {
   	if (Data->Changed)
   		Canvas->Font->Style = Canvas->Font->Style << fsBold;
   	else
   		Canvas->Font->Style = Canvas->Font->Style >> fsBold;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEditorForm::TreeClick(TObject *Sender) {
   TVirtualStringTree* Tree = (TVirtualStringTree*)Sender;
   if (!Tree->IsEditing() && Tree->FocusedNode) Tree->EditNode(Tree->FocusedNode, 2);
}

//===========================================================================
//----------------- TPropertyEdit -------------------------------------------

void __fastcall TPropertyEdit::EditWindowProc(TMessage &Message) {
	// here we can capture some messages for keeping track of focus changes
   switch (Message.Msg) {
   	case WM_KILLFOCUS:  EndEdit();  if ((int)FTree->Handle != Message.WParam) SetFocus((HWND)Message.WParam);  break;
       default:            FOldEditProc(Message);
   }
}
//---------------------------------------------------------------------------

void __fastcall TPropertyEdit::EditKeyDown(TObject* Sender, WORD &Key, TShiftState Shift) {
   //  als Key hat man hier: VK_HOME VK_END VK_UP VK_DOWN VK_PRIOR VK_NEXT VK_ESCAPE VK_RETURN
   bool CanAdvance;  TShiftState Empty;  bool Handled = false;  
   if ((!Handled) && ((Key == VK_PRIOR) || (Key == VK_NEXT) || (Key == VK_UP) || (Key == VK_DOWN))) {
       CanAdvance = (Shift.Contains(ssShift) || Shift.Contains(ssCtrl));
   	switch (Key) {
           case VK_UP:     if (!CanAdvance) FTree->FocusedNode = FTree->GetPreviousVisible(FTree->FocusedNode); break;
           case VK_DOWN:   if (!CanAdvance) FTree->FocusedNode = FTree->GetNextVisible(FTree->FocusedNode);     break;
       }
       if (FTree->FocusedNode && (!CanAdvance) && ((Key == VK_UP) || (Key == VK_DOWN))) {
       	EndEdit();
       	FTree->Selected[FTree->FocusedNode] = true;
       }
       Handled = true;
   }
   if ((!Handled) && (FData->PropertyTyp == ptCombo) && ((Key == VK_UP) || (Key == VK_DOWN))) {     
   	Empty.Clear();
       CanAdvance = (Shift == Empty);
       CanAdvance = CanAdvance & ~((TComboBox*)FEdit)->DroppedDown;
       if (CanAdvance) {
       	switch (Key) {
           	case VK_UP:   FTree->FocusedNode = FTree->GetPreviousVisible(FTree->FocusedNode); break;
               case VK_DOWN: FTree->FocusedNode = FTree->GetNextVisible(FTree->FocusedNode);     break;
           }
           if (FTree->FocusedNode) {
       		EndEdit();
       		FTree->Selected[FTree->FocusedNode] = true;
           }
       	Handled = true;
       }
	}
   if (Handled) Key = 0;
}
//---------------------------------------------------------------------------

__fastcall TPropertyEdit::TPropertyEdit(TVirtualStringTree* Tree, TVirtualNode* Node, int Column, TPropertyData* Data) {
	FRefCount = 0;
   FTree = Tree;
   FNode = Node;
   FColumn = Column;
   FData = Data;
   FStopping = false;
   switch (FData->PropertyTyp) {
     	case ptNone:     FEdit = NULL;                        break;
       case ptEdit:     FEdit = new TEdit((void*)NULL);      break;
       case ptCombo:    FEdit = new TComboBox((void*)NULL);  break;
       default:         FEdit = NULL;
   }
   if (FEdit != NULL) FEdit->Parent = Tree;
}
//---------------------------------------------------------------------------

__fastcall TPropertyEdit::~TPropertyEdit(void) {
   //  Release() will delete myself if counter == 0 !!!!!
}
//---------------------------------------------------------------------------

bool __stdcall TPropertyEdit::BeginEdit(void) {
   bool Result = !FStopping;
   if (Result && (FEdit != NULL)) {
   	FEdit->Show();
      	FEdit->SetFocus();
    	FOldEditProc = FEdit->WindowProc;
    	FEdit->WindowProc = &EditWindowProc;
  	}
   return(Result);
}
//---------------------------------------------------------------------------

bool __stdcall TPropertyEdit::CancelEdit(void) {
   bool Result = FStopping;
   if (!FStopping) {
    	FStopping = true;
       if (FEdit != NULL) {
       	FEdit->WindowProc = FOldEditProc;
           FEdit->Hide();
       }
    	FTree->CancelEditNode();
   }
   return(Result);
}
//---------------------------------------------------------------------------

bool __stdcall TPropertyEdit::EndEdit(void) {
   char  Buffer[1024];  WideString S;  //  TVirtualNode* Node;
   //  bool Result = FStopping;
	if (!FStopping) {
  		FStopping = true;
		FEdit->WindowProc = FOldEditProc;                             // restore the edit's window proc
   	switch (FData->PropertyTyp) {
     		case ptNone:    FData->Changed = false;                                break;
       	case ptEdit:    GetWindowText(FEdit->Handle, Buffer, 1024);
           	            S = Buffer;
   						if (S != FData->Value) {
   							FData->Value = S;
      							FData->Changed = true;
      							FTree->InvalidateNode(FData->Node);
   						}
       		break;
       	case ptCombo:   /* must in Orginal the code working with ItemIndex */
           	            //  Node = FTree->GetNextSibling(FData->Node);
           	            FData->Changed = (((TComboBox*)FEdit)->ItemIndex != 0);
       		break;
   	}
  		FEdit->Hide();
  		FTree->EndEditNode();
    	//  if (FData->Changed) FTree.DoNewText(FNode, FColumn, FCaption);
  	}
   //  return(Result);
   //  with version 2.5.40 return true,   when you are ready
   //                      return false,  when you detect an error and want editing again
   return(true);
}
//---------------------------------------------------------------------------

bool __stdcall TPropertyEdit::PrepareEdit(TBaseVirtualTree* Tree, PVirtualNode Node, int Column) {
   int iP;  AnsiString Strg;  bool Result = false;  WideString FCaption;
   if (!Node) return(Result);                                        //  only for safety the program !
   if (FEdit == NULL) return(Result);
   FTree = (TVirtualStringTree*)Tree;
   FNode = Node;
   FColumn = Column;
   FData = (TPropertyData*)FTree->GetNodeData(FNode);
   FEdit->Parent = FTree;
   switch (FData->PropertyTyp) {
     	case ptNone:    break;
       case ptEdit:    ((TEdit*)FEdit)->Visible = false;
					    ((TEdit*)FEdit)->Text = FData->Value;
				        ((TEdit*)FEdit)->OnKeyDown = &EditKeyDown;
                       Result = true;
          	break;
       case ptCombo:   ((TComboBox*)FEdit)->Visible = false;
                       Strg = FData->Value;
                       while (Strg != "") {
                           iP = Strg.Pos("|");
                           if (iP == 0) iP = Strg.Length()+1;
                      		((TComboBox*)FEdit)->Items->Add(Strg.SubString(1, iP-1));
                           Strg = Strg.Delete(1, iP);
                       }
                       ((TComboBox*)FEdit)->ItemIndex = 0;
                       ((TComboBox*)FEdit)->Style = Stdctrls::csDropDownList;
				        ((TComboBox*)FEdit)->OnKeyDown = &EditKeyDown;
                       Result = true;
          	break;
   }
   return(Result);
}
//---------------------------------------------------------------------------

/*
bool __stdcall TPropertyEdit::CanStop(void) {
   bool Result = !FStopping;
	return(Result);
}
//---------------------------------------------------------------------------
*/

Windows::TRect __stdcall TPropertyEdit::GetBounds(void) {
   TRect R;
   if (FEdit != NULL) R = FEdit->BoundsRect;  else  memset(&R, 0, sizeof(TRect));
   return(R);
}
//---------------------------------------------------------------------------

void __stdcall TPropertyEdit::SetBounds(const Windows::TRect R) {
  if (!FStopping && (FEdit != NULL)) FEdit->BoundsRect = R;
}
//---------------------------------------------------------------------------

void __stdcall TPropertyEdit::ProcessMessage(Messages::TMessage &Message) {
	FEdit->WindowProc(Message);
}
//---------------------------------------------------------------------------

ULONG __stdcall TPropertyEdit::AddRef() {                  //  Implement IUnknown interface
	return InterlockedIncrement((LPLONG)&FRefCount);        //  thread safe increment
}
//---------------------------------------------------------------------------

ULONG __stdcall TPropertyEdit::Release() {                 //  Implement IUnknown interface
	InterlockedDecrement((LPLONG)&FRefCount);               //  thread safe decrement
	if (FRefCount == 0) {                                   //  if last user then
		delete this;                                        //    delete myself
		return 0;                                           //    => 0 !
	}
	return FRefCount;                                       //  return internal counter
}
//---------------------------------------------------------------------------

HResult  __stdcall TPropertyEdit::QueryInterface(const GUID& IID, void**ppv) {
	if (IID == __uuidof(IUnknown))                          //  IUnknown as basic interface
   	*ppv = (IUnknown*)this;
	else if (IID == __uuidof(IVTEditLink))                  //  i am self
		*ppv = (IVTEditLink*)this;
	else {
		*ppv = NULL;                                        //  unknowm interface
		return E_NOINTERFACE;
	}
	((IUnknown*)(*ppv))->AddRef();                          //  increment if used
	return S_OK;                                            //  Ok !
}
//===========================================================================





