//---------------------------------------------------------------------------
#ifndef PropertyEditorMainH
#define PropertyEditorMainH

//---------------------------------------------------------------------------
//  Demoprogram for own editor under VT 2.4x
//
//  This demoprogram is based on VirtualTrees Library from Mike Lischke.
//  Please read the copyright info in VirtualTrees.pas
//
//  In this place many thanks to Alexander Egorushkin for the
//  very very important information to implement an interface to use
//  own editor under BCB5.
//
//  Copyright (c) 4.2001  information about               Alexander Egorushkin
//                        "interface" and how to use it
//                        this demoprogram                Roland Beduerftig
//                        EMail:                          Beduerftig@SoftwareCreation.de
//
//  Free: You may use this code in every way you find it useful or fun.
//
//  Credits for their valuable assistance and code donations go to:
//    see VirtualTrees.pas
//---------------------------------------------------------------------------

#include "VirtualTrees.hpp"

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <Dialogs.hpp>
#include <ImgList.hpp>
#include <ComCtrls.hpp>

#define  ROOT      NULL
class  TPropertyEdit;

//---------------------------------------------------------------------------

typedef enum {
   ptNone,
   ptEdit,
   ptCombo,
   ptNumber
} TPropertyTyp;
//---------------------------------------------------------------------------

typedef struct {
   TPropertyTyp    PropertyTyp;
   TVirtualNode*   Node;
   WideString      Comment;
   WideString      Value;
   bool            Changed;
   void*           Data;
} TPropertyData, *lpPropertyData;

//===========================================================================
//  Important note:
//
//  C++Builder can't support multipart interfaces, from help on TInterfacedObject
//
//  "Do not use TInterfacedObject in C++Builder applications. COM support in
//  C++Builder is based on the ATL library. TInterfacedObject exists for
//  interfaces that are used by the C++Builder IDE, such as component editors".
//
//  IVTEditLink is a simle interface so we can implement it without ATL.
//
//  1. declare an own class xxx : public IVTEditLink
//  2. implement the complet inferface in this class
//  3. doubleclick on notify event OnCreateEditor in VT
//  4. modify the autogenerate code yyyCreateEditor(...., IVTEditLink **EditLink);
//                                                        ^inferface  ^pointer
//     note: modify in *.h and *.cpp file
//  5. build instance of your own editor
//     ... yyyCreateEditor(...., IVTEditLink **EditLink) {
//        TPropertyData* Data = (TPropertyData*)Sender->GetNodeData(Node);
//        TPropertyEdit* PropertyLink = new TPropertyEdit((TVirtualStringTree*)Sender, Node, Column, Data);
//        PropertyLink->QueryInterface(__uuidof(IVTEditLink), (void**)EditLink);
//     }
//  6. now your own editor will work like as the demos in Delphi5
//===========================================================================

class TPropertyEdit : public IVTEditLink {
	ULONG FRefCount;                                        //  used by our own implementation of the interface
   bool  FStopping;                                        //  local merker
   int   FColumn;                                          //  we save our own info about,
   TVirtualStringTree* FTree;                              //  parent, tree, node, column and our data
   TVirtualNode*       FNode;
   TPropertyData*      FData;
   TWndMethod          FOldEditProc;                       //  used to capture some important messages
   TWinControl*        FEdit;                              //  regardless of the type of edit control we use
protected:
   // private members
   void __fastcall EditWindowProc(TMessage &Message);
   void __fastcall EditKeyDown(TObject* Sender, WORD &Key, TShiftState Shift);
public:
	// Constructor/Destructor
	__fastcall TPropertyEdit(TVirtualStringTree* Tree, TVirtualNode* Node, int Column, TPropertyData* Data);
	virtual __fastcall ~TPropertyEdit(void);
	// IUnknown members
	virtual ULONG   __stdcall AddRef();
	virtual ULONG   __stdcall Release();
	virtual HResult  __stdcall QueryInterface(const GUID& IID, void** ppv);
	// IVTEditLink members
	virtual bool __stdcall BeginEdit(void);
	virtual bool __stdcall CancelEdit(void);
//	virtual bool __stdcall CanStop(void);
	virtual bool __stdcall EndEdit(void);
	virtual bool __stdcall PrepareEdit(TBaseVirtualTree* Tree, PVirtualNode Node, int Column);
	virtual Windows::TRect __stdcall GetBounds(void);
	virtual void __stdcall SetBounds(const Windows::TRect R);
	virtual void __stdcall ProcessMessage(Messages::TMessage &Message);
};
//---------------------------------------------------------------------------

class TPropertyEditorForm : public TForm {
__published:
	TButton *Quitt;
	TImageList *TreeImage;
	TVirtualStringTree *Tree;
	TCheckBox *AllowEdit;
	void __fastcall QuittClick(TObject *Sender);
	void __fastcall FormDestroy(TObject *Sender);
	void __fastcall FormShow(TObject *Sender);
	void __fastcall FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift);
	void __fastcall TreeGetText(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, TVSTTextType TextType, WideString &Text);
	void __fastcall TreeFreeNode(TBaseVirtualTree *Sender, PVirtualNode Node);
	void __fastcall TreeChange(TBaseVirtualTree *Sender, PVirtualNode Node);
	void __fastcall TreeEditing(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, bool &Allowed);
	void __fastcall TreeGetHint(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, TVSTTextType TextType, WideString &Text);
	void __fastcall TreePaintText(TBaseVirtualTree *Sender, const TCanvas *Canvas, PVirtualNode Node, int Column, TVSTTextType TextType);
   void __fastcall TreeCreateEditor(TBaseVirtualTree *Sender, PVirtualNode Node, int Column, IVTEditLink **EditLink);
	void __fastcall TreeGetImageIndex(TBaseVirtualTree *Sender, PVirtualNode Node, TVTImageKind Kind, int Column, bool &Ghosted, int &Index);
	void __fastcall TreeClick(TObject *Sender);
private:
   TVirtualNode* __fastcall AddString(TVirtualNode* BasisNode, AnsiString Str1, AnsiString Str2, TPropertyTyp Property);
public:
	__fastcall TPropertyEditorForm(TComponent* Owner);
};

//---------------------------------------------------------------------------
extern PACKAGE TPropertyEditorForm *PropertyEditorForm;
//---------------------------------------------------------------------------
#endif
