unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, StdCtrls;

type
  PNodeData = ^TNodeData;
  TNodeData = packed record
    Name,
    NameColumn1,
    NameColumn2: string[255];
  end;

type
  TForm1 = class(TForm)
    VirtualStringTree1: TVirtualStringTree;
    procedure VirtualStringTree1CreateEditor(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: Integer; out EditLink: IVTEditLink);
    procedure FormCreate(Sender: TObject);
    procedure VirtualStringTree1InitNode(Sender: TBaseVirtualTree;
      ParentNode, Node: PVirtualNode;
      var InitialStates: TVirtualNodeInitStates);
    procedure VirtualStringTree1GetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
      var Text: WideString);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

{ IVTEditLink is defined like this so any class that implements this interface  }
{ must define all these functions.                                              }

(*  IVTEditLink = interface
    ['{2BE3EAFA-5ACB-45B4-9D9A-B58BCC496E17}']
    function BeginEdit: Boolean; stdcall;                  // called when editing actually starts
    function CancelEdit: Boolean; stdcall;                 // called when editing has been cancelled by the tree
    function EndEdit: Boolean; stdcall;                    // called when editing has been finished by the tree
    function PrepareEdit(Tree: TBaseVirtualTree; Node: PVirtualNode; Column: Integer): Boolean; stdcall;
                                                           // called after creation to allow a setup
    function GetBounds: TRect; stdcall;                    // called to get the current size of the edit window
                                                           // (only important if the edit resizes itself)
    procedure ProcessMessage(var Message: TMessage); stdcall;
                                                           // used to forward messages to the edit window(s)
    procedure SetBounds(R: TRect); stdcall;                // called to place the editor
  end;
*)

{ Make TMyCustomEditLink a referenced counted object so it will be freed like a }
{ COM object as well as implementing the IVTEditList interface.                 }
type
  TMyCustomEditLink = class(TInterfacedObject, IVTEditLink)
  private
    FComboBox: TComboBox;
    FEditing: Boolean;
    FEditColumn: integer;
    FEditNode: PVirtualNode;
    FEditTree: TBaseVirtualTree;
    FNodeRect: TRect;
  public
    constructor Create;
    destructor Destroy; override;

    function BeginEdit: Boolean; stdcall;
    function CancelEdit: Boolean; stdcall;
    function EndEdit: Boolean; stdcall;
    function PrepareEdit(Tree: TBaseVirtualTree; Node: PVirtualNode; Column: Integer): Boolean; stdcall;
    function GetBounds: TRect; stdcall;
    procedure ProcessMessage(var Message: TMessage); stdcall;
    procedure SetBounds(R: TRect); stdcall;

    property ComboBox: TComboBox read FComboBox write FComboBox;
    property EditColumn: integer read FEditColumn write FEditColumn;
    property EditNode: PVirtualNode read FEditNode write FEditNode;
    property EditTree: TBaseVirtualTree read FEditTree write FEditTree;
    property Editing: Boolean read FEditing write FEditing;
    property NodeRect: TRect read FNodeRect write FNodeRect;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.VirtualStringTree1CreateEditor(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; out EditLink: IVTEditLink);
begin
  { Create one of my EditLinks }
  EditLink :=  TMyCustomEditLink.Create;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  VirtualStringTree1.NodeDataSize := SizeOf(TNodeData);
end;

procedure TForm1.VirtualStringTree1InitNode(Sender: TBaseVirtualTree;
  ParentNode, Node: PVirtualNode;
  var InitialStates: TVirtualNodeInitStates);
var
  NodeData: PNodeData;
begin
  NodeData := Sender.GetNodeData(Node);
  if Assigned(NodeData) then
  begin
    NodeData^.Name := 'Initial Title';
    NodeData^.NameColumn1 := 'Initial Title Column 1';
    NodeData^.NameColumn2 := 'Initial Title Column 2';
  end
end;

procedure TForm1.VirtualStringTree1GetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
  var Text: WideString);
var
  NodeData: PNodeData;
begin
  NodeData := Sender.GetNodeData(Node);
  if Assigned(NodeData) then
  begin
    case Column of
      0:  Text := NodeData^.Name;
      1:  Text := NodeData^.NameColumn1;
      2:  Text := NodeData^.NameColumn2;
    end;
  end
end;

{ TMyCustomEditLink }

function TMyCustomEditLink.BeginEdit: Boolean;
begin
  if Assigned(ComboBox) then
  begin
    Result := True;
    ComboBox.Show;
    ComboBox.SetFocus;
    Editing := True;
  end else
    Result := False;
end;

function TMyCustomEditLink.CancelEdit: Boolean;
begin
  Result := True;  // Always allow a cancel
  if Editing then
  begin
    Editing := False;
    ComboBox.Hide;
    EditTree.SetFocus;
    EditTree := nil;
    EditColumn := -1;
    EditNode := nil;
  end
end;

constructor TMyCustomEditLink.Create;
begin
  ComboBox := TComboBox.Create(nil); // Nobody owns it
  ComboBox.Visible := False;
end;

destructor TMyCustomEditLink.Destroy;
begin
  ComboBox.Free;
  inherited;
end;

function TMyCustomEditLink.EndEdit: Boolean;
var
  NodeData: PNodeData;
begin
  Result := True;  // Always allow end
  if Editing then
  begin
    if ComboBox.ItemIndex > -1 then
    begin
      NodeData := EditTree.GetNodeData(EditNode);
      case EditColumn of
        0: NodeData^.Name := ComboBox.Items[ComboBox.ItemIndex];
        1: NodeData^.NameColumn1 := ComboBox.Items[ComboBox.ItemIndex];
        2: NodeData^.NameColumn2 := ComboBox.Items[ComboBox.ItemIndex];
      end
    end;
    EditTree := nil;
    EditColumn := -1;
    EditNode := nil;
  end
end;

function TMyCustomEditLink.GetBounds: TRect;
begin
  Result := ComboBox.BoundsRect
end;

function TMyCustomEditLink.PrepareEdit(Tree: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer): Boolean;
begin
  Result := False;
  if Assigned(ComboBox) then
  begin
    Result := True;
    EditTree := Tree;
    ComboBox.Parent := EditTree;
    EditNode := Node;
    EditColumn := Column;
    ComboBox.Items.Add('New Title 1');
    ComboBox.Items.Add('New Title 2');
    ComboBox.Items.Add('New Title 3');
    ComboBox.Items.Add('New Title 4');
    ComboBox.Items.Add('New Title 5');
    ComboBox.Items.Add('New Title 6');
    NodeRect := Tree.GetDisplayRect(Node, Column, False, False);
    ComboBox.Top := NodeRect.Top;
    ComboBox.Left := NodeRect.Left;
    ComboBox.Width := NodeRect.Right - NodeRect.Left;
    ComboBox.Height := 100;
    ComboBox.ItemIndex := 0;
  end
end;

procedure TMyCustomEditLink.ProcessMessage(var Message: TMessage);
begin
  ComboBox.WindowProc(Message);
end;

procedure TMyCustomEditLink.SetBounds(R: TRect);
begin
  ComboBox.BoundsRect := R
end;


end.
