unit DemoUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, MFEdit, VSTEdit, ComCtrls, StdCtrls, Mask, Buttons;

type
  PNodeData = ^TNodeData;
  TNodeData = record
    Key          : String;
    EditType     : TVSTEditorType;
    EditString   : String;
    EditInteger  : Integer;
    EditReal     : Double;
    EditDateTime : TDateTime;
    EditColor    : TColor;
    EditFont     : TFont;
  end;
  TfrmEdit = class(TForm)
    vstEdit : TVirtualStringTree;
    MFNumEdit: TMFNumEdit;
    MFButtonEdit: TMFButtonEdit;
    MFEdit: TMFEdit;
    MFDateTimeEdit: TMFDateTimeEdit;
    VSTEditor: TVSTEditor;
    MFComboEdit: TMFComboEdit;
    procedure FormCreate(Sender: TObject);
    procedure vstEditCreateEditor(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: Integer; out EditLink: IVTEditLink);
    procedure vstEditEditCancelled(Sender: TBaseVirtualTree;
      Column: Integer);
    procedure vstEditEdited(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer);
    procedure vstEditEditing(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer; var Allowed: Boolean);
    procedure vstEditFreeNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure vstEditGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer; TextType: TVSTTextType; var Text: WideString);
    procedure vstEditPaintText(Sender: TBaseVirtualTree;
      const Canvas: TCanvas; Node: PVirtualNode; Column: Integer;
      TextType: TVSTTextType);
    procedure MFButtonEditButtonClick(Sender: TObject);
    procedure vstEditNewText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer; Text: WideString);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  frmEdit: TfrmEdit;

implementation

{$R *.DFM}


function AddNodeWithData(V: TVirtualStringTree; K: String;
  T: TVSTEditorType; D: Variant): PVirtualNode;
var
    Data : PNodeData;
begin
    Result := V.AddChild( nil );
    Data   := V.GetNodeData( Result );
    with Data^ do
    begin
        Key      := K;
        EditType := T;
        case T of
            etEditor,
            etButtonEditor,
            etComboEditor,
            etFilenameEditor,
            etDirectoryEditor:
                EditString   := D;
            etNumEditor,
            etCurrencyEditor:
                EditReal     := D;
            etDateTimeEditor:
                EditDateTime := D;
            etColorEditor:
                EditColor    := D;
            etFontEditor:
                EditString   := D;
        end;
    end;
    V.ValidateNode( Result, False );
end;

// ****************************************************************************

procedure TfrmEdit.FormCreate(Sender: TObject);
var
    Node : PVirtualNode;
begin
    vstEdit.NodeDataSize := Sizeof( TNodeData );

{
    with DropDownEdit do
    begin
        Items.Add( 'Entry 1' );
        Items.Add( 'Entry 2' );
        Items.Add( 'Entry 3' );
        Items.Add( 'Entry 4' );
        Items.Add( 'Entry 5' );
    end;
}

    AddNodeWithData( vstEdit, 'String Editor',               etEditor,         'String' );
    AddNodeWithData( vstEdit, 'String Editor with Button',   etButtonEditor,   'Another String' );
    AddNodeWithData( vstEdit, 'Combo Editor (DropDown)',     etComboEditor,    'Entry 1' );
    Node := AddNodeWithData( vstEdit, 'Combo Editor (DropDownList)', etComboEditor,    'Entry 3' );
    Node^.Dummy := 1;
    AddNodeWithData( vstEdit, 'Numeric Editor',              etNumEditor,      12.24 );
    AddNodeWithData( vstEdit, 'Date/Time Editor',            etDateTimeEditor, Date );
{
    AddNodeWithData( vstEdit, 'Filename',           etFilename,     'C:\WinNT\Notepad.exe' );
    AddNodeWithData( vstEdit, 'Directory',          etDirectory,    'C:\WinNT' );
    AddNodeWithData( vstEdit, 'Color',              etColor,        clRed );
    AddNodeWithData( vstEdit, 'Font',               etFont,         'Arial' );
}
end;

procedure TfrmEdit.vstEditCreateEditor(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; out EditLink: IVTEditLink);
var
    Data : PNodeData;
begin
    Data := Sender.GetNodeData( Node );
    with Data^ do
    begin
        case EditType of
            etEditor:
                VSTEditor.LinkEditor( EditLink, EditString );
            etButtonEditor:
                VSTEditor.LinkButtonEditor( EditLink, EditString );
            etComboEditor:
            begin
                if Node^.Dummy = 1 then
                    VSTEditor.ComboEditor.Style := csDropDownList
                else
                    VSTEditor.ComboEditor.Style := csDropDown;
                VSTEditor.LinkComboEditor( EditLink, EditString );
            end;
            etNumEditor:
                VSTEditor.LinkNumEditor( EditLink, EditReal );
            etDateTimeEditor:
                VSTEditor.LinkDateTimeEditor( EditLink, EditDateTime, 0 );
{
            etComboEditor:
            etCurrencyEditor:
            etFilenameEditor:
            etDirectoryEditor:
            etColorEditor:
            etFontEditor:
}
        end;
    end;
end;

procedure TfrmEdit.vstEditEditCancelled(Sender: TBaseVirtualTree;
  Column: Integer);
begin
    ShowMessage( 'Aborted!' );
end;

procedure TfrmEdit.vstEditEdited(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer);
begin
//
end;

procedure TfrmEdit.vstEditEditing(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; var Allowed: Boolean);
begin
    Allowed := Column > 0;
end;

procedure TfrmEdit.vstEditFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
    Data : PNodeData;
begin
    Data := vstEdit.GetNodeData( Node );
    Data^.EditString := '';
end;

procedure TfrmEdit.vstEditGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
  var Text: WideString);
var
    Data : PNodeData;
begin
    Data := vstEdit.GetNodeData( Node );
    with Data^ do
    begin
        if Column = 0 then
        begin
            Text := Key;
            Exit;
        end;

        case EditType of
            etEditor,
            etButtonEditor,
            etComboEditor,
            etFilenameEditor,
            etDirectoryEditor,
            etFontEditor:
                Text := EditString;
            etNumEditor:
                Text := FloatToStr( EditReal );
            etCurrencyEditor:
                Text := FloatToStr( EditReal ) + ' DM';
            etDateTimeEditor:
                Text := FormatDateTime( 'dd.mm.yyyy', EditDateTime );
            etColorEditor:
                Text := IntToStr( EditColor );
        end;
    end;
end;

procedure TfrmEdit.vstEditPaintText(Sender: TBaseVirtualTree;
  const Canvas: TCanvas; Node: PVirtualNode; Column: Integer;
  TextType: TVSTTextType);
begin
//
end;

procedure TfrmEdit.MFButtonEditButtonClick(Sender: TObject);
begin
    ShowMessage( 'Button clicked!' );
end;

procedure TfrmEdit.vstEditNewText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; Text: WideString);
var
    Data : PNodeData;
begin
    Data := Sender.GetNodeData( Node );
    with Data^ do
    begin
        case EditType of
            etEditor,
            etButtonEditor,
            etComboEditor:
                EditString   := Text;
            etNumEditor:
                EditReal     := MFNumEdit.Value;
            etDateTimeEditor:
                EditDateTime := MFDateTimeEdit.Date;
{
            etCurrencyEditor:
            etFilenameEditor:
            etDirectoryEditor:
            etColorEditor:
            etFontEditor:
}
        end;
    end;
end;

end.
