unit Mp3Names;


interface

function booleantochar(val: boolean): char;
function chartoboolean(val: char): boolean;
function secondsToString(seconds: integer): string;
function sizetostr(size: int64): string;
function getDurationFromExtint(extint: string): integer;
function removeTrackNumber(name: string): string;
function get_trackname(filename: string): string;
function get_artist(filename: string): string;
function get_album(filename: string): string;
function Genre2string(Genre: byte): string;

implementation

uses jbstr, sysutils;

function booleantochar(val: boolean): char;
begin
  result := 'F';
  if val then result := 'T';
end;

function chartoboolean(val: char): boolean;
begin
  result := false;
  if val = 'T' then result := true;
end;


function secondsToString(seconds: integer): string;
var
  hr, min, sec: integer;
begin
  sec := seconds mod 60;
  min := seconds div 60;
  if min > 60 then
  begin
    hr := min div 60;
    min := min mod 60;
    result := inttostr(hr) + ':' + leftpadch(inttostr(min), '0', 2) + ':' +
      leftpadch(inttostr(sec), '0', 2);
  end
  else
  begin
    result := leftpad(inttostr(min), 2) + ':' + leftpadch(inttostr(sec), '0', 2);
  end;

end;

function sizetostr(size: int64): string;
var str: string;
begin
  if size > 10240 then
  begin
    size := size shr 10;
    str := 'kb';
    if size > 10240 then
    begin
      size := size shr 10;
      str := 'Mb';
      if size > 10240 then
      begin
        size := size shr 10;
        str := 'Gb';
      end;
    end;
  end;
  result := inttostr(size) + ' ' + str;
end;

function removeTrackNumber(name: string): string;
begin
  result := name;
  if length(name) > 2 then
  begin
    if name[1] in ['0'..'9'] then
      if name[2] in ['0'..'9'] then
        if (name[3] = ' ') or (name[3] = '_') then
        begin
          result := copy(name, 4, length(name));
          if length(name) > 1 then
          begin
            if result[1] = '-' then
              if result[2] = ' ' then
                result := copy(result, 3, length(result));
          end;
        end;
  end;
end;

function get_trackname(filename: string): string;
var str : string;
  i,p : integer;
begin
  str := justfilename(filename);
  p := length(str)+1;
  for i := 1 to length(str) do
    if str[i]='.' then p := i;
  result := removeTrackNumber(copy(str,1,p-1));
end;

function get_album(filename: string): string;
var i, j: integer;
begin
  i := length(filename);
  if (i > 0) then
  begin
    while ((not (filename[i] = '\')) and (i > 1)) do dec(i);
    j := i - 1;
    while ((not (filename[j] = '\')) and (j > 1)) do dec(j);
    result := trim(copy(filename, j + 1, i - j - 1));
    if length(result)>1 then
      if result[1] = '-' then
        result := copy(result,2,length(result));
      if result[length(result)] = '-' then
        result := copy(result,1,length(result)-1);
  end;
end;

function get_artist(filename: string): string;
var i, j: integer;
begin
  i := length(filename);
  if (i > 0) then
  begin
    while ((not (filename[i] = '\')) and (i > 1)) do dec(i);
    dec(i);
    while ((not (filename[i] = '\')) and (i > 1)) do dec(i);
    j := i - 1;
    while ((not (filename[j] = '\')) and (j > 1)) do dec(j);
    result := copy(filename, j + 1, i - j - 1);
  end;
end;

function getDurationFromExtint(extint: string): integer;
begin
  result := strtoint(copy(extint, 9, pos(',', extint) - 9));
end;

function Genre2string(Genre: byte): string;
begin
  case genre of
    0: result := 'Blues';
    1: result := 'Classic Rock';
    2: result := 'Country';
    3: result := 'Dance';
    4: result := 'Disco';
    5: result := 'Funk';
    6: result := 'Grunge';
    7: result := 'Hip-Hop';
    8: result := 'Jazz';
    9: result := 'Metal';
    10: result := 'New Age';
    11: result := 'Oldies';
    12: result := 'Other';
    13: result := 'Pop';
    14: result := 'R&B';
    15: result := 'Rap';
    16: result := 'Reggae';
    17: result := 'Rock';
    18: result := 'Techno';
    19: result := 'Industrial';
    20: result := 'Alternative';
    21: result := 'Ska';
    22: result := 'Death Metal';
    23: result := 'Pranks';
    24: result := 'Soundtrack';
    25: result := 'Euro-Techno';
    26: result := 'Ambient';
    27: result := 'Trip-Hop';
    28: result := 'Vocal';
    29: result := 'Jazz+Funk';
    30: result := 'Fusion';
    31: result := 'Trance';
    32: result := 'Classical';
    33: result := 'Instrumental';
    34: result := 'Acid';
    35: result := 'House';
    36: result := 'Game';
    37: result := 'Sound Clip';
    38: result := 'Gospel';
    39: result := 'Noise';
    40: result := 'AlternRock';
    41: result := 'Bass';
    42: result := 'Soul';
    43: result := 'Punk';
    44: result := 'Space';
    45: result := 'Meditative';
    46: result := 'Instrumental Pop';
    47: result := 'Instrumental Rock';
    48: result := 'Ethnic';
    49: result := 'Gothic';
    50: result := 'Darkwave';
    51: result := 'Techno-Industrial';
    52: result := 'Electronic';
    53: result := 'Pop-Folk';
    54: result := 'Eurodance';
    55: result := 'Dream';
    56: result := 'Southern Rock';
    57: result := 'Comedy';
    58: result := 'Cult';
    59: result := 'Gangsta';
    60: result := 'Top 40';
    61: result := 'Christian Rap';
    62: result := 'Pop/Funk';
    63: result := 'Jungle';
    64: result := 'Native American';
    65: result := 'Cabaret';
    66: result := 'New Wave';
    67: result := 'Psychadelic';
    68: result := 'Rave';
    69: result := 'Showtunes';
    70: result := 'Trailer';
    71: result := 'Lo-Fi';
    72: result := 'Tribal';
    73: result := 'Acid Punk';
    74: result := 'Acid Jazz';
    75: result := 'Polka';
    76: result := 'Retro';
    77: result := 'Musical';
    78: result := 'Rock & Roll';
    79: result := 'Hard Rock';
    80: result := 'Folk';
    81: result := 'Folk/Rock';
    82: result := 'National Folk';
    83: result := 'Swing';
    84: result := 'Fast Fusion';
    85: result := 'Bebob';
    86: result := 'Latin';
    87: result := 'Revival';
    88: result := 'Celtic';
    89: result := 'Bluegrass';
    90: result := 'Avantgarde';
    91: result := 'Gothic Rock';
    92: result := 'Progressive Rock';
    93: result := 'Psychedelic Rock';
    94: result := 'Symphonic Rock';
    95: result := 'Slow Rock';
    96: result := 'Big Band';
    97: result := 'Chorus';
    98: result := 'Easy Listening';
    99: result := 'Acoustic';
    100: result := 'Humour';
    101: result := 'Speech';
    102: result := 'Chanson';
    103: result := 'Opera';
    104: result := 'Chamber Music';
    105: result := 'Sonata';
    106: result := 'Symphony';
    107: result := 'Booty Bass';
    108: result := 'Primus';
    109: result := 'Porn Groove';
    110: result := 'Satire';
    111: result := 'Slow Jam';
    112: result := 'Club';
    113: result := 'Tango';
    114: result := 'Samba';
    115: result := 'Folklore';
    116: result := 'Ballad';
    117: result := 'Power Ballad';
    118: result := 'Rhythmic Soul';
    119: result := 'Freestyle';
    120: result := 'Duet';
    121: result := 'Punk Rock';
    122: result := 'Drum Solo';
    123: result := 'Acapella';
    124: result := 'Euro-House';
    125: result := 'Dance Hall';
  else result := '';
  end;
end;

end.
