unit VTrackTreeView;
{
VTrackTreeView is a treeview based on Virtual TreeView which is build
to display mp3 files (and attributes)

Together with TracklistView, it can be used to build an mp3 player.

main features are:

scan's directories for mp3's
background scan for mp3 tags
display of mp3's in tree with selectable attributes in grid
supports drag&drop with TrackListView
supports search
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  {ComCtrls,} menus, VirtualTrees, Buttons, registry, Mp3TrackData;

type
  TStrEvent = procedure(sender: TObject; str: string) of object;
  TReqEvent = function: boolean of object;

  TPlayMode = (normal, shuffle);

// TNodeData is the record which is stored in the virtualtree
// it contains a pointer to either a TTagobject or a TSumobject
//
  tnodetype = (ntEmpty, ntSum, ntTag);
  TNodeData = record
    nodeType: tnodetype;
    obj: pointer;
  end;
  PNodeData = ^TNodeData;

// the actual TrackTreeView

  VTTrackTreeView = class(TCustomVirtualStringTree)
  private
    function GetOptions: TStringTreeOptions;
    procedure SetOptions(const Value: TStringTreeOptions);
  protected
    function GetOptionsClass: TTreeOptionsClass; override;
  private
    { Private declarations }
    getcurrentIndex: integer;
    currentselected: PVirtualNode;

    currentNode: PVirtualNode;
    currentindex: integer;
    scanner: TMp3Data;
    initcnt:integer;
    procedure SetUserName(const Value: string);
  protected
    { Protected declarations }
    FOnTrackClick: TStrEvent;
    FOnTrackDblClick: TStrEvent;
    FOnScanProgress: TStrEvent;
    FOnScanReady: TNotifyEvent;
    FOnDebug: TStrEvent;
    FPlayMode: TPlayMode;
    FNotPlayed: boolean;
    FGenreFilter: integer;
    FStyleFilter: string;
    FPopupMenu: TPopupMenu;
    FSettingsDir: string;
    FUserName: string;
    FSortColumn: integer;
    FSortDirection: TSortDirection;

    basesum: TSumObject;
//    genresum: TSumObject;

    loaded: boolean;

    columns: array[0..20] of TColumnname;
    colwidth: array[0..20] of integer;
    columncnt: integer;

    currentSel: TTagObject;

    procedure DblClick(node: PVirtualNode);
    procedure ScanProgress(str: string);
    procedure Click(Sender: TObject);
    procedure MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure popuphandler(Sender: TObject);
    procedure ScannerReady(Sender: TObject);
    procedure Debug(str: string);
    function DebugStr(node: PVIrtualNode): string;
    function FindNextInTree: boolean;
    procedure saveSettingsReg;
    procedure loadSettingsReg;


    procedure ShowInfo(node: PVirtualNode; x, y: integer);
    procedure SetColumns;

    procedure CompareNodes(Sender: TBaseVirtualTree; Node1, Node2: PVirtualNode;
      Column: Integer; var Result: Integer);

    procedure InitNode(Sender: TBaseVirtualTree;
      ParentNode, Node: PVirtualNode;
      var InitialStates: TVirtualNodeInitStates);
    procedure GetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
      var Text: WideString);
    procedure InitChildren(Sender: TBaseVirtualTree;
      Node: PVirtualNode; var ChildCount: Cardinal);
    procedure GetHint(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
      var Text: WideString);
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure KeyPress(Sender: TObject; var Key: Char);
    procedure HeaderClick(Sender: TVTHeader; Column: Integer);
    procedure RefreshTree(sender:TObject);


  public
    { Public declarations }
    dirs: TStringList;
    styles: TStringlist;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure fields;
    procedure showoptions;
    procedure scan;
    procedure save;
    procedure load(filename: string);
    procedure loadDefault;
//    procedure saveuserdata(filename: string);
//    procedure loaduserdata(filename: string);

// the following functions are used for a dragdrop operation,
// when a drag operation is accepted which is started from this tree
// the selected node(s) can be retrieved with thes functions
    function GetFirstSelectedNode: string;   // use this function once to get the first selected node
    function GetNextSelectedNode: string;    // retrieve next selected node, until an empty string signals the end
    function GetCurrentSize: LongInt;        // attributes of selection can be got with these functions
    function GetCurrentDuration: integer;
    function GetCurrentTitle: string;
    function GetCurrentArtist: string;
    function GetCurrentAlbum: string;

//  the following functions are for continuous play, the next mp3 is played
    function DoNext: boolean;
    function DoPrev: boolean;

    procedure Played(filename: string);
    function  GetTagObject(filename: string): TTagObject;
    procedure search(str: string; onfound: TTagEvent; muststop: TReqEvent;
       title,artist,album,all : boolean);
    procedure SaveArtists(filename: string);
    procedure SaveArtistsNoStyle(filename: string);


  published
    { Published declarations }
    property UserName: string read FUserName write SetUserName;
    property SettingsDir: string read FSettingsDir write FSettingsDir;
    property Playmode: TPlayMode read FPlayMode write FPlayMode;
    property GenreFilter: integer read FGenreFilter write FGenreFilter default -1;
    property StyleFilter: string read FStyleFilter write FStyleFilter;
    property NotPlayed: boolean read FNotPlayed write FNotPlayed;
    property OnTrackClick: TStrEvent read FOnTrackClick
      write FOnTrackClick;
    property OnTrackDblClick: TStrEvent read FOnTrackDblClick
      write FOnTrackDblClick;
    property OnScanProgress: TStrEvent read FOnScanProgress
      write FOnScanProgress;
    property OnDebug: TStrEvent read FOnDebug
      write FOnDebug;
    property OnScanReady: TNotifyEvent read FOnScanReady write FOnScanReady;
{    property OnDurationChange: TStrEvent read FOnDurationChange
      write FOnDurationChange;}
    property Anchors;
    property Color;
    property Font;
    property Align;
    property BackGround;
    property TreeOptions: TStringTreeOptions read GetOptions write SetOptions;
  end;

function colToStr(col: TColumnName): string;
procedure Register;

implementation

uses  mpgtools, mp3names, fieldunit, shuffleUnit, scanoptionunit,
uDbg
;

function colToStr(col: TColumnName): string;
begin
  case col of
    cnName: result := 'Name';
    cnPath: result := 'Path';
    cnDuration: result := 'Duration';
    cnCount: result := 'Count';
    cnSize: result := 'Size';
    cnPlayed: result := 'Played';
    cnPlaycnt: result := 'Playcnt';
    cnTagged: result := 'Tagged';
    cnYear: result := 'Year';
    cnArtist: result := 'Artist';
    cnAlbum: result := 'Album';
    cnTitle: result := 'Title';
    cnComment: result := 'Comment';
    cnGenre: result := 'Genre';
    cnTrack: result := 'Track';
    cnBitrate: result := 'Bitrate';
    cnBpm: result := 'BPM';
    cnMode: result := 'Mode';
    cnStyle: result := 'Style';
  end;
end;
function colToAlign(col: TColumnName):TAlignment;
begin
  case col of
    cnName: result := taLeftJustify;
    cnPath: result := taLeftJustify;
    cnDuration: result := taRightJustify;
    cnCount: result := taRightJustify;
    cnSize: result := taRightJustify;
    cnPlayed: result := taRightJustify;
    cnPlaycnt: result := taRightJustify;
    cnTagged: result := taRightJustify;
    cnYear: result := taLeftJustify;
    cnArtist: result := taLeftJustify;
    cnAlbum: result := taLeftJustify;
    cnTitle: result := taLeftJustify;
    cnComment: result := taLeftJustify;
    cnGenre: result := taLeftJustify;
    cnTrack: result := taRightJustify;
    cnBitrate: result := taRightJustify;
    cnBpm: result := taRightJustify;
    cnMode: result := taRightJustify;
    cnStyle: result := taLeftJustify;
  end;
end;

function sizetostr(size: int64): string;
var str: string;
begin
  if size > 10240 then
  begin
    size := size shr 10;
    str := 'kb';
    if size > 10240 then
    begin
      size := size shr 10;
      str := 'Mb';
      if size > 10240 then
      begin
        size := size shr 10;
        str := 'Gb';
      end;
    end;
  end;
  result := inttostr(size) + ' ' + str;
end;

function secondsToString(seconds: integer): string;
  function twodigit(val:integer):string;
  begin
    if val < 10 then
      result := '0' + inttostr(val)
    else
      result := inttostr(val);
  end;
var
  hr, min, sec: integer;
begin
  sec := seconds mod 60;
  min := seconds div 60;
  if min > 60 then
  begin
    hr := min div 60;
    min := min mod 60;
    result := inttostr(hr) + ':' + twodigit(min) + ':' +
      twodigit(sec);
  end
  else
  begin
    result := twodigit(min) + ':' + twodigit(sec);
  end;
end;

function booleantochar(val: boolean): char;
begin
  result := 'F';
  if val then result := 'T';
end;

function chartoboolean(val: char): boolean;
begin
  result := false;
  if val = 'T' then result := true;
end;

procedure VTTrackTreeView.saveSettingsReg;
var
  reg: TRegistry;
  str: string;
  i: integer;
begin
  for i := 1 to columncnt do
    colwidth[i - 1] := header.Columns[i].Width;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey('\Software\Mp3tool', True)
      then
    begin
      str := inttostr(integer(columns[0])) + ',' +
        inttostr(colwidth[0]);
      for i := 1 to columncnt - 1 do
        str := str + ',' + inttostr(integer(columns[i])) + ',' + inttostr(colwidth[i]);
      Reg.WriteString('Columns', str);
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

procedure VTTrackTreeView.loadSettingsReg;
var
  reg: TRegistry;
  str, val: string;
  i, k: integer;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey('\Software\Mp3tool', False)
      then
    begin
      str := Reg.ReadString('Columns');
      // read the columns
      i := pos(',', str);
      k := 0;
      while (i > 0) and (k < 19) do
      begin
        val := copy(str, 1, i - 1);
        columns[k] := TColumnName(strtoint(val));
        str := copy(str, i + 1, 1000);
        i := pos(',', str);
        try
          if i < 1 then
            i := length(str) + 1;
          val := copy(str, 1, i - 1);
          colwidth[k] := strtoint(val);
          str := copy(str, i + 1, 1000);
          i := pos(',', str);
        except
          i := 0;
          colwidth[k] := 50;
        end;
        inc(k);
      end;
      columncnt := k;
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

procedure VTTrackTreeView.Debug(str: string);
begin
  if assigned(FOnDebug) then
    FOnDebug(self, str);
end;

function VTTrackTreeView.DebugStr(node: PVIrtualNode): string;
var p: PNodeData;
begin
  result := '??';
  if not assigned(node) then
    result := 'node is nil'
  else
  begin
    p := GetNodeData(node);
    if p.nodeType = ntSum then
      result := 'Sum: ' + TSumObject(p.obj).name
    else if p.nodeType = ntTag then
      result := 'Tag: ' + TTagObject(p.obj).filename
    else
      result := 'Wrong type';
  end;
end;

procedure VTTrackTreeView.ShowInfo(node: PVirtualNode; x, y: integer);
var data: PNodeData;
  str: string;
begin
  if assigned(node) then
  begin
    data := GetNodeData(node);
    if data.nodeType = ntTag then
    else
    begin
      str := str + 'played: ' + inttostr(TSumObject(data.obj).played);
      str := str + ' playcnt: ' + inttostr(TSumObject(data.obj).playcnt);
      application.messagebox(pchar(str), 'Info', MB_OK);
    end;
  end;
end;

procedure VTTrackTreeView.Played(filename: string);
var i: integer;
  src: TSearchRec;
  str: string;
  dir: string;
  done: boolean;
  cnt: integer;
begin
  scanner.update_playcount(filename);
  invalidate;
end;

function  VTTrackTreeView.GetTagObject(filename: string): TTagObject;
begin
  result := scanner.GetTagObject(filename);
end;



procedure VTTrackTreeView.ScannerReady(Sender: TObject);
begin
  save;
//  if length(FUserName) > 0 then
//    saveuserdata('vt' + FUserName + '.dta');
  if assigned(FOnScanReady) then
    FOnScanReady(self);
end;

procedure VTTrackTreeView.ScanProgress(str: string);
begin
  if assigned(FOnScanProgress) then
    FOnScanProgress(self, str);
end;

procedure VTTrackTreeView.scan;
var text_in: textfile;
  regel: string;
  tag: TTagObject;
  i: integer;
  scanoptions : Tscanoptionform;
begin
  scanoptions := Tscanoptionform.Create(owner);
  if scanoptions.showmodal = idok then
  begin
  RootNodeCount := 0;
  screen.cursor := crHourglass;
  try
    scanner.OnProgress := scanProgress;
    if assigned(dirs) then
      scanner.setDirs(dirs);
    basesum := scanner.basesum;
    scanner.ScanDir(scanoptions.cbScanDir.checked,
      scanoptions.cbScanMp3.checked,scanoptions.cbMpg.checked);
  except;
  end;
  RootNodeCount := 1; //basesum.childs.count;
  screen.cursor := crDefault;
  end;
  scanoptions.free;
end;

procedure VTTrackTreeView.Click(Sender: TObject);
begin
// uitzoeken hoe selectie gaat
{  if (selected <> nil) then
  begin
    if not (selected.data = nil) then
    begin
      if (integer(selected.data) > 0) then
      begin
        if assigned(FOnTrackCLick) then
          FOnTrackCLick(self, TStringObject(selected.data).str);
      end;
    end;
  end;}
end;

procedure VTTrackTreeView.WMLButtonDblClk(var Message: TWMLButtonDblClk);
var
  HitInfo: THitInfo;
  p: PNodeData;
begin
  inherited;
  GetHitTestInfoAt(Message.XPos, Message.YPos, true, HitInfo);
  if hiOnItemLabel in HitInfo.HitPositions then
  begin
    DblCLick(HitInfo.HitNode);
  end;
end;

procedure VTTrackTreeView.save;
var i, index: integer;
  filenames: textfile;
begin
  if assigned(scanner) then
  if loaded then
    scanner.Save(settingsdir + '\vtdata.dta');
end;


procedure VTTrackTreeView.loadDefault;
begin
  load(settingsdir + '\vtdata.dta');
end;

procedure VTTrackTreeView.load(filename: string);
begin
  scanner.load(filename);
  clear;
  basesum := scanner.basesum;
  rootnodecount := 1;
  SortTree(FSortColumn, FSortDirection, false);
  Header.SortColumn := FSortColumn;
  Header.SortDirection := FSortDirection;
  loaded := true;
end;

procedure VTTrackTreeView.MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  NewItem: TMenuItem;
  hit: THitInfo;
begin
  if (ssAlt in Shift) and (Button = mbRight) then
  begin
    GetHitTestInfoAt(x, y, true, hit);
    ShowInfo(hit.HitNode, x, y);
  end
  else if Button = mbRight then
  begin
    if not assigned(FPopupMenu) then
    begin
      FPopupmenu := TPopupMenu.Create(parent);

      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'scan';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);

      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'load';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);

      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'save';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);

      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'info';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);

      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'sum genre';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);
      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'sum artist';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);
      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'sum style';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);

      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'field chooser';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);

      NewItem := TMenuItem.Create(self);
      NewItem.caption := 'play options';
      NewItem.OnClick := popuphandler;
      FPopupmenu.items.add(NewItem);

    end;
    Fpopupmenu.Popup(parent.left + x, parent.top + y);
  end;
end;

procedure VTTrackTreeView.popuphandler(Sender: TObject);
begin
  if pos('genre', TMenuItem(sender).caption) > 0 then
  begin
    scanner.SumGenre;
    rootnodecount := scanner.bases.count + 1;
  end
  else if pos('artist', TMenuItem(sender).caption) > 0 then
  begin
    scanner.SumArtist;
    rootnodecount := scanner.bases.count + 1;
  end
  else if pos('tyle', TMenuItem(sender).caption) > 0 then
  begin
    scanner.SumStyle;
    rootnodecount := scanner.bases.count + 1;
  end
  else if pos('can', TMenuItem(sender).caption) > 0 then scan
  else if pos('oad', TMenuItem(sender).caption) > 0 then loadDefault
  else if pos('ave', TMenuItem(sender).caption) > 0 then save
  else if pos('ield chooser', TMenuItem(sender).caption) > 0 then fields
  else if pos('options', TMenuItem(sender).caption) > 0 then showoptions;
//  FPopupmenu.Destroy;
//  FPopupmenu := nil;
end;

function VTTrackTreeView.GetFirstSelectedNode: string;
var p: PNodeDAta;
  node: PVirtualNode;
begin
  result := '';
  if selectedcount > 0 then
  begin
    currentselected := GetFirstSelected;
    getcurrentindex := -1;
    if assigned(currentselected) then
    begin
      getcurrentindex := 0;
      p := GetNodeData(currentSelected);
      if p.nodeType = ntTag then
      begin
        result := TTagObject(p.obj).fullName;
        currentSel := TTagObject(p.obj);
      end
      else
      begin
        if (currentselected.ChildCount > 0) or
          (vsHasChildren in currentselected.States)
          then
        begin
          node := GetFirstChild(currentselected);
          p := GetNodeData(node);
          if p.nodeType = ntTag then
          begin
            currentselected := node;
            result := TTagObject(p.obj).fullName;
            currentSel := TTagObject(p.obj);
          end
          else
          begin
            getcurrentindex := 0;
            result := getNextSelectedNode;
          end;
        end;
      end;
    end;
  end;
  Debug('first selected: ' + result);
end;

function VTTrackTreeView.GetNextSelectedNode: string;
var node: PVirtualNode;
  p: PNodeData;
  stop: boolean;
begin
  result := '';
  p := getNodeData(currentselected);
  if p.nodeType = ntSum then
  begin
    currentselected := GetFirstChild(currentselected);
    p := getNodeData(currentselected);
    if p.nodeType = ntTag then
    begin
      result := TTagObject(p.obj).fullName;
      currentSel := TTagObject(p.obj);
    end
    else
      Result := GetNextSelectedNode;
  end
  else
  begin
    if not selected[currentselected] then
    begin
      node := GetNextSibling(currentselected);
      if assigned(node) then
      begin
        currentselected := node;
        p := getNodeData(node);
        if p.nodeType = ntTag then
        begin
          result := TTagObject(p.obj).fullName;
          currentSel := TTagObject(p.obj);
        end
        else
          Result := GetNextSelectedNode;
      end
      else
      begin
        stop := false;
        while not stop do
        begin
          node := currentselected.parent;
          if not Selected[node] then
          begin
            node := GetNextSibling(node);
            if assigned(node) then
            begin
              currentselected := node;
              p := getNodeData(node);
              stop := true;
              if p.nodeType = ntTag then
              begin
                result := TTagObject(p.obj).fullName;
                currentSel := TTagObject(p.obj);
              end
              else
                Result := GetNextSelectedNode;
            end
            else
              currentselected := currentselected.parent;
          end
          else
            stop := true;
        end;
      end;
    end;
  end;
  Debug('next selected: ' + result);
end;

function VTTrackTreeView.GetCurrentSize: LongInt;
begin
  result := currentSel.filelength;
end;

function VTTrackTreeView.GetCurrentDuration: integer;
begin
  result := currentSel.Duration;
end;

function VTTrackTreeView.GetCurrentArtist: string;
begin
  result := currentSel.Artist;
end;

function VTTrackTreeView.GetCurrentAlbum: string;
begin
  result := currentSel.Album;
end;

function VTTrackTreeView.GetCurrentTitle: string;
begin
  result := currentSel.Title;
end;

function VTTrackTreeView.FindNextInTree: boolean;
var next, down: PVirtualNode;
  continue: boolean;
  p: PNodeData;
begin
  if assigned(currentnode) then
  begin
    next := GetNextSibling(currentnode);
    debug('next: ' + debugstr(next));
    if assigned(next) then
    begin
      Selected[currentnode] := false;
      DblClick(next);
    end
    else
    begin
      continue := true;
      next := currentnode;
      while continue do
      begin
        down := next.parent;
        if down = RootNode then
          down := nil;
        debug('down: ' + debugstr(down));
        if assigned(down) then
        begin
          next := GetNextSibling(down);
          debug('next: ' + debugstr(next));
          if assigned(next) then
          begin
            p := GetNodeData(next);
            if p.nodetype = ntSum then
            begin
              continue := false;
            end;
          end
          else
            next := down;
        end
        else
          continue := false;
      end;
      if assigned(next) then
      begin
        p := GetNodeData(next);
        if p.nodeType = ntSum then
        begin
          continue := true;
          while continue do
          begin
            next := GetFirstChild(next);
            debug('first child: ' + debugstr(next));
            if assigned(next) then
            begin
              p := GetNodeDAta(next);
              if p.nodeType = ntTag then
                continue := false
            end
            else
              continue := false;
          end;
        end;
      end;
      if assigned(next) then
      begin
        debug('dblclk: ' + debugstr(next));
        Selected[currentnode] := false;
        DblClick(next);
      end;
    end;
  end;
end;

function VTTrackTreeView.Donext: boolean;
var atag: TTagObject;
  i: integer;
  ok: boolean;
  it: integer;

begin
  case playmode of
    normal:
      begin
        FindNextInTree;
      end;
    shuffle:
      begin
        it := 0;
        repeat
          i := random(scanner.files.count - 1);
          atag := TTagObject(scanner.files.Objects[i]);
          ok := true;
          if FNotPlayed then ok := (atag.PlayCnt = 0);
          if FGenreFilter >= 0 then ok := ok and (atag.Genre = FGenreFilter);
          if length(FStyleFilter) > 0 then
            ok := (ok and (pos(FStyleFilter,lowercase(atag.style))>0));
          inc(it);
        until ok or (it > 1000); // if not found after 1000 iterations then we have to find another way
        if not ok then
        begin
          i := -1;
          repeat
            inc(i);
            atag := TTagObject(scanner.files.Objects[i]);
            ok := true;
            if FNotPlayed then ok := (atag.PlayCnt = 0);
            if FGenreFilter >= 0 then ok := ok and (atag.Genre = FGenreFilter);
            if length(FStyleFilter) > 0 then
              ok := (ok and (pos(FStyleFilter,lowercase(atag.style))>0));
          until ok or (i = scanner.files.count - 1);
        end;
        if ok and assigned(FOnTrackDblCLick) then
        begin
          FOnTrackDblCLick(self, atag.path + '\' + atag.filename);
          currentNode := nil;
          currentindex := i;
        end;
      end;
  end;
end;

procedure VTTrackTreeView.DblClick(node: PVirtualNode);
var
  p: PNodeData;
begin
  p := GetNodeData(node);
  if p.nodetype = ntTag then
  begin
    if assigned(FOnTrackDblCLick) then
    begin
      FOnTrackDblCLick(self, TTagObject(p.obj).path + '\' + TTAgObject(p.obj).filename);
      currentNode := Node;
      currentindex := scanner.files.IndexOfObject(TObject(GetNodeData(currentnode)));
      Selected[currentnode] := true;
      IsVisible[currentnode] := true;
    end;
  end;
  if p.obj = basesum then FullExpand;
end;

function VTTrackTreeView.DoPrev: boolean;
var index: integer;
  prev, down: PVirtualNode;
  p: PNodeData;
  continue: boolean;
begin
  result := false;
  if assigned(currentnode) then
  begin
    prev := GetPreviousSibling(currentnode); // probeer op huidige nivo
    if assigned(prev) then
    begin
      p := GetNodeData(prev);
      if p.nodeType = ntSum then
      begin
        debug('doprev, found direct sibling');
      end;
      Selected[currentnode] := false;
      DblClick(prev);
    end
    else
    begin
      continue := true;
      prev := currentnode;
      while continue do
      begin
        down := GetPrevious(prev);
        debug('down: ' + debugstr(down));
        if assigned(down) then
        begin
          prev := GetPreviousSibling(down);
          debug('prev: ' + debugstr(prev));
          if assigned(prev) then
          begin
            p := GetNodeData(prev);
            if p.nodetype = ntSum then
            begin
              continue := false;
            end;
          end
          else
            prev := down;
        end
        else
          continue := false;
      end;
      if assigned(prev) then
      begin
        p := GetNodeData(prev);
        if p.nodeType = ntSum then
        begin
          continue := true;
          while continue do
          begin
            prev := GetLastChild(prev);
            debug('last child: ' + debugstr(prev));
            if assigned(prev) then
            begin
              p := GetNodeDAta(prev);
              if p.nodeType = ntTag then
                continue := false
            end
            else
              continue := false;
          end;
        end;
      end;
      if assigned(prev) then
      begin
        debug('dblclick: ' + debugstr(prev));
        Selected[currentnode] := false;
        DblClick(prev);
      end;
    end
  end;
end;

procedure VTTrackTreeView.search(str: string; onfound: TTagEvent;
        muststop: TReqEvent; title,artist,album,all : boolean);
var i: integer;
  tag: TTagObject;
  srchstr: string;
  found : boolean;
begin
  if not loaded then
    loadDefault;
  srchstr := lowercase(str);

  for i := 0 to scanner.files.count - 1 do
  begin
    tag := TTagObject(scanner.files.objects[i]);
    found := false;
    if all then
      if pos(srchstr, lowercase(tag.fullName)) > 0 then
        found := true;
    if not found then
      if  title then
        if pos(srchstr, lowercase(tag.title)) > 0  then
          found := true;
    if not found then
      if  artist then
        if pos(srchstr, lowercase(tag.artist)) > 0  then
          found := true;
    if not found then
      if  album then
        if pos(srchstr, lowercase(tag.album)) > 0  then
          found := true;
    if found then
         onfound(tag);
    if muststop then abort;
  end;
end;

procedure VTTrackTreeView.SaveArtists(filename: string);
var list: TStringList;
begin
  if assigned(scanner) then
  begin
  list := TStringList.Create;
  scanner.artists(list);
  list.savetofile(filename);
  list.free;
  end;
end;

procedure VTTrackTreeView.SaveArtistsNoStyle(filename: string);
var list: TStringList;
begin
  if assigned(scanner) then
  begin
  list := TStringList.Create;
  scanner.artistsnostyle(list);
  list.savetofile(filename);
  list.free;
  end;
end;

procedure VTTrackTreeView.fields;
var colSelectForm: TcolSelectForm;
  i: integer;
begin
  colSelectForm := TcolSelectForm.Create(Owner);
  colSelectForm.ClearFields;
  for i := 0 to columncnt - 1 do
    colSelectForm.AddField(columns[i]);

  colSelectForm.ShowModal;

  columncnt := colSelectForm.GetFieldCount;
  for i := 0 to columncnt - 1 do
  begin
    columns[i] := colSelectForm.GetField(i);
    colwidth[i] := 50;
  end;
  colSelectForm.Free;
  SetColumns;
  invalidate;
end;

procedure VTTrackTreeView.showOptions;
var optform: TShuffleForm;
begin
  optForm := TShuffleForm.Create(Owner);
  optform.vtparent := self;
  optform.genre := FGenreFilter;
  optform.shuffle   := (FPlaymode = shuffle);
  optform.notplayed := FNotPlayed;
  optForm.Style     := FStyleFilter;
  optform.showmodal;
  optform.free;
end;


constructor VTTrackTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OnCLick := CLick;
  OnMouseDown := MouseDown;

  dragmode := dmAutomatic;
  dragtype := dtVCL;
  hint := 'Mp3 library';
  hintMode := hmHintandDefault;
  hintMode := hmTooltip;

  FUserName := 'info';
  loaded := false;
  currentnode := nil;
  currentindex := -1;
  NodeDataSize := sizeof(TNodeData);

  OnInitNode := InitNode;
  OnGetText := GetText;
  OnInitChildren := InitChildren;
  OnGetHint := GetHint;
  OnKeyPress := KeyPress;
  OnCompareNodes := CompareNodes;
  OnHeaderClick := HeaderClick;

//  if voToggleOnDblClick in options then
//    options := options - [voToggleOnDblClick];
//  if not (voAutoScroll in options) then
//    options := options + [voAutoScroll];
//  include(treeoptions,toAutoSort);
// first set default column setting
  columns[0] := cnSize;
  colwidth[0] := 70;
  columns[1] := cnCount;
  colwidth[1] := 70;
  columns[2] := cnGenre;
  colwidth[2] := 50;
  columncnt := 3;
// then try to load from registry
  loadSettingsReg;
// and now initialize the headers
  SetColumns;
  scanner := TMp3Data.Create;
  scanner.OnProgress := ScanProgress;
  scanner.OnRefreshTree := RefreshTree;
  styles  := scanner.styles;
  initcnt := 0;

  FGenreFilter := -1;
  FStyleFilter := '';
end;

procedure VTTrackTreeView.SetColumns;
var
  i: integer;
  col: TVirtualTreeCOlumn;
begin
  header.columns.clear;
  col := header.Columns.Add;
  col.Text := 'Path';
  FSOrtColumn := 0;

  for i := 0 to columncnt - 1 do
  begin
    col := header.Columns.Add;
    col.Text := coltostr(columns[i]);
    col.width := colwidth[i];
    col.Alignment := colToAlign(columns[i]);
  end;
  header.columns[0].Width := 200;
  header.Options :=
  ([hoVisible,
    hoColumnResize,
//    hoAutoResize,
//    hoDblClickResize,
//    hoShowHint,
//    hoDblClickResize,
    hoShowSortGlyphs]);
end;

destructor VTTrackTreeView.Destroy;
var i: integer;
begin
  saveSettingsReg;
  if assigned(scanner) then
  begin
    save;
    scanner.destroy;
  end;
  inherited Destroy;
end;

procedure VTTrackTreeView.CompareNodes(Sender: TBaseVirtualTree; Node1, Node2: PVirtualNode;
  Column: Integer; var Result: Integer);
var
  p1: PNodeData;
  p2: PNodeData;
  ps1: TSumObject;
  ps2: TSumObject;
  pt1: TTagObject;
  pt2: TTagObject;
  str1, str2: string;
begin

  p1 := sender.GetNodeData(Node1);
  p2 := sender.GetNodeData(Node2);
  if p1.nodeType = ntSum then
  begin
    if p2.nodeType = ntSum then // both sum, so we have to compare sums
    begin
      ps1 := TSumObject(p1.obj);
      ps2 := TSumObject(p2.obj);
      result := 0;
      if column > 0 then
      begin
        str1 := ps1.getFieldVal(columns[column - 1]);
        str2 := ps2.getFieldVal(columns[column - 1]);
        result := comparestr(str1, str2);
      end;
      if result = 0 then
        result := comparestr(ps1.name, ps2.name);
    end
    else
      result := -1;
  end
  else
  begin
    if p2.nodeType = ntTag then // both sum, so we have to compare sums
    begin
      pt1 := TTagObject(p1.obj);
      pt2 := TTagObject(p2.obj);
      result := 0;
      if column > 0 then
      begin
        str1 := pt1.getFieldVal(columns[column - 1]);
        str2 := pt2.getFieldVal(columns[column - 1]);
        result := comparestr(str1, str2);
      end;
      if result = 0 then
        result := comparestr(pt1.filename, pt2.filename);
    end
    else
      result := 1;
  end;


end;

procedure VTTrackTreeView.InitNode(Sender: TBaseVirtualTree;
  ParentNode, Node: PVirtualNode;
  var InitialStates: TVirtualNodeInitStates);
var
  p: PNodeData;
  parent: TSumObject;
  str : string;
begin
  if ParentNode = nil then
  begin
    if Node.Index <= scanner.bases.count then
    begin
      p := Sender.GetNodeData(node);
      p.nodeType := ntSum;
      str := 'sum';
      if node.index = 0 then
        p.obj := TSumObject(basesum)
      else if node.index <= scanner.bases.count then
      begin
        p.obj := TSumObject(scanner.bases[node.index-1]);
      end;

      if TSumObject(p.obj).childs.count > 0 then
        Include(InitialStates, ivsHasChildren);
    end;
  end
  else
  begin
    p := sender.GetNodeData(node);
    if p.nodeType = ntSum then
    if TSumObject(p.obj).childs.count > 0 then
      Include(InitialStates, ivsHasChildren);
  end;

{  begin
    p := Sender.GetNodeData(Parentnode);
    if p.nodeType = ntSum then
    begin
      parent := TSumObject(p.obj);
      if node.index < parent.childs.Count then
      begin
        p := Sender.GetNodeData(node);
        if parent.childs.objects[node.index] is TSumObject then
        begin
          str := 'sum';
          p.nodeType := ntSum;
          p.obj := TSumObject(parent.childs.objects[Node.index]);
          if TSumObject(p.obj).childs.count > 0 then
            Include(InitialStates, ivsHasChildren);
        end
        else
        begin
          p.nodeType := ntTag;
          str := 'tag';
          p.obj := parent.childs.objects[Node.index];
          Exclude(InitialStates, ivsHasChildren);
        end;
      end;
    end;
  end;}
  //debugger.logmsg('initnode: ' + inttohex(integer(node),8) + ' ' + str);
  inc(initcnt);
end;

procedure VTTrackTreeView.GetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
  var Text: WideString);
var
  p: PNodeData;
begin
  p := sender.GetNodeData(Node);
  if p.nodetype = ntSum then
  begin
    if Column <= 0 then
      Text := TSumObject(p.obj).name
    else
      Text := TSumObject(p.obj).getField(columns[column - 1]);
  end
  else if p.nodetype = ntTag then
  begin
    if Column <= 0 then
      Text := TTagObject(p.obj).filename
    else
      Text := TTagObject(p.obj).getField(columns[column - 1]);
  end;
end;

procedure VTTrackTreeView.InitChildren(Sender: TBaseVirtualTree;
  Node: PVirtualNode; var ChildCount: Cardinal);
var
  p: PNodeData;
  newnode: PVirtualNode;
  newp: PNodeData;
  i: integer;
begin
  p := Sender.GetNodeData(node);
  if p.nodeType = ntSum then
  begin
    childcount := TSumObject(p.obj).childs.count;
    with TSumObject(p.obj) do
    begin
      for i := 0 to childs.count-1 do
      begin
        newnode := AddChild(node);
        newp := GetNodeData(newnode);
        newp.obj := childs.Objects[i];
        if childs.Objects[i] is TSumObject then
        begin
          newp.nodeType := ntSum;
        end
        else
          newp.nodeType := ntTag;
      end;
    end;
  end;
end;

procedure VTTrackTreeView.GetHint(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer; TextType: TVSTTextType;
  var Text: WideString);
var
  p: PNodeData;
  tag: TTagObject;
begin
  p := Sender.GetNodeData(node);
  if p.nodeType = ntTag then
  begin
    tag := TTagObject(p.obj);
    Text := tag.artist + chr(13) + tag.album + chr(13) + tag.title;
  end
  else
    Text := 'Dit is geen tag';

end;

procedure VTTrackTreeView.KeyPress(Sender: TObject; var Key: Char);
begin
  case Key of
    'a':
      begin
//    Report doubles
        application.messagebox('pressed a', 'keypress', MB_OK);
      end;
    'd':
      begin
        scanner.free;
        scanner := nil;
      end;
  end;
end;

procedure VTTrackTreeView.HeaderClick(Sender: TVTHeader; Column: Integer);
begin
  if FSortColumn = Column then
  begin
    if FSortDirection = sdAscending then
      FSortDirection := sdDescending
    else
      FSortDirection := sdAscending;
  end
  else
  begin
  end;
  debugger.logmsg('before sort: ' +inttostr(initcnt));
  SortTree(Column, FSortDirection, false);
  debugger.logmsg('after sort: ' +inttostr(initcnt));
  FSortColumn := Column;
  Header.SortColumn := FSortColumn;
  Header.SortDirection := FSortDirection;
end;

procedure VTTrackTreeView.RefreshTree(sender:TObject);
begin
  RootNodeCount := 0;
  RootNodeCount := 1;
//  ReInitNode(RootNode,true);
//  ReInitChildren(RootNode,true);
end;

procedure VTTrackTreeView.SetUserName(const Value: string);
begin
  FUserName := value;
  scanner.UserName := value;
end;

function VTTrackTreeView.GetOptions: TStringTreeOptions;
begin
  Result := inherited TreeOptions as TStringTreeOptions;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure VTTrackTreeView.SetOptions(const Value: TStringTreeOptions);

begin
  inherited TreeOptions.Assign(Value);
end;

//----------------------------------------------------------------------------------------------------------------------

function VTTrackTreeView.GetOptionsClass: TTreeOptionsClass;
begin
  Result := TStringTreeOptions;
end;


procedure Register;
begin
  RegisterComponents('Virtual Controls', [VTTrackTreeView]);
end;

end.

