unit TrackListView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Mp3TrackData;

type
  TStrEvent = procedure(sender: TObject; str: string) of object;
  TSizeEvent = procedure(sender: TObject; size: int64) of object;

  TStringObject = class(TObject)
    str: string;
    duration: integer;
    size : integer;
    destructor Destroy; override;
  end;

  TTrackListView = class(TCustomListView)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FOnTrackClick: TStrEvent;
    FOnTrackDblClick: TStrEvent;
    FOnDurationChange: TStrEvent;
    FOnSizeChange: TSizeEvent;
    FUseTags: boolean;
    FSortable: boolean;
    FAutoFirstSelect: boolean;
    ColumnToSort: integer;
    FFilename: string;
    selIndex: integer;
    currentplaying: integer;

    procedure Click(Sender: TObject);
    procedure DblClick(Sender: TObject);
    procedure ColumnClick(Sender: TObject; Column: TListColumn);
    procedure Compare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure DragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure SetTrackProperties(item: TListItem; filename: string);
    procedure LoadM3U(f: TStringList);
    procedure AddTrackExtint(extint, filename: string);
    procedure Deletion(Sender: TObject; Item: TListItem);
    procedure UpdateDuration;
    procedure UpdateSize;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddTrack(filename: string);
    procedure AddTrackTag(tag: TTagObject);
    procedure AddTrackProp(filename: string; fsize, duration : integer; artist,album,title:string);
    procedure AddDirectory(dirname: string);
    procedure ClearList;
    procedure Load(filename: string);
    procedure SaveAs(filename: string);
    procedure Save;
    function GetFirstSelected: string;
    function GetNextSelected: string;
    procedure Rewind;
    function Donext: boolean;
    function DoPrev: boolean;
    function count : integer;

  protected
    procedure KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

  published
    { Published declarations }
    property OnTrackClick: TStrEvent read FOnTrackClick write FOnTrackClick;
    property OnTrackDblClick: TStrEvent read FOnTrackDblClick write FOnTrackDblClick;
    property OnDurationChange: TStrEvent read FOnDurationChange write FOnDurationChange;
    property OnSizeChange: TSizeEvent read FOnSizeChange write FOnSizeChange;
    property UseTags: boolean read FUseTags write FUseTags default true;
    property Sortable: boolean read FSortable write FSortable default true;
    property AutoFirstSelect: boolean read FAutoFirstSelect
      write FAutoFirstSelect default true;
    property Anchors;
    property Color;
    property Font;
    property Align;
  end;

procedure Register;

implementation

uses mpgtools, mp3names, jbstr, stdctrls, {tracktreeview,} VTrackTreeView;

procedure Register;
begin
  RegisterComponents('Virtual Controls', [TTrackListView]);
end;


destructor TStringObject.Destroy;
begin
  inherited;
end;

constructor TTrackListView.Create(AOwner: TComponent);
var col: TListColumn;
  totalwidth: integer;
  i: integer;
begin
  inherited Create(AOwner);
  col := Columns.Add;
  col.Caption := 'Title';
  col.Width := 100;

  col := Columns.Add;
  col.Caption := 'Artist';
  col.Width := 100;

  col := Columns.Add;
  col.Caption := 'Album';
  col.Width := 100;

  col := columns.add;
  col.caption := 'time';
  col.width := 40;

  totalwidth := 0;
  for i := 0 to columns.count - 1 do
    totalwidth := totalwidth + columns[i].width;

  width := totalwidth;
  viewStyle := vsReport;
  OnCLick := CLick;
  OnDblCLick := DblClick;
  OnColumnClick := ColumnCLick;
  OnCompare := Compare;

  dragmode := dmAutomatic;
  OnDragDrop := DragDrop;
  OnDragOver := DragOver;

  OnKeyDown := KeyDown;
  OnDeletion := Deletion;

  FUseTags := true;
  FSortable := true;
  MultiSelect := true;
  currentplaying := -1;
  FAutoFirstSelect := true;
  HideSelection := false;
end;

destructor TTrackListView.Destroy;
begin
  ClearList;
  inherited;
end;

procedure TTrackListView.ClearList;
var i : integer;
begin
// for i := 0 to items.count-1 do
//   TStringObject(items[i].data).free;
  items.clear;
end;

procedure TTrackListView.Click(Sender: TObject);
begin
  if assigned(FOnTrackClick) then
  begin
    if assigned(selected) then
      FOnTrackCLick(self, TStringObject(selected.data).str);
  end;
end;

function TTrackListView.count : integer;
begin
  result := items.count;
end;

procedure TTrackListView.DblClick(Sender: TObject);
begin
  if assigned(FOnTrackDblClick) then
  begin
    if assigned(selected) then
      FOnTrackDblCLick(self, TStringObject(selected.data).str);
  end;
  currentplaying := items.indexof(selected);
end;

procedure TTrackListView.ColumnClick(Sender: TObject; Column: TListColumn);
begin
  if FSortable then
  begin
    ColumnToSort := Column.Index;
    AlphaSort;
  end;
end;

procedure TTrackListView.KeyDOwn(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = 46 then
  begin
    if selcount = 1 then
      items.Delete(items.indexof(selected));
  end;
end;


procedure TTrackListView.Compare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
  ix: Integer;
begin
  if ColumnToSort = 0 then
    Compare := CompareText(Item1.Caption, Item2.Caption)
  else begin
    ix := ColumnToSort - 1;
    Compare := CompareText(Item1.SubItems[ix], Item2.SubItems[ix]);
  end;
end;

procedure TTrackListView.DragDrop(Sender, Source: TObject; X, Y: Integer);
var
  item: TListItem;
  index: integer;
  str: string;
begin
  item := GetItemAt(x, y);
  if (source = self) then
  begin
    str := TStringObject(selected.data).str;
//    TStringObject(selected.data).free;
    delete(selected);
    index := GetItemIndex(item);
    item := items.Insert(index);
    setTrackProperties(item, str);
  end
  else
  begin
    if source is TTrackListView then
    begin
      str := TTrackListView(source).GetFirstSelected;
      while length(str) > 0 do
      begin
        AddTrack(str);
        str := TTrackListView(source).GetNextSelected;
      end;
    end
{    else if source is TTrackTreeView then
    begin
      with TTrackTreeView(source) do
      begin
        str := GetFirstSelected;
        while length(str) > 0 do
        begin
          AddTrack(str);
          str := GetNextSelected;
        end;
      end;
    end}
    else if source is VTTrackTreeView then
    begin
      with VTTrackTreeView(source) do
      begin
        str := GetFirstSelectedNode;
        while length(str) > 0 do
        begin
          AddTrackProp(str,GetCurrentSize,GetCurrentDuration,
             GetCurrentArtist,GetCurrentAlbum,GetCurrentTitle);
          str := GetNextSelectedNode;
        end;
      end;
    end;
  end;
end;

procedure TTrackListView.DragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  if Sender is TTrackListView then
    accept := true;
end;

procedure TTrackListView.AddTrackExtint(extint, filename: string);
var
  item: TListItem;
  artist, album, title: string;
  duration: integer;
  pd, pc: integer;
  strobj: TStringObject;
begin
  item := items.Add;
  duration := getDurationFromExtint(extint);
  pd := pos('-', extint);
  pc := pos(',', extint);
  if pd > 0 then
  begin
    title := removeTrackNumber(copy(extint, pd + 1, 1000));
    artist := copy(extint, pc + 1, pd - pc - 1);
  end
  else
  begin
    title := removeTrackNUmber(copy(extint, pc + 1, 1000));
    artist := get_Artist(filename);
  end;

  album := get_album(filename);

  strobj := TStringObject.create;
  strobj.str := filename;
  strobj.duration := duration;
  strobj.size := 0;
  item.caption := title;
  item.subitems.Add(artist);
  item.subitems.add(album);
  item.subitems.add(secondstostring(duration));
  item.Data := pointer(strobj);
end;

procedure TTrackListView.Deletion(Sender: TObject; Item: TListItem);
begin
  TStringObject(item.data).free;
  item.data := nil;
  UpdateDuration;
  UpdateSize;
end;


procedure TTrackListView.AddTrack(filename: string);
var
  item: TListItem;
begin
  item := items.Add;
  SetTrackProperties(item, filename);
  UpdateDuration;
  UpdateSize;
end;

procedure TTrackListView.AddTrackTag(tag: TTagObject);
var
  item: TListItem;
begin
  AddTrackProp(tag.fullname,tag.FileLength,tag.duration,tag.artist,tag.album,tag.title);
{  item := items.Add;
  SetTrackProperties(item, filename);
  UpdateDuration;
  UpdateSize;}
end;

procedure TTrackListView.AddTrackProp(filename: string; fsize, duration : integer; artist,album,title:string);
var
  item: TListItem;
  strobj: TStringObject;
begin
  item := items.Add;

  strobj          := TStringObject.create;
  strobj.str      := filename;
  strobj.duration := duration;
  strobj.size     := fsize;
  item.caption    := title;
  item.subitems.Add(artist);
  item.subitems.add(album);
  item.subitems.add(secondstostring(duration));
  item.Data := pointer(strobj);

  UpdateDuration;
  UpdateSize;
end;


procedure TTrackListView.UpdateDuration;
var zero: boolean;
  i, duration: integer;
begin
  zero := false;
  if assigned(FOndurationChange) then
  begin
    duration := 0;
    for i := 0 to items.count - 1 do
    begin
      if assigned(items[i].data) then
      if TStringObject(items[i].data).duration = 0 then
        zero := true
      else
        if TStringObject(items[i].data).duration > 0 then
          duration := duration + TStringObject(items[i].data).duration;
    end;
    if zero then
      FOnDurationCHange(self, secondsToString(duration) + ' +')
    else
      FOnDurationCHange(self, secondsToString(duration));
  end;
end;

procedure TTrackListView.UpdateSize;
var size : int64;
  i: integer;
begin
  if assigned(FOnSizeChange) then
  begin
    size := 0;
    for i := 0 to items.count - 1 do
    begin
      if assigned(items[i].data) then
      if TStringObject(items[i].data).size > 0 then
          size := size + TStringObject(items[i].data).size;
    end;
    FOnSizeChange(self, size);
  end;
end;

procedure TTrackListView.SetTrackProperties(item: TListItem; filename: string);
var done: boolean;
  mpg: TMpegAudio;
  title, artist, album, time: string;
  duration, size: integer;
  tracknumber: integer;
  strobj: TStringObject;
begin
  done := false;
  duration := 0;
  size := 0;
  if FUseTags then
  begin
    if fileexists(filename) then
    begin
      mpg := TMpegAudio.Create;
      mpg.filename := filename;
      if mpg.isTagged then
      begin
        title := mpg.Title;
        artist := mpg.Artist;
        album := mpg.Album;
        tracknumber := mpg.Track;
        done := true;
      end;
      size := mpg.FileLength;
      duration := mpg.duration;
      mpg.free;
    end;
  end;
  if length(title) = 0 then
    title := get_trackname(filename);
  if length(artist) = 0 then
    artist := get_Artist(filename);
  if length(album) = 0 then
    album := get_album(filename);
  strobj          := TStringObject.create;
  strobj.str      := filename;
  strobj.duration := duration;
  strobj.size     := size;
  item.caption    := title;
  item.subitems.Add(artist);
  item.subitems.add(album);
  item.subitems.add(secondstostring(duration));
  item.Data := pointer(strobj);
end;

procedure TTrackListView.LoadM3U(f: TStringList);
var i: integer;
  extint: string;
  oldtags: boolean;
begin
  oldtags := FUseTags;
  FUseTags := false;
  for i := 1 to f.count - 1 do
  begin
    if copy(f[i], 1, 8) = '#EXTINF:' then
    begin
      extint := f[i];
    end
    else
    begin
      if length(extint) > 0 then
      begin
        addTrackExtint(extint, f[i]);
        extint := '';
      end
      else
      begin
        addTrack(f[i]);
      end;
    end;
  end;
  FUseTags := oldtags;
  UpdateDuration;
end;

procedure TTrackListView.Load(filename: string);
var
  f: TStringList;
  i: integer;
begin
  FFilename := filename;
  f := TStringList.create;
  f.loadfromfile(filename);
  if f[0] = '#EXTM3U' then
    LoadM3U(f)
  else
    application.messagebox('Invalid file format', 'Load playlist', MB_OK);
  f.free;
end;

procedure TTrackListView.SaveAs(filename: string);
begin
  FFilename := filename;
  Save;
end;

procedure TTrackListView.Save;
var
  f: TStringList;
  i: integer;
  strobj: TStringObject;
begin
  f := TStringList.create;
  f.Add('#EXTM3U');
  for i := 0 to items.count - 1 do
  begin
    strobj := TStringObject(items[i].data);
    if strobj.duration > 0 then
    begin
      f.add('#EXTINF:' + inttostr(strobj.duration) + ',' +
        items[i].subitems[0] + ' - ' + items[i].caption);
    end;
    f.add(strobj.str);
  end;
  f.SaveToFile(FFilename);
end;

procedure TTrackListView.AddDirectory(dirname: string);
var
  search: Tsearchrec;
  result: integer;
begin
  result := findfirst(dirname + '\*.*', faanyfile, search);
  while result = 0 do
  begin
    if search.attr = fadirectory then
    begin
      if (search.Name <> '.') and
        (search.name <> '..') then
      begin
        AddDirectory(dirname + '\' + search.name);
      end;
    end
    else
    begin
      if justextension(search.name) = 'mp3' then
      begin
        AddTrack(dirname + '\' + search.name);
      end;
    end;
    result := findnext(search);
  end;
  sysutils.findclose(search);
end;

function TTrackListView.GetFirstSelected: string;
begin
  selIndex := -1;
  if assigned(selected) then
  begin
    selIndex := getItemIndex(selected);
    result := TStringObject(selected.data).str;
  end;
end;

function TTrackListView.GetNextSelected: string;
var found: boolean;
begin
  result := '';
  if selIndex >= 0 then
  begin
    found := false;
    inc(selIndex);
    while (not found) and (selIndex < items.count) do
    begin
      if items[selIndex].Selected then
      begin
        found := true;
        result := TStringObject(items[selIndex].data).str;
      end
      else
        inc(selIndex);
    end;
    if not found then
      selIndex := -1;
  end;
end;

procedure TTrackListView.Rewind;
begin
  currentPlaying := -1;
end;

function TTrackListView.Donext: boolean;
var index: integer;
begin
  result := false;
  index := currentplaying; //items.IndexOf(selected);
  if (index < items.count - 1) and (index >= 0) then
  begin
    items[currentplaying].Selected := false;
    selected := items[index + 1];
    DblClick(Self);
    result := true;
  end
  else
  begin
    if FAutoFirstSelect and (index = -1) and (items.count > 0) then
    begin
      selected := items[0];
      DblClick(Self);
      result := true;
    end;
  end;
end;

function TTrackListView.DoPrev: boolean;
var index: integer;
begin
  result := false;
  index := currentplaying;
  if (index > 0) and (index < items.count) then
  begin
    items[currentplaying].Selected := false;
    selected := items[index - 1];
    DblClick(Self);
    result := true;
  end;
end;

end.

