unit ListUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,  Grids, mpgtools;

type
  TListForm = class(TForm)
    dg: TDrawGrid;
    Button1: TButton;
    btnTag: TButton;
    artistleft: TCheckBox;
    split: TCheckBox;
    sep: TEdit;
    Label1: TLabel;
    Button3: TButton;
    cmbGenre: TComboBox;
    btnGenre: TButton;
    edtArtist: TEdit;
    btnArtist: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure dgDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure dgDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure dgDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
    procedure btnTagClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure dgGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: string);
    procedure dgSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: string);
    procedure FormShow(Sender: TObject);
    procedure btnGenreClick(Sender: TObject);
    procedure btnArtistClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    mpglist: TMPEGAudioList;
    init: boolean;
  public
    { Public declarations }
  end;

var
  ListForm: TListForm;

implementation

{$R *.DFM}
uses VTrackTreeView, mp3names;

procedure TListForm.FormCreate(Sender: TObject);
begin
  mpglist := TMPEGAudioList.Create;
  init := true;
end;

procedure TListForm.FormDragDrop(Sender, Source: TObject; X, Y: Integer);
var str: string;
begin
  if Source is VTTrackTreeView then
  begin
    with VTTrackTreeView(source) do
    begin
      str := GetFirstSelectedNode;
      while length(str) > 0 do
      begin
        mpglist.AddFile(str);
        str := GetNextSelectedNode;
      end;
      if mpglist.count > 1 then
      begin
        dg.rowcount := mpglist.count + 1;
        if length(edtArtist.text) = 0 then
          edtArtist.text := mpglist.items[0].artist;
      end;
    end;
  end;
end;

procedure TListForm.dgDragDrop(Sender, Source: TObject; X, Y: Integer);
var str: string;
begin
  if Source is VTTrackTreeView then
  begin
    with VTTrackTreeView(source) do
    begin
      str := GetFirstSelectedNode;
      while length(str) > 0 do
      begin
        mpglist.AddFile(str);
        str := GetNextSelectedNode;
      end;
      if mpglist.count > 1 then
      begin
        dg.rowcount := mpglist.count + 1;
        if length(edtArtist.text) = 0 then
          edtArtist.text := mpglist.items[0].artist;
      end;
    end;
  end;
  dg.invalidate;
end;

procedure TListForm.dgDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  accept := false;
  if source is VTTrackTreeView then
    accept := true;
end;

procedure TListForm.dgDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  value: string;
begin
  if AROw = 0 then
    case ACol of
      0: Value := 'filename';
      1: Value := 'artist';
      2: Value := 'title';
      3: Value := 'album';
      4: Value := 'genre';
    end
  else
    if ARow <= mpglist.count then
      case ACol of
        0: begin
           Value := extractfilename(mpglist.items[ARow - 1].filename);
           value := copy(value,1,length(value)-4);
           end;
        1: Value := mpglist.items[ARow - 1].artist;
        2: Value := mpglist.items[ARow - 1].title;
        3: Value := mpglist.items[ARow - 1].album;
        4: Value := mpglist.items[ARow - 1].GenreStr;
      end;
  dg.Canvas.TextRect(rect, rect.left + 2, rect.top + 2, value);
end;

procedure TListForm.btnTagClick(Sender: TObject);
var
  i: integer;
  str, left, right: string;
  p: integer;
begin
  for i := 0 to mpglist.count - 1 do
  begin
    // tag from name
    with mpglist.items[i] do
    begin
      if split.checked then
      begin
        str := get_trackname(filename);
        p := pos(sep.text, str);
        if (p > 0) then
        begin
          left := trim(copy(str, 1, p - 1));
          right := trim(copy(str, p + 1, length(str)));
          if artistleft.checked then
          begin
            artist := left;
            title := right;
          end
          else
          begin
            artist := right;
            title := left;
          end;
        end
        else
        begin
          title := str;
          artist := '';
        end;
      end
      else
      begin
        Artist := get_artist(filename);
        Title := get_trackname(filename);
      end;
      Album := get_album(filename);
{      if cbCapitalize.checked then
      begin
        Artist := capitalizeword(artist);
        Title := CapitalizeWord(Title);
        Album := CapitalizeWord(Album);
      end;}
    end;
  end;
  dg.Invalidate;
end;

procedure TListForm.Button1Click(Sender: TObject);
var i: integer;
begin
  for i := 0 to mpglist.count - 1 do
  begin
    mpglist.items[i].WriteTag;
  end;
end;

procedure TListForm.Button3Click(Sender: TObject);
begin
  mpglist.Clear;
  dg.rowcount := 2;
  dg.invalidate;
  edtArtist.text := '';
end;

procedure TListForm.FormDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  if source is VTTrackTreeView then
    accept := true;
end;

procedure TListForm.dgGetEditText(Sender: TObject; ACol, ARow: Integer;
  var Value: string);
begin
  if ARow > 0 then
    if ARow <= mpglist.count then
      case ACol of
        0: Abort;
        1: Value := mpglist.items[ARow - 1].artist;
        2: Value := mpglist.items[ARow - 1].title;
        3: Value := mpglist.items[ARow - 1].album;
      end;
end;

procedure TListForm.dgSetEditText(Sender: TObject; ACol, ARow: Integer;
  const Value: string);
begin
  case ACol of
    1: mpglist.items[ARow - 1].artist := value;
    2: mpglist.items[ARow - 1].title := value;
    3: mpglist.items[ARow - 1].album := value;
  end;

end;

procedure TListForm.FormShow(Sender: TObject);
var i: integer;
begin
  if init then
  begin
    init := false;
    for i := 0 to MaxStyles do
      cmbGenre.Items.AddObject(musicstyle[i], TObject(i));
  end;
end;

procedure TListForm.btnGenreClick(Sender: TObject);
var i : integer;
    newGenre : byte;
begin
  newGenre := byte(cmbGenre.items.Objects[cmbGenre.ItemIndex]);
  for i := 0 to mpglist.count - 1 do
  begin
    mpglist.items[i].genre := newGenre;
  end;
  dg.Invalidate;
end;

procedure TListForm.btnArtistClick(Sender: TObject);
var i : integer;
begin
  for i := 0 to mpglist.count - 1 do
  begin
    mpglist.items[i].artist := edtArtist.text;
  end;
  dg.Invalidate;
end;

procedure TListForm.FormDestroy(Sender: TObject);
begin
//  mpglist.free;
end;

end.

