{
        MP3Demo 0.1.1

        This is a demo for the VTTrackTreeView component

        It is a mp3 player which can handle large number of mp3 files
        specific features:
          stores mp3 info in local files (directory content)
          handles playcount for each user
          has additional attributes stored in the local files
          search capabilities
          playlists

        History

        13-01-2001
          0.1.1 added BPM tool, click on bpm value to open

        11-01-2001
          0.1.0 first public release

        remarks, bugs, ideas can be sent to

        fbouwmans@spiditel.nl

        Frans Bouwmans, the netherlands
}

unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, VTrackTreeView, TrackListView, Playlistunit, StdCtrls,
  XaudioPlayer, ComCtrls, ExtCtrls, slider, mp3trackdata;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    PlayListBox: TListBox;
    Button1: TButton;
    btnDirect: TButton;
    btnSearch: TButton;
    Xaudio: TXaudioPlayer;
    btnLib: TButton;
    btnOptions: TButton;
    btnTagEdit: TButton;
    Panel1: TPanel;
    lblPlayTime: TLabel;
    tbVolume: TTrackBar;
    btnPause: TButton;
    btnPlay: TButton;
    btnPrev: TButton;
    btnStop: TButton;
    btnNext: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    lblArtist: TLabel;
    lblAlbum: TLabel;
    lblTitle: TLabel;
    lblGenre: TLabel;
    lblStyle: TLabel;
    Label6: TLabel;
    lblPlayCnt: TLabel;
    Label7: TLabel;
    btnMute: TButton;
    TrackImage: TImage;
    ThumbNormalImage: TImage;
    ThumbClickedImage: TImage;
    ThumbHighlightedImage: TImage;
    Label8: TLabel;
    lblBpm: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure tbVolumeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnLibClick(Sender: TObject);
    procedure btnOptionsClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnPrevClick(Sender: TObject);
    procedure btnDirectClick(Sender: TObject);
    procedure btnSearchClick(Sender: TObject);
    procedure XaudioNotifyPlayerState(Sender: TXaudioPlayer;
      State: Integer);
    procedure XaudioNotifyInputTimecode(Sender: TXaudioPlayer; Hours,
      Minutes, Seconds, Fractions: Byte);
    procedure XaudioNotifyInputPosition(Sender: TXaudioPlayer; Offset,
      Range: Cardinal);
    procedure btnTagEditClick(Sender: TObject);
    procedure btnMuteClick(Sender: TObject);
    procedure XaudioNotifyStreamDuration(Sender: TXaudioPlayer;
      Duration: Cardinal);

    procedure SliderChange(Sender: TObject);
    procedure SliderMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SliderMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure lblBpmMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);

  private
    { Private declarations }
    init:       boolean;
    FPlaying:   boolean;

    activeList: TTrackListView;
    CurrentSong: string;
    totalTime:   string;

    playerstate: integer;

    playedList: TTrackListView;
    directList: TTrackListView;

    PlayChanging: boolean;
    AutoChange:   boolean;
    storedVolume: integer;
    Slider: TSlider;
    Tracking: boolean;
    tag: TTagObject;

  public
    { Public declarations }
    lib:        VTTrackTreeView;

    procedure PlaySong(filename: string);
    function  GetUser:string;
    function  playing: boolean;
    procedure next;
    procedure prev;

    procedure ActivateList(trackList: TTrackListView);
    procedure NamePlayList(form: TPlayListForm; aname: string);
    procedure RemovePlaylistForm(form: TPlayListForm);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}
uses directunit, searchunit, optionunit, treeunit,
  ListUnit,  mp3names, BpmUnit;

function PlayerStateToStr(state: integer): string;
begin
  case state of
    XA_PLAYER_STATE_STOPPED: result := 'stopped';
    XA_PLAYER_STATE_PLAYING: result := 'playing';
    XA_PLAYER_STATE_PAUSED: result := 'paused';
    XA_PLAYER_STATE_EOF: result := 'eof';
  end;
end;

function secondsToString(seconds: integer): string;
  function twodigit(val:integer):string;
  begin
    if val < 10 then
      result := '0' + inttostr(val)
    else
      result := inttostr(val);
  end;
var
  hr, min, sec: integer;
begin
  sec := seconds mod 60;
  min := seconds div 60;
  if min > 60 then
  begin
    hr := min div 60;
    min := min mod 60;
    result := inttostr(hr) + ':' + twodigit(min) + ':' +
      twodigit(sec);
  end
  else
  begin
    result := twodigit(min) + ':' + twodigit(sec);
  end;
end;


procedure TMainForm.PlaySong(filename: string);
begin
  tag := nil;
  if fileexists(filename) then
  begin
    if playerstate = XA_PLAYER_STATE_PLAYING then
      xaudio.Stop;
    xaudio.inputopen(filename);
    xaudio.play;
    FPlaying := true;
    CurrentSong := filename;
    Caption := 'Playing ' + currentsong;
    tag := lib.GetTagObject(filename);
    if assigned(tag) then
    begin
      lblArtist.Caption := tag.artist;
      lblAlbum.caption  := tag.album;
      lblTitle.Caption  := tag.title;
      lblGenre.Caption  := genre2string(tag.genre);
      lblStyle.Caption  := tag.style;
      lblPlayCnt.Caption := inttostr(tag.playcnt);
      lblBpm.Caption    := '--';
      if  tag.bpm > 0 then
        lblBpm.Caption    := inttostr(tag.bpm)
    end
    else
    begin
      lblArtist.Caption := '--';
      lblAlbum.caption  := '--';
      lblTitle.Caption  := '--';
      lblGenre.Caption  := '--';
      lblStyle.Caption  := '--';
      lblBpm.Caption    := '--';
    end;
  end;
end;

procedure TMainForm.Button1Click(Sender: TObject);
var
  PlayList : TPlayListForm;
begin
  PlayList := TPlayListForm.Create(self);
  PlayList.Caption := 'playlist' + inttostr(PlayListBox.items.count);
  PlayListBox.items.AddObject(PlayList.Caption,PlayList);
  PlayList.show;
end;

procedure TMainForm.RemovePlaylistForm(form: TPlayListForm);
var i : integer;
begin
  if not (csDestroying in ComponentState) then
  begin
  i := Playlistbox.items.IndexOfObject(form);
  if i >= 0 then
    PlayListBox.Items.Delete(i);
  end;
end;

procedure TMainForm.NamePlayList(form: TPlayListForm; aname: string);
var i : integer;
begin
  i := Playlistbox.items.IndexOfObject(form);
  if i >= 0 then
    PlayListBox.Items[i] := aname;
end;

procedure TMainForm.ActivateList(trackList: TTrackListView);
begin
  activelist := tracklist;
end;

procedure TMainForm.next;
var done : boolean;
begin
  done := false;
  FPlaying := false;
  if assigned(activelist) then
    done := activelist.Donext;
  if (not done) and assigned(directlist) then
    done := directlist.Donext;
  if (not done) and assigned(lib) then
    lib.Donext;
end;

procedure TMainForm.prev;
var done:boolean;
begin
  FPlaying := false;
  done := false;
  if assigned(activelist) then
    done := activelist.Doprev;
  if  (not done) and assigned(directlist) then
    done := directlist.Doprev;
  if (not done) and assigned(lib) then
    lib.Doprev;
end;


function TMainForm.Playing:boolean;
begin
  result := FPlaying;
end;

procedure TMainForm.tbVolumeChange(Sender: TObject);
begin
  xaudio.SetOutputVolume(50,tbVolume.Position,100);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  init := true;
  Xaudio.Init;
  Slider := TSlider.Create(self);
  Slider.Parent := self;
  Slider.ThumbNormal := ThumbNormalImage.Picture.Bitmap;
  Slider.ThumbHighlighted := ThumbHighlightedImage.Picture.Bitmap;
  Slider.ThumbClicked := ThumbClickedImage.Picture.Bitmap;
  Slider.Track := TrackImage.Picture.Bitmap;
  Slider.Minimum := 0;
  Slider.Maximum := 400;
  Slider.Left := 8 + panel1.left;
  Slider.Top := 118;
  Slider.Width := panel1.width - 16;
  Slider.Height := Slider.Track.Height+3;
  Slider.OnChange    := SliderChange;
  Slider.OnMouseDown := SliderMouseDown;
  Slider.OnMouseUp   := SliderMouseUp;
  Tracking := False;
end;

function TMainForm.GetUser:string;
var
   UserName : PChar;
   NameSize : DWORD;
begin
   result := '';
   UserName := #0;
   NameSize := 50;
   try
      GetMem(UserName, NameSize);
      Windows.GetUserName(UserName, NameSize);
      result := StrPas(UserName);
   finally
      FreeMem(UserName);
   end;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  if init then
  begin
    lib := treeform.lib;
    searchform.tracktreeview := lib;
    lib.username := getuser;
    treeform.lib.SettingsDir := ExtractFilePath(paramstr(0));
    directlist := directform.tl;
    lib.dirs := optionform.dirs;
    init := false;
    tbVolume.position := 62;
    tbVolume.Position := 63;
  end;                            

end;

procedure TMainForm.btnLibClick(Sender: TObject);
begin
  treeform.Show;
end;

procedure TMainForm.btnOptionsClick(Sender: TObject);
begin
  optionform.show;
end;

procedure TMainForm.btnStopClick(Sender: TObject);
begin
  xaudio.Stop;
end;

procedure TMainForm.btnPlayClick(Sender: TObject);
begin
  xaudio.play;
end;

procedure TMainForm.btnPauseClick(Sender: TObject);
begin
  xaudio.Pause;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if playerstate = XA_PLAYER_STATE_PLAYING then
  begin
    xaudio.SetOutputVolume(50,0,100);
    xaudio.stop;
  end;
end;

procedure TMainForm.btnNextClick(Sender: TObject);
begin
  next;
end;

procedure TMainForm.btnPrevClick(Sender: TObject);
begin
  prev;
end;

procedure TMainForm.btnDirectClick(Sender: TObject);
begin
  directform.Show;
end;

procedure TMainForm.btnSearchClick(Sender: TObject);
begin
  searchform.show;
end;

procedure TMainForm.XaudioNotifyPlayerState(Sender: TXaudioPlayer;
  State: Integer);
var str : string;
begin
  playerstate := state;
  caption := playerstatetostr(state) + currentsong;
  FPlaying := (state = XA_PLAYER_STATE_PLAYING);
  if state = XA_PLAYER_STATE_EOF then
  begin
    str := currentsong;
    next;
    lib.played(str);
  end;
end;

procedure TMainForm.XaudioNotifyInputTimecode(Sender: TXaudioPlayer; Hours,
  Minutes, Seconds, Fractions: Byte);
begin
  if (hours > 0) then
    lblPlayTime.Caption := Format('%.2d:%.2d:%.2d', [Hours, Minutes, Seconds])
       + ' (' + totalTime + ')'
  else
    lblPlayTime.Caption := Format('%.2d:%.2d', [Minutes, Seconds])
      + ' (' +totalTime + ')';
end;

procedure TMainForm.XaudioNotifyInputPosition(Sender: TXaudioPlayer;
  Offset, Range: Cardinal);
begin
  if Tracking then Exit;
    Slider.Position := Offset * (Range div (Slider.Maximum - Slider.Minimum));
end;

procedure TMainForm.SliderChange(Sender: TObject);
begin
  Xaudio.InputSeek(Slider.Position, Slider.Maximum);
end;

procedure TMainForm.SliderMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  Tracking := True;
end;

procedure TMainForm.SliderMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  Tracking := False;
end;


procedure TMainForm.btnTagEditClick(Sender: TObject);
begin
  treeform.left   := 0;
  treeform.show;
  listform.left   := treeform.left + treeform.width;
  listform.height := treeform.height;
  listform.top    := treeform.top;
  listform.show;
end;

procedure TMainForm.btnMuteClick(Sender: TObject);
begin
  if tbVolume.position = 0 then
    tbVolume.position := storedVolume
  else
  begin
    storedVolume := tbVolume.position;
    tbVolume.position := 0;
  end;
end;

procedure TMainForm.XaudioNotifyStreamDuration(Sender: TXaudioPlayer;
  Duration: Cardinal);
begin
  totalTime :=  secondstostring(round(Duration));
end;

procedure TMainForm.lblBpmMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
 bTag: TTagObject;
begin
  if assigned(tag) then
  begin
    bTag := tag;
    bpmForm.Bpm := btag.bpm;
    if bpmFOrm.showmodal = IDOK then
    begin
      btag.bpm := bpmForm.Bpm;
      btag.saveParent;
      lblBpm.Caption := inttostr(btag.bpm);
    end;
  end;
end;

end.
