unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VirtualTrees, StdCtrls, ActiveX, ImgList;

type
  PMyData = ^TMyData;
  TMyData = packed record
    Column1,
    Column2,
    Column3,
    Column4,
    Column5: string;
    ImageIndex1,
    ImageIndex2,
    ImageIndex3,
    ImageIndex4,
    ImageIndex5: integer;
  end;

type
  TForm1 = class(TForm)
    VST: TVirtualStringTree;
    ImageList1: TImageList;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure VSTInitNode(Sender: TBaseVirtualTree; ParentNode,
      Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
    procedure VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer; TextType: TVSTTextType; var Text: WideString);
    procedure VSTGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: Integer;
      var Ghosted: Boolean; var Index: Integer);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
  Column: TVirtualTreeColumn;
begin
  { Reserve data in each node for the data }
  VST.NodeDataSize := SizeOf(TMyData);
  { Set up 50 Nodes, note they don't care about any data yet }
  VST.RootNodeCount := 50;

  { This all can be done at design time too in the Object inspector }
  VST.Header.Options := VST.Header.Options + [hoVisible];
  Column := VST.Header.Columns.Add;
  Column.Text := 'Column 1';
  Column.Width := 120;
  Column := VST.Header.Columns.Add;
  Column.Text := 'Column 2';
  Column.Width := 120;
  Column := VST.Header.Columns.Add;
  Column.Text := 'Column 3';
  Column.Width := 120;
  Column := VST.Header.Columns.Add;
  Column.Text := 'Column 4';
  Column.Width := 120;
  Column := VST.Header.Columns.Add;
  Column.Text := 'Column 5';
  Column.Width := 120;
end;

procedure TForm1.VSTInitNode(Sender: TBaseVirtualTree; ParentNode,
  Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
var
  MyData: PMyData;
begin
  { Here is where we finally set up the node since VT needs to do something     }
  { with it.                                                                    }

  { Get a pointer to the data we reserved }
  MyData := PMyData( VST.GetNodeData(Node));
  { Since VT memorys allocation is ignorant of dynamic strings and arrays we    }
  { must call Initialize and Finalize to ensure property creation and           }
  { destruction of the strings or arrays.                                       }
  Initialize(MyData^);
  MyData.Column1 := 'Column 1';
  MyData.Column2 := 'Column 2';
  MyData.Column3 := 'Column 3';
  MyData.Column4 := 'Column 4';
  MyData.Column5 := 'Column 5';
  MyData.ImageIndex1 := 0;
  MyData.ImageIndex2 := 1;
  MyData.ImageIndex3 := 2;
  MyData.ImageIndex4 := 3;
  MyData.ImageIndex5 := 4;

     { If the nodes InitalState can be set here too. }
     {  ivsDisabled,                                 }
     {  ivsExpanded,                                 }
     {  ivsHasChildren,                              }
     {  ivsSelected                                  }
     { You must respond to the OnInitChildren and    }
     { return the number of children the node has    }
     { to allow the node to expand.                  }
//  InitialStates := [ivsHasChildren];

     { Also the checkboxes are set here too.         }
     { ** Make sure toCheckSupport is selected in    }
     {                TreeOptions *****              }
     {    ctNone,                                    }
     {    ctTriStateCheckBox,                        }
     {    ctCheckBox,                                }
     {    ctRadioButton,                             }
     {    ctButton                                   }
//  Node.CheckType := ctCheckBox;

     { If checkboxes are used then the state of the checkbox can be set as well }
     {     csUncheckedNormal,  // unchecked and not pressed           }
     {     csUncheckedPressed, // unchecked and pressed               }
     {     csCheckedNormal,    // checked and not pressed             }
     {     csCheckedPressed,   // checked and pressed                 }
     {     csMixedNormal,      // 3-state check box and not pressed   }
     {     csMixedPressed      // 3-state check box and pressed       }
//  Node.CheckState := csCheckedNormal
end;

procedure TForm1.VSTGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: Integer; TextType: TVSTTextType; var Text: WideString);
var
  MyData: PMyData;
begin
  { This is were VT need to display the node, it is call many, many, many times }
  { so cache this information.                                                  }

  { Get a pointer to the data we reserved and initialized in OnInitNode }
  MyData := PMyData( VST.GetNodeData(Node));

  case Column of
   -1: Text := MyData.Column1; {If columns are not used then VT uses -1 for the tree}
    0: Text := MyData.Column1;
    1: Text := MyData.Column2;
    2: Text := MyData.Column3;
    3: Text := MyData.Column4;
    4: Text := MyData.Column5;
  end;
end;

procedure TForm1.VSTGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: Integer;
  var Ghosted: Boolean; var Index: Integer);
var
  MyData: PMyData;
begin
  { This is were VT need to display the image, it is call many, many, many times }
  { so cache this information.                                                   }

  { Get a pointer to the data we reserved and initialized in OnInitNode }
  MyData := PMyData( VST.GetNodeData(Node));

  { Make sure you are responding the the right image kind.  If you reply to the }
  { Overlay and the index does not exist in the imagelist weird things happen.  }
  {      ikNormal,         }
  {      ikSelected,       }
  {      ikState,          }
  {      ikOverlay         }

  if (Kind = ikNormal) or (Kind = ikSelected) then
    case Column of
     -1: Index := MyData.ImageIndex1; {If columns are not used then VT uses -1 for the tree}
      0: Index := MyData.ImageIndex1;
      1: Index := MyData.ImageIndex2;
      2: Index := MyData.ImageIndex3;
      3: Index := MyData.ImageIndex4;
      4: Index := MyData.ImageIndex5;
    end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  MyData: PMyData;
  Node: PVirtualNode;
  i: integer;
begin
  { VT is not good for linear lookups.  It is a linked list so it is better to  }
  { try to think about how find you target node through using data stored while }
  { building the tree.  One way for Listview type tree is to create an array    }
  { of PVirtualNode pointers.  The array can be filled in during the InitNode   }
  { calls.  That way and absolute index to a line can be directly indexed to    }
  { array and the node read directly.  This way we have to walk the node chain  }
  { to find index 10.                                                           }

  { Freeze any drawing }
  VST.BeginUpdate;
  try
    i := 0;
    Node := VST.GetFirst;
    while (i < 9) and Assigned(Node) do
    begin
      Node := Node.NextSibling;
      Inc(i);
    end;

    if Assigned(Node) then
    begin
      { Get a pointer to the data we reserved and initialized in OnInitNode }
      MyData := PMyData( VST.GetNodeData(Node));
      MyData.Column1 := 'New Text Column 1';
      MyData.Column2 := 'New Text Column 2';
      MyData.Column3 := 'New Text Column 3';
      MyData.Column4 := 'New Text Column 4';
      MyData.Column5 := 'New Text Column 5';
      MyData.ImageIndex1 := Random(4);
      MyData.ImageIndex2 := Random(4);
      MyData.ImageIndex3 := Random(4);
      MyData.ImageIndex4 := Random(4);
      MyData.ImageIndex5 := Random(4);
    end;

  finally
    VST.EndUpdate
  end
end;

end.


