Unit mainunit;

{$INCLUDE ZipVers.inc}

Interface

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  StdCtrls,
  ZipMstr,
  Menus,
{$IFDEF VERD4+} // if Delphi v4.xx or higher
  VirtualTrees,
  ComCtrls,
  ImageListEx,
  ActnList,
{$ENDIF}
  FileCtrl, Dialogs;

{$IFDEF VERD2D3} // if not Delphi v4.xx
Type
  Int64 = Comp; // 64 bit integers are supported differently by Delphi 2 and 3
{$ENDIF}

Type
  TMainForm = Class(TForm)
    ActionAddToZip: TAction;
    ActionConvertToEXE: TAction;
    ActionDestroyFile: TAction;
    ActionExtract: TAction;
    ActionFileClose: TAction;
    ActionFileExit: TAction;
    ActionFileNew: TAction;
    ActionFileOpen: TAction;
    ActionListAll: TActionList;
    ActionOptionsTrace: TAction;
    ActionOptionsUnattended: TAction;
    ActionOptionsVerbose: TAction;
    ActionRemove: TAction;
    ActionRename: TAction;
    ActionShowMessages: TAction;
    ActionTestZip: TAction;
    ActionViewDLLVersionInfo: TAction;
    ActionViewLastError: TAction;
    ActionViewZipComment: TAction;
    MainMenu1: TMainMenu;
    MainMenu: TMenuItem;
    MainMenuAddToZip: TMenuItem;
    MainMenuBlank1: TMenuItem;
    MainMenuBlank2: TMenuItem;
    MainMenuBlank3: TMenuItem;
    MainMenuCloseZip: TMenuItem;
    MainMenuConverttoEXE: TMenuItem;
    MainMenuDestroyZip: TMenuItem;
    MainMenuExit: TMenuItem;
    MainMenuExtract: TMenuItem;
    MainMenuNewZip: TMenuItem;
    MainMenuOpenZip: TMenuItem;
    MainMenuOptions: TMenuItem;
    MainMenuOptionsTrace: TMenuItem;
    MainMenuOptionsUnattended: TMenuItem;
    MainMenuOptionsVerbose: TMenuItem;
    MainMenuRemove: TMenuItem;
    MainMenuRename: TMenuItem;
    MainMenuShowMesages: TMenuItem;
    MainMenuTestZip: TMenuItem;
    MainMenuView: TMenuItem;
    MainMenuViewDLLVersionInfo: TMenuItem;
    MainMenuViewShowLasterror: TMenuItem;
    MainMenuViewZipComment: TMenuItem;
    OpenDialog1: TOpenDialog;
    PopupMenuContent: TPopupMenu;
    PopupMenuContentExtract: TMenuItem;
    PopupMenuContentRemove: TMenuItem;
    PopupMenuContentRename: TMenuItem;
    StatusBar1: TStatusBar;
    VTree1: TVirtualStringTree;
    ZipMaster1: TZipMaster;
    Procedure DeleteZipButClick(Sender: TObject);
    Procedure ZipMaster1DirUpdate(Sender: TObject);
    Procedure FormCreate(Sender: TObject);
    Procedure VTreeFillData;
    Procedure ZipMaster1Message(Sender: TObject; ErrCode: Integer; Message: String);
    Procedure ZipMaster1Progress(Sender: TObject; ProgrType: ProgressType; FileName: String; FileSize: Integer);
    Procedure FormDestroy(Sender: TObject);
    Procedure MainMenuViewDLLVersionInfoClick(Sender: TObject);
    Procedure VTree1GetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: Integer; TextType: TVSTTextType; Var Text: WideString);
    Procedure VTree1CompareNodes(Sender: TBaseVirtualTree; Node1, Node2: PVirtualNode; Column: Integer; Var Result: Integer);
    Procedure VTree1PaintText(Sender: TBaseVirtualTree; Const Canvas: TCanvas; Node: PVirtualNode; Column: Integer; TextType: TVSTTextType);
    Procedure VTree1FocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: Integer);
    Procedure ActionAddToZipExecute(Sender: TObject);
    Procedure ActionExtractExecute(Sender: TObject);
    Procedure ActionRemoveExecute(Sender: TObject);
    Procedure ActionRenameExecute(Sender: TObject);
    Procedure ActionFileOpenExecute(Sender: TObject);
    Procedure ActionFileNewExecute(Sender: TObject);
    Procedure ActionTestZipExecute(Sender: TObject);
    Procedure ActionConvertToEXEExecute(Sender: TObject);
    Procedure ActionShowMessagesExecute(Sender: TObject);
    Procedure ActionViewLastErrorExecute(Sender: TObject);
    Procedure ActionViewZipCommentExecute(Sender: TObject);
    Procedure ActionViewDLLVersionInfoExecute(Sender: TObject);
    Procedure ActionOptionsVerboseExecute(Sender: TObject);
    Procedure ActionOptionsTraceExecute(Sender: TObject);
    Procedure ActionOptionsUnattendedExecute(Sender: TObject);
    Procedure ActionFileExitExecute(Sender: TObject);
    Procedure VTree1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    Procedure VTree1GetImageIndex(Sender: TBaseVirtualTree; Node: PVirtualNode; Kind: TVTImageKind; Column: Integer; Var Ghosted: Boolean; Var Index: Integer);
    procedure VTree1HeaderClick(Sender: TVTHeader; Column: Integer;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  Private
    SystemImageList1: TImageListEx;
    FUncompressedTotal: Cardinal;
    FCompressedTotal: Cardinal;
    FTotalSize1: Int64;
    FTotalProgress1: Int64;
    FTotalSize2: Int64;
    FTotalProgress2: Int64;
    Function ShowLTime(s, f: Longint): String;
    Procedure SetZipFName(aCaption: String; AssignName: Boolean);
  Public
    Function GetSpecialFolder(aFolder: Integer; Var Location: String): LongWord;
    Function AskDirDialog(Const FormHandle: HWND; Var DirPath: String): Boolean;
  End;

Var
  MainForm: TMainForm;
  ExtractDir: String;
  ExpandDirs: Boolean;
  OverWr: Boolean;
  AllFiles: Boolean;

Implementation

Uses
  ShlObj,
  extrunit,
  msgunit,
  addunit,
  sfxunit,
  renunit;

{$R *.DFM}

Procedure TMainForm.FormCreate(Sender: TObject);
Begin
  SystemImageList1 := TImageListEx.Create(Self);
  VTree1.Images := SystemImageList1;
  ZipMaster1.Load_Zip_Dll;
  ZipMaster1.Load_Unz_Dll;
  {If we had args on the cmd line, then try to open the first one as a zip/exe file.
  This is most useful in case user has an association to ".zip" that causes this program to run when user dble clicks on a zip file in Explorer.}
  If ParamCount > 0 Then ZipMaster1.ZipFilename := ParamStr(1);
End;

Procedure TMainForm.FormDestroy(Sender: TObject);
Begin
  ZipMaster1.Unload_Zip_Dll;
  ZipMaster1.Unload_Unz_Dll;
End;

Procedure TMainForm.DeleteZipButClick(Sender: TObject);
Var
  Answer1: Boolean;
Begin
  If FileExists(ZipMaster1.ZipFilename) Then Begin
    Answer1 := MessageDlg('Are you sure you want to delete: ' + ZipMaster1.ZipFilename + '?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
    If Answer1 Then Begin
      DeleteFile(ZipMaster1.ZipFilename);
      SetZipFName('', True);
    End Else Exit; { Don't use the new name }
  End Else ShowMessage('Zip file not found: ' + ZipMaster1.ZipFilename);
End;

Procedure TMainForm.MainMenuViewDLLVersionInfoClick(Sender: TObject);
Begin
End;

//***********************ZipMaster Event handling***************************

// This is the "OnMessage" event handler

Procedure TMainForm.ZipMaster1Message(Sender: TObject; ErrCode: Integer; Message: String);
Begin
  MsgForm.RichEdit1.Lines.Append(Message);
  PostMessage(MsgForm.RichEdit1.Handle, EM_SCROLLCARET, 0, 0);
  If (ErrCode > 0) And Not ZipMaster1.Unattended Then ShowMessage('Error Msg: ' + Message);
End;

Procedure TMainForm.ZipMaster1DirUpdate(Sender: TObject);
Begin
  VTreeFillData;
  StatusBar1.Panels[0].Text := 'Files: ' + IntToStr(ZipMaster1.Count);
  If UpperCase(ExtractFileExt(ZipMaster1.ZipFilename)) = '.EXE' Then ActionConvertToEXE.Caption := 'Con&vert to ZIP'
  Else ActionConvertToEXE.Caption := 'Con&vert to EXE';
End;

Procedure TMainForm.ZipMaster1Progress(Sender: TObject; ProgrType: ProgressType; Filename: String; FileSize: Integer);
Var
  Step: Integer;
Begin
  Case ProgrType Of
    TotalSize2Process: Begin
        MsgForm.StatusBar1.Panels.Items[0].Text := 'Total size: ' + IntToStr(FileSize Div 1024) + ' Kb';
        MsgForm.ProgressBar2.Position := 1;
        FTotalSize2 := FileSize;
        FTotalProgress2 := 0;
      End;
    TotalFiles2Process: MsgForm.StatusBar1.Panels.Items[1].Text := IntToStr(FileSize) + ' files';
    NewFile: Begin
        MsgForm.FileBeingZipped.Caption := Filename;
        MsgForm.ProgressBar1.Position := 1; // Current position of bar.
        FTotalSize1 := FileSize;
        FTotalProgress1 := 0;
      End;
    ProgressUpdate: Begin
        // FileSize gives now the bytes processed since the last call.
        FTotalProgress1 := FTotalProgress1 + FileSize;
        FTotalProgress2 := FTotalProgress2 + FileSize;
        If FTotalSize1 <> 0 Then Begin
{$IFDEF VERD4+} // D4+   (D5 gives a compiler error when using Int64 conversion!?)
          Step := MulDiv(FTotalProgress1, 10000, FTotalSize1);
{$ELSE} // D2 and D3
          Try
            Step := Round(FTotalProgress1 * 10000 / FTotalSize1);
          Except
            Step := 2147483647;
          End;
{$ENDIF}
          MsgForm.ProgressBar1.Position := 1 + Step;
        End Else MsgForm.ProgressBar1.Position := 10001;
        If FTotalSize2 <> 0 Then Begin
{$IFDEF VERD4+}
          Step := MulDiv(FTotalProgress2, 10000, FTotalSize2);
{$ELSE}
          Try
            Step := Round(FTotalProgress2 * 10000 / FTotalSize2);
          Except
            Step := 2147483647;
          End;
{$ENDIF}
          MsgForm.ProgressBar2.Position := 1 + Step;
        End;
      End;
    EndOfBatch: Begin // Reset the progress bar and filename.
        MsgForm.FileBeingZipped.Caption := '';
        MsgForm.ProgressBar1.Position := 1;
        MsgForm.StatusBar1.Panels[0].Text := '';
        MsgForm.StatusBar1.Panels[1].Text := '';
        MsgForm.ProgressBar2.Position := 1;
      End;
  End;
End;

//***********************User defined functions *****************************

Function TMainForm.ShowLTime(s, f: Longint): String;
Var
  min, sec, {mil,} st: Integer;
  smin, ssec {,smil}: String;
Begin
  st := f - s;
  //mil := st mod 1000;
  //mil := mil mod 60;
  sec := st Div 1000;
  min := sec Div 60;
  sec := sec Mod 60;
  //if mil > 9 then smil := IntToStr( mil ) else smil := '0' + IntToStr( mil );
  If sec > 9 Then ssec := IntToStr(sec) Else ssec := '0' + IntToStr(sec);
  If min > 9 Then smin := IntToStr(min) Else smin := '0' + IntToStr(min);
  Result := smin + ':' + ssec;
End;

Procedure TMainForm.SetZipFName(aCaption: String; AssignName: Boolean);
Begin
  // Assigning the filename will cause the table of contents to be read. and possibly reset it to an empty string (If error found).
  If AssignName Then ZipMaster1.ZipFilename := aCaption;
End;

Function TMainForm.AskDirDialog(Const FormHandle: HWND; Var DirPath: String): Boolean;
Var
  pidl: PItemIDList;
  FBrowseInfo: TBrowseInfo;
  Success: Boolean;
  TitleName: String;
  Buffer: Array[0..MAX_PATH] Of Char;
Begin
  Result := False;
  ZeroMemory(@FBrowseInfo, SizeOf(FBrowseInfo));
  Try
    GetMem(FBrowseInfo.pszDisplayName, MAX_PATH);
    FBrowseInfo.hwndOwner := FormHandle;
    TitleName := 'Please specify a directory';
    FBrowseInfo.lpszTitle := PChar(TitleName);
    pidl := ShBrowseForFolder(FBrowseInfo);
    If pidl <> Nil Then Begin
      Success := SHGetPathFromIDList(pidl, Buffer);
      // if False then pidl not part of namespace
      If Success Then Begin
        DirPath := Buffer;
        If DirPath[Length(DirPath)] <> '\' Then DirPath := DirPath + '\';
        Result := True;
      End;
      GlobalFreePtr(pidl);
    End;
  Finally
    If Assigned(FBrowseInfo.pszDisplayName) Then FreeMem(FBrowseInfo.pszDisplayName, Max_Path);
  End;
End;

{* Folder types are a.o.
 *	CSIDL_DESKTOPDIRECTORY, CSIDL_STARTMENU, CSIDL_SENDTO,
 * CSIDL_PROGRAMS, CSIDL_STARTUP etc.
 *}

Function TMainForm.GetSpecialFolder(aFolder: Integer; Var Location: String): LongWord;
Var
  pidl: PItemIDList;
  hRes: HRESULT;
  RealPath: Array[0..MAX_PATH] Of Char;
  Success: Boolean;
Begin
  Result := 0;
  hRes := SHGetSpecialFolderLocation(Handle, aFolder, pidl);
  If hRes = NO_ERROR Then Begin
    Success := SHGetPathFromIDList(pidl, RealPath);
    If Success Then Location := String(RealPath) + '\'
    Else
      Result := LongWord(E_UNEXPECTED);
  End Else
    Result := hRes;
End;

//**************************ListView functions **********************************
Type
  TPNodeData = ^TNodeData;
  TNodeData = Record
    Index: Integer;
    Path: String;
    FullPath: String;
  End;

Procedure TMainForm.VTree1PaintText(Sender: TBaseVirtualTree; Const Canvas: TCanvas; Node: PVirtualNode; Column: Integer; TextType: TVSTTextType);
Begin
  Canvas.Font.Color := clBlack;
  If vsSelected In Node.States Then Canvas.Font.Style := Canvas.Font.Style + [fsBold]
  Else Canvas.Font.Style := Canvas.Font.Style - [fsBold];
End;

Procedure TMainForm.VTree1CompareNodes(Sender: TBaseVirtualTree; Node1, Node2: PVirtualNode; Column: Integer; Var Result: Integer);
Var
  Ratio1: Integer;
  Ratio2: Integer;
  Data1: TPNodeData;
  Data2: TPNodeData;
  ZipEntry1: pZipDirEntry;
  ZipEntry2: pZipDirEntry;
Begin
  Result := 0;
  Data1 := TPNodeData(VTree1.GetNodeData(Node1));
  Data2 := TPNodeData(VTree1.GetNodeData(Node2));
  If (Data1.Index >= 0) And (Data2.Index >= 0) Then Begin
    ZipEntry1 := pZipDirEntry(ZipMaster1.ZipContents[Data1.Index]);
    ZipEntry2 := pZipDirEntry(ZipMaster1.ZipContents[Data2.Index]);
    Case Column Of
      0: Result := AnsiCompareText(Data1.FullPath, Data2.FullPath);
      1: Result := ZipEntry1.CompressedSize - ZipEntry2.CompressedSize;
      2: Result := ZipEntry1.UncompressedSize - ZipEntry2.UncompressedSize;
      3: Result := Round(FileDateToDateTime(ZipEntry1.DateTime) - FileDateToDateTime(ZipEntry2.DateTime));
      4: Begin
          Ratio1 := 0;
          Ratio2 := 0;
          If ZipEntry1.UncompressedSize <> 0 Then Ratio1 := Round((1 - (ZipEntry1.CompressedSize / ZipEntry1.UnCompressedSize)) * 100);
          If ZipEntry2.UncompressedSize <> 0 Then Ratio1 := Round((1 - (ZipEntry2.CompressedSize / ZipEntry2.UnCompressedSize)) * 100);
          Result := Ratio1 - Ratio2;
        End;
    End;
  End Else If (Data1.Index < 0) And (Data2.Index < 0) Then Result := AnsiCompareText(Data1.FullPath, Data2.FullPath)
  Else If (Data1.Index < 0) Then Result := -1
  Else If (Data2.Index < 0) Then Result := 1;
End;

Procedure TMainForm.VTree1GetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: Integer; TextType: TVSTTextType; Var Text: WideString);
Var
  Data1: TPNodeData;
  ZipEntry1: ZipDirEntry;
Begin
  If Node.Parent <> VTree1.RootNode Then Begin
    Data1 := TPNodeData(VTree1.GetNodeData(Node));
    If Data1.Index >= 0 Then Begin
      ZipEntry1 := ZipDirEntry(ZipMaster1.ZipContents[Data1.Index]^);
      Case Column Of
        0: Text := Data1.Path;
        1: Text := Format('%3.0n', [ZipEntry1.CompressedSize * 1.0]);
        2: Text := Format('%3.0n', [ZipEntry1.UncompressedSize * 1.0]);
        3: Text := FormatDateTime('ddddd  t', FileDateToDateTime(ZipEntry1.DateTime));
        4: Begin
            Text := '0% ';
            If ZipEntry1.UncompressedSize <> 0 Then Text := IntToStr(Round((1 - (ZipEntry1.CompressedSize / ZipEntry1.UnCompressedSize)) * 100)) + '% ';
          End;
      End;
    End Else If Column = 0 Then Text := Data1.Path
    Else Text := '';
  End Else Begin
    Case Column Of
      0: Text := ZipMaster1.ZipFileName;
      1: Text := Format('%3.0n', [FCompressedTotal * 1.0]);
      2: Text := Format('%3.0n', [FUncompressedTotal * 1.0]);
      3: Text := ''; //FormatDateTime('ddddd  t', FileDateToDateTime(ZipEntry1.DateTime));
      4: Begin
          Text := '0% ';
          If FUncompressedTotal <> 0 Then Text := IntToStr(Round((1 - (FCompressedTotal / FUncompressedTotal)) * 100)) + '% ';
        End;
    End;
  End;
End;

Procedure TMainForm.VTreeFillData;
  Procedure GetParsedPath(AInput: String; AList: TStrings);
  Const
    Delimiter = '\';
    DelimiterLen = Length(Delimiter);
  Var
    DelimiterPos1: Integer;
    TempStr: String;
  Begin
    AList.Clear;
    Repeat
      DelimiterPos1 := Pos(Delimiter, AInput);
      If DelimiterPos1 = 0 Then TempStr := AInput
      Else TempStr := Copy(AInput, 1, DelimiterPos1 - 1);
      If (TempStr <> '') Then Begin
        AList.Add(Trim(TempStr));
        AInput := Copy(AInput, DelimiterPos1 + DelimiterLen, Length(AInput) - DelimiterPos1);
      End;
    Until DelimiterPos1 = 0;
  End;

  Procedure AddEndLeaf(ANode: PVirtualNode; AList: TStrings; ANodeData: TNodeData);
  Var
    Data1: TPNodeData;
    Data2: TPNodeData;
    Node1: PVirtualNode;
    Node2: PVirtualNode;
  Begin
    Node1 := VTree1.AddChild(ANode);
    Node2 := Node1.Parent;
    Data1 := TPNodeData(VTree1.GetNodeData(Node1));
    Data2 := TPNodeData(VTree1.GetNodeData(Node2));
    Data1.Index := -1;
    Data1.Path := AList[0];
    If Data2.FullPath <> '' Then Data1.FullPath := Data2.FullPath + '\' + Data1.Path
    Else Data1.FullPath := Data1.Path;

    AList.Delete(0);
    If AList.Count >= 1 Then AddEndLeaf(Node1, AList, ANodeData)
    Else Data1.Index := ANodeData.Index;
  End;

  Procedure PlaceIntoTree(ANode: PVirtualNode; AList: TStrings; ANodeData: TNodeData);
  Var
    Data1: TNodeData;
    Node1: PVirtualNode;
  Begin
    If (AList.Count > 0) And (ANode.ChildCount > 0) Then Begin
      Node1 := ANode.FirstChild;
      While Assigned(Node1) Do Begin
        Data1 := TNodeData(VTree1.GetNodeData(Node1)^);
        If Data1.Path = AList[0] Then Begin
          AList.Delete(0);
          If AList.Count > 0 Then PlaceIntoTree(Node1, AList, ANodeData);
          Exit;
        End;
        Node1 := Node1.NextSibling;
      End;
    End;
    AddEndLeaf(ANode, AList, ANodeData); //If it has not exited, it must add as leaf node.
  End;
Var
  i: Integer;
  Data1: TNodeData;
  Data2: TPNodeData;
  PathList1: TStrings;
  Node1: PVirtualNode;
  ZipDirEntry1: ZipDirEntry;
Begin
  FUncompressedTotal := 0;
  FCompressedTotal := 0;
  VTree1.BeginUpdate;
  If ZipMaster1.Count = 0 Then Begin
    VTree1.Clear;
    VTree1.EndUpdate;
    Exit;
  End;

  VTree1.Clear;
  PathList1 := TStringList.Create;
  VTree1.NodeDataSize := SizeOf(TNodeData);
  Node1 := VTree1.AddChild(VTree1.RootNode);
  Data2 := TPNodeData(VTree1.GetNodeData(Node1));
  Data2.Index := -1;
  Data2.Path := ZipMaster1.ZipFileName;

  For i := 0 To ZipMaster1.Count - 1 Do Begin
    Data1.Index := i;
    Data1.Path := '';
    ZipDirEntry1 := ZipDirEntry(ZipMaster1.ZipContents[i]^);
    GetParsedPath(ZipDirEntry1.FileName, PathList1);
    PlaceIntoTree(Node1, PathList1, Data1);
    FUncompressedTotal := FUncompressedTotal + Cardinal(ZipDirEntry1.UncompressedSize);
    Inc(FCompressedTotal, ZipDirEntry1.CompressedSize);
  End;
  PathList1.Free;
  VTree1.SortTree(0, sdAscending, True);
  VTree1.EndUpdate;
End;

Procedure TMainForm.ActionAddToZipExecute(Sender: TObject);
Var
  s, f: LongInt;
  IsOne: String;
Begin
  If ZipMaster1.ZipFileName = '' Then Begin
    ShowMessage('Error - open a zip file first');
    Exit;
  End;
  AddForm.Left := Left;
  AddForm.Top := Top;
  AddForm.Width := Width;
  AddForm.Height := Height;
  If AddForm.ShowModal <> mrOK Then Exit; { let user pick filenames to add }
  If AddForm.SelectedList.Items.Count = 0 Then Begin
    ShowMessage('No files selected');
    Exit;
  End;
  MsgForm.RichEdit1.Clear;
  MsgForm.Show;
  { Put this message into the message form's memo }
  ZipMaster1Message(self, 0, 'Beginning Add to ' + ZipMaster1.ZipFileName);

  With ZipMaster1 Do Begin { We want any DLL error messages to show over the top of the message form. }
    AddOptions := [];
    Case AddForm.ZipAction Of // Default is plain ADD.
      2: AddOptions := AddOptions + [AddUpdate]; // Update
      3: AddOptions := AddOptions + [AddFreshen]; // Freshen
      4: AddOptions := AddOptions + [AddMove]; // Move
    End;
    If AddForm.RecurseCB.Checked Then AddOptions := AddOptions + [AddRecurseDirs]; { we want recursion }
    If AddForm.DirnameCB.Checked Then AddOptions := AddOptions + [AddDirNames]; { we want dirnames }
    If AddForm.DiskSpanCB.Checked Then AddOptions := AddOptions + [AddDiskSpan]; { we want diskspanning }
    If AddForm.EncryptCB.Checked Then Begin
      AddOptions := AddOptions + [AddEncrypt]; { we want a password }
      //GetAddPassword;
      //If Password = '' Then GetAddPassword; { The 2 password's entered by user didn't match. We'll give him one more try; if he still messes it up, the DLL itself will prompt him one final time. }
    End;
    FSpecArgs.Clear;
    FSpecArgs.Assign(AddForm.SelectedList.Items); { specify filenames }
    AddForm.SelectedList.Clear;
    s := GetTickCount;
    Try
      Add;
    Except
      ShowMessage('Error in Add; Fatal DLL Exception in mainunit');
    End;
    f := GetTickCount;
    StatusBar1.Panels[1].Text := 'Time: ' + ShowLTime(s, f);
    If SuccessCnt = 1 Then IsOne := ' was'
    Else IsOne := 's were';
    ShowMessage(IntToStr(SuccessCnt) + ' file' + IsOne + ' added');
  End;
End;

Procedure TMainForm.ActionExtractExecute(Sender: TObject);
Var
  s, f: Longint;
  IsOne: String;
  Data1: Pointer;
  Node1: PVirtualNode;
Begin
  If Not FileExists(ZipMaster1.ZipFilename) Then Begin
    ShowMessage('Error: file not found: ' + ZipMaster1.ZipFilename);
    Exit;
  End;
  If Extract.ShowModal <> MrOK Then Exit;
  If (ExtractDir = '') Then Exit;

  If ZipMaster1.Count < 1 Then Begin
    ShowMessage('Error - no files to extract');
    Exit;
  End;

  ZipMaster1.FSpecArgs.Clear;
  If Not AllFiles Then Begin { Get fspecs of selected files, unless user wants all files extracted }
    Node1 := VTree1.GetFirstSelected;
    While Assigned(Node1) Do Begin
      Data1 := VTree1.GetNodeData(Node1);
      ZipMaster1.FSpecArgs.Add(pZipDirEntry(Data1)^.FileName);
      Node1 := VTree1.GetNextSelected(Node1);
    End;
    If ZipMaster1.FSpecArgs.Count < 1 Then Begin
      ShowMessage('Error - no files selected');
      Exit;
    End;
  End;

  MsgForm.RichEdit1.Clear;
  MsgForm.Show;
  { Put this message into the message form's memo }
  ZipMaster1Message(self, 0, 'Beginning Extract from ' + ZipMaster1.ZipFilename);

  With ZipMaster1 Do Begin
    ExtrBaseDir := ExtractDir;
    ExtrOptions := [];
    If ExpandDirs Then ExtrOptions := ExtrOptions + [ExtrDirNames];
    If Overwr Then ExtrOptions := ExtrOptions + [ExtrOverwrite];
    s := GetTickCount;
    Try
      Extract;
    Except
      ShowMessage('Error in Extract; Fatal DLL Exception in mainunit');
    End;
    f := GetTickCount;
    StatusBar1.Panels[1].Text := 'Time: ' + ShowLTime(s, f);
    If SuccessCnt = 1 Then IsOne := ' was'
    Else IsOne := 's were';
    ShowMessage(IntToStr(SuccessCnt) + ' file' + IsOne + ' extracted');
  End;
End;

Procedure TMainForm.ActionRemoveExecute(Sender: TObject);
Var
  Answer1: Boolean;
  s, f: LongInt;
  IsOne: String;
  Data1: Pointer;
  Node1: PVirtualNode;
Begin
  If ZipMaster1.Count < 1 Then Begin
    ShowMessage('Error - no files to delete');
    Exit;
  End;
  Answer1 := MessageDlg('Delete selected files from: ' + ZipMaster1.ZipFileName + '?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
  If Not Answer1 Then Exit;

  ZipMaster1.FSpecArgs.Clear;
  Node1 := VTree1.GetFirstSelected;
  While Assigned(Node1) Do Begin
    Data1 := VTree1.GetNodeData(Node1);
    ZipMaster1.FSpecArgs.Add(pZipDirEntry(Data1)^.FileName);
    Node1 := VTree1.GetNextSelected(Node1);
  End;

  If ZipMaster1.FSpecArgs.Count < 1 Then Begin
    ShowMessage('Error - no files selected');
    Exit;
  End;

  MsgForm.RichEdit1.Clear;
  MsgForm.Show;
  { Put this message into the message form's memo }
  ZipMaster1Message(self, 0, 'Beginning delete from ' + ZipMaster1.ZipFileName);

  s := GetTickCount;
  Try
    ZipMaster1.Delete;
  Except
    ShowMessage('Fatal error trying to delete');
  End;
  f := GetTickCount;
  StatusBar1.Panels[1].Text := 'Time: ' + ShowLTime(s, f);
  If ZipMaster1.SuccessCnt = 1 Then IsOne := ' was'
  Else IsOne := 's were';
  ShowMessage(IntToStr(ZipMaster1.SuccessCnt) + ' file' + IsOne + ' deleted');
End;

Procedure TMainForm.ActionRenameExecute(Sender: TObject);
Begin
  RenForm.Show();
End;

Procedure TMainForm.ActionFileOpenExecute(Sender: TObject);
Const
  InitialDir1: String = '';
Begin
  OpenDialog1.InitialDir := ExtractFilePath(Application.Exename); //Remove this line if you want default behavior

  //Uncomment these two lines if you want default behavior
  //If InitialDir1 = '' Then GetSpecialFolder(CSIDL_DESKTOPDIRECTORY, InitialDir1);
  //OpenDialog1.InitialDir := InitialDir1;
  OpenDialog1.Title := 'Open Existing ZIP File';
  OpenDialog1.Filter := 'ZIP Files (*.ZIP, *.EXE)|*.zip;*.exe';
  OpenDialog1.FileName := '';
  OpenDialog1.Options := OpenDialog1.Options + [ofHideReadOnly, ofShareAware, ofPathMustExist, ofFileMustExist];
  If OpenDialog1.Execute Then Begin
    InitialDir1 := ExtractFilePath(OpenDialog1.FileName); { Set the caption after assigning the filename. This way, the filename will be null if the open failed. }
    SetZipFName(OpenDialog1.FileName, True);
  End;
End;

Procedure TMainForm.ActionFileNewExecute(Sender: TObject);
Const
  InitialDir1: String = '';
Var
  Answer1: Boolean;
Begin
  If InitialDir1 = '' Then GetSpecialFolder(CSIDL_DESKTOPDIRECTORY, InitialDir1);
  With OpenDialog1 Do Begin
    InitialDir := InitialDir1;
    FileName := '';
    Filter := 'ZIP Files (*.ZIP)|*.zip';
    DefaultExt := 'Zip';
    Title := 'Create New ZIP File';
    Options := Options + [ofHideReadOnly, ofShareAware];
    Options := Options - [ofPathMustExist, ofFileMustExist];
    If Execute Then Begin
      InitialDir1 := ExtractFilePath(FileName);
      If FileExists(FileName) Then Begin
        Answer1 := MessageDlg('Overwrite Existing File: ' + FileName + '?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
        If Answer1 Then DeleteFile(FileName)
        Else Exit; { Don't use the new name }
      End;
      SetZipFName(FileName, True);
    End;
  End;
End;

Procedure TMainForm.ActionTestZipExecute(Sender: TObject);
Var
  s, f: LongInt;
Begin
  If ZipMaster1.Count < 1 Then Begin
    ShowMessage('Error - nothing to Test');
    Exit;
  End;
  If Zipmaster1.Zipfilename = '' Then Exit;
  MsgForm.RichEdit1.Clear;
  MsgForm.Show;
  ZipMaster1Message(self, 0, 'Beginning test of ' + ZipMaster1.ZipFileName);
  With ZipMaster1 Do Begin
    FSpecArgs.Clear;
    ExtrOptions := ExtrOptions + [ExtrTest];
    FSpecArgs.Add('*.*'); // Test all the files in the .zip
    // IMPORTANT: In this release, you must test all files.
    s := GetTickCount;
    Extract; // This will really do a test
  End;
  f := GetTickCount;
  StatusBar1.Panels[1].Text := 'Time: ' + ShowLTime(s, f);

  With ZipMaster1 Do Begin
    If SuccessCnt = DirOnlyCount + Count Then ShowMessage('All ' + IntToStr(DirOnlyCount + Count) + ' files tested OK')
    Else ShowMessage('ERROR: ' + IntToStr(DirOnlyCount + Count - SuccessCnt) + ' files tested as bad, or skipped!');
  End;
End;

Procedure TMainForm.ActionConvertToEXEExecute(Sender: TObject);
Var
  ConvertErr: Integer;
Begin
  If ZipMaster1.Count = 0 Then Begin
    ShowMessage('Error: no files in archive');
    Exit;
  End;
  { determine which conversion is to be done }
  If UpperCase(ExtractFileExt(ZipMaster1.ZipFilename)) = '.EXE' Then Begin
    { Convert .EXE to .ZIP }
    ConvertErr := ZipMaster1.ConvertZIP;
    If ConvertErr = 0 Then ShowMessage('Filename is now: ' + ZipMaster1.ZipFilename)
    Else ShowMessage('Error ' + IntToStr(ConvertErr) + ' occured in making .ZIP file');
  End Else Begin
    { Convert .ZIP to .EXE }
    { NOTE: If you put the ZIPSFX.BIN file into the WINDOWS
      or WINDOWS SYSTEM dir, then you don't need to set the
      SFXPath property below: }
    { ZipMaster1.SFXPath := 'c:\windows\system\zipsfx.bin'; }
    If Not MakeSFX.ShowModal <> mrOK Then Exit;
    ConvertErr := ZipMaster1.ConvertSFX;
    If ConvertErr = 0 Then ShowMessage('Filename is now: ' + ZipMaster1.ZipFilename)
    Else ShowMessage('Error ' + IntToStr(ConvertErr) + ' occured in making .EXE file');
  End;
End;

Procedure TMainForm.ActionShowMessagesExecute(Sender: TObject);
Begin
  MsgForm.Show;
End;

Procedure TMainForm.ActionViewLastErrorExecute(Sender: TObject);
Begin
  If ZipMaster1.ErrCode <> 0 Then ShowMessage(IntToStr(ZipMaster1.ErrCode) + ' ' + ZipMaster1.Message)
  Else ShowMessage('No last error present');
End;

Procedure TMainForm.ActionViewZipCommentExecute(Sender: TObject);
Begin
  If ZipMaster1.ZipComment <> '' Then Begin
    MsgForm.RichEdit1.Clear;
    MsgForm.RichEdit1.Lines.Add(ZipMAster1.ZipComment);
    MsgForm.Show;
  End Else ShowMessage('No Zip comment in this zip file');
End;

Procedure TMainForm.ActionViewDLLVersionInfoExecute(Sender: TObject);
Begin
  ShowMessage('UnZip Dll version: ' + IntToStr(ZipMaster1.UnzVers) + #10 + '  Zip Dll version: ' + IntToStr(ZipMaster1.ZipVers));
End;

Procedure TMainForm.ActionOptionsVerboseExecute(Sender: TObject);
Begin
  ActionOptionsVerbose.Checked := Not ActionOptionsVerbose.Checked;
  ZipMaster1.Verbose := ActionOptionsVerbose.Checked;
End;

Procedure TMainForm.ActionOptionsTraceExecute(Sender: TObject);
Begin
  ActionOptionsTrace.Checked := Not ActionOptionsTrace.Checked;
  ZipMaster1.Trace := ActionOptionsTrace.Checked;
End;

Procedure TMainForm.ActionOptionsUnattendedExecute(Sender: TObject);
Begin
  ActionOptionsUnattended.Checked := Not ActionOptionsUnattended.Checked;
  ZipMaster1.Unattended := ActionOptionsUnattended.Checked;
End;

Procedure TMainForm.ActionFileExitExecute(Sender: TObject);
Begin
  Close;
End;

Procedure TMainForm.VTree1FocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: Integer);
Begin
  ActionAddToZip.Visible := False;
  ActionExtract.Visible := False;
  ActionRemove.Visible := False;
  ActionRename.Visible := False;
End;

Procedure TMainForm.VTree1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Begin
  ActionAddToZip.Visible := (VTree1.GetFirstSelected <> Nil);
  ActionExtract.Visible := (VTree1.GetFirstSelected <> Nil);
  ActionRemove.Visible := (VTree1.GetFirstSelected <> Nil);
  ActionRename.Visible := (VTree1.GetFirstSelected <> Nil);
End;

Procedure TMainForm.VTree1GetImageIndex(Sender: TBaseVirtualTree; Node: PVirtualNode; Kind: TVTImageKind; Column: Integer; Var Ghosted: Boolean; Var Index: Integer);
Var
  Data1: TPNodeData;
  ZipEntry1: ZipDirEntry;
Begin
  If Column <> 0 Then Exit;
  Data1 := TPNodeData(VTree1.GetNodeData(Node));
  If Data1.Index >= 0 Then Begin
    ZipEntry1 := ZipDirEntry(ZipMaster1.ZipContents[Data1.Index]^);
    Index := SystemImageList1.GetVirtualFileSystemIconIndex(ZipEntry1.FileName);
  End Else If (vsSelected In Node.States) Then Index := SystemImageList1.GetOpenFolderIndex
  Else Index := SystemImageList1.GetClosedFolderIndex;
End;

procedure TMainForm.VTree1HeaderClick(Sender: TVTHeader; Column: Integer;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  VTree1.Header.SortColumn := Column;
  If VTree1.Header.SortDirection = sdAscending Then VTree1.Header.SortDirection := sdDescending
  Else VTree1.Header.SortDirection := sdAscending;
  VTree1.SortTree(VTree1.Header.SortColumn, VTree1.Header.SortDirection, True);
end;

End.

