unit frmMainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, VirtualTrees;

type
  PData  = ^TData;
  TData  = record
             NodeLevel : string[20];
             Data1, Data2 : widestring;
           end;

  TForm1 = class(TForm)
    VST: TVirtualStringTree;
    Edit1: TEdit;
    Label1: TLabel;
    btnAdd: TBitBtn;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure VSTFocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: Integer);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
  private
    { Private-Deklarationen }
    FindNodeLevel: string;

    function CaptureNodeLevel (Tree: TBaseVirtualTree; Node: PVirtualNode) : string;
  public
    { Public-Deklarationen }

    procedure SaveNodes (Sender: TBaseVirtualTree; Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
    procedure FindNode (Sender: TBaseVirtualTree; Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
  end;

var
  Form1: TForm1;
  DataList : array of TData;

implementation

{$R *.DFM}

function TForm1.CaptureNodeLevel (Tree: TBaseVirtualTree; Node: PVirtualNode) : string;
var
  ParentNode : PVirtualNode;

begin
  if Node <> nil then
  begin
    Result := IntToStr (Node^.Index);

    ParentNode := Node^.Parent;
    while ParentNode <> Tree.RootNode do
    begin
      Node := ParentNode;
      Result := IntToStr (Node^.Index) + '.' + Result;

      ParentNode := Node^.Parent;
    end;
  end
  else
    Result := '';
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  VST.NodeDataSize := SizeOf (TData);
end;

procedure TForm1.btnAddClick(Sender: TObject);
var
  NewNode : PVirtualNode;

begin
  NewNode := VST.AddChild (VST.FocusedNode);

  with PData(VST.GetNodeData(NewNode))^ do
  begin
    NodeLevel := CaptureNodeLevel (VST, NewNode);
    Data1 := '';
    Data2 := '';
  end;

  VST.FocusedNode := NewNode;
  VST.Selected[NewNode] := true;

  VST.InvalidateNode (NewNode);
end;

procedure TForm1.VSTFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: Integer);
begin
  Edit1.Text := PData(Sender.GetNodeData(Node))^.NodeLevel;
end;

procedure TForm1.SaveNodes (Sender: TBaseVirtualTree; Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
begin
  SetLength (DataList, Length(DataList)+1);
  DataList[Length(DataList)-1] := PData(Sender.GetNodeData(Node))^;
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
begin
  SetLength (DataList, 0);
  VST.IterateSubtree (nil, SaveNodes, nil);

  MessageDlg ('Daten gespeichert ('+IntToStr(Length(DataList))+' Eintrge)', mtInformation, [mbOk], 0);
end;

procedure TForm1.BitBtn2Click(Sender: TObject);
begin
  VST.Clear;
end;

procedure TForm1.FindNode (Sender: TBaseVirtualTree; Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
begin
  Abort := (CaptureNodeLevel (Sender, Node) = FindNodeLevel);
end;

procedure TForm1.BitBtn3Click(Sender: TObject);
var
  i : integer;
  RemainingSNL,
  SearchNodeLevel: string;
  SearchNode,
  NewNode : PVirtualNode;
  NodeData : PData;

 function FindNodeAtLevel (NodeLevel: string) : PVirtualNode;
 begin
   //NodeLevel := Copy (Nod
   FindNodeLevel := NodeLevel;
   Result := VST.IterateSubtree (nil, FindNode, nil, [], true, true);
 end;

 function FindChildNode (Node: PVirtualNode; NodeLevel: string) : PVirtualNode;
 begin
   FindNodeLevel := NodeLevel;
   Result := VST.IterateSubtree (Node, FindNode, nil, [], true, true);
 end;

 function RPos (SearchChar: char; SearchStr: string): integer;
 var
   i : integer;

 begin
   Result := Length(SearchStr)+1;
   for i := Length(SearchStr) downto 1 do
     if SearchStr[i] = SearchChar then
     begin
       Result := i;
       Break;
     end;
 end;

begin
  SearchNode := nil;
  SearchNodeLevel := '';

  for i := 0 to (Length(DataList)-1) do
  begin
    while Copy (DataList[i].NodeLevel, 1, Length(SearchNodeLevel)) <> SearchNodeLevel do
    begin
      SearchNode := SearchNode^.Parent;
      SearchNodeLevel := CaptureNodeLevel (VST, SearchNode);
    end;
    RemainingSNL := Copy (DataList[i].NodeLevel, Length(SearchNodeLevel)+2, MaxInt);
    if Trim(RemainingSNL) <> '' then RemainingSNL := RemainingSNL + '.';
    while (Pos('.',RemainingSNL) > 0) and
          (VST.ChildCount[SearchNode] >= StrToInt (Copy (RemainingSNL, 1, Pos ('.', RemainingSNL) -1))) do
    begin
      // Child suchen
      SearchNode := FindChildNode (SearchNode, Copy(DataList[i].NodeLevel, 1, RPos('.', DataList[i].NodeLevel)-1));
      RemainingSNL := Copy (RemainingSNL, Pos('.',RemainingSNL)+1, MaxInt);
    end;

    SearchNodeLevel := CaptureNodeLevel (VST, SearchNode);

    NewNode := VST.AddChild (SearchNode);
    NodeData := VST.GetNodeData(NewNode);
    NodeData^ := DataList[i];
  end;
end;

end.
